/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <limits.h>
#include "merc.h"
#include "tables.h"
#include "recycle.h"


static BANK_ACCOUNT *bank_accounts;
static bool dirty;


void load_accounts()
{
    BANK_ACCOUNT *account;
    char letter;
    FILE *fp;

    if ( (fp = fopen(ACCOUNT_FILE, "r")) == NULL )
    {
	bug( "load_accounts: fopen", 0 );
	perror( ACCOUNT_FILE );
	return;
    }
    
    while ( 1 ) {
    	letter = fread_letter( fp );
    	
    	if ( letter == '#' ) {
    	    break;
    	}
    	
    	ungetc( letter, fp );
    	
    	account 		= new_account();
    	
    	free_string( account->name );
    	account->balance	= fread_number( fp );
    	account->name		= fread_string( fp );

    	account->next		= bank_accounts;
    	bank_accounts		= account;
    }

    fclose( fp );
}


void save_accounts()
{
    BANK_ACCOUNT *account;
    FILE *fp;

    if ( !dirty )
	return;

    fclose( fpReserve );
    if ( (fp = fopen(ACCOUNT_FILE, "w")) == NULL )
    {
	bug( "save_accounts: fopen", 0 );
	perror( ACCOUNT_FILE );
    }
    
    for ( account = bank_accounts; account != NULL; account = account->next )
	fprintf( fp, "%ld %s~\n", account->balance, account->name );
    fprintf( fp, "#\n" );
    
    fclose( fp );
    fpReserve = fopen( NULL_FILE, "r" );
    
    dirty = FALSE;
}


BANK_ACCOUNT *find_account( const char *account_name )
{
    BANK_ACCOUNT *account;

    for ( account = bank_accounts; account != NULL; account = account->next )
    {
	if ( !str_cmp(account_name, account->name) )
	    return account;
    }
    
    return NULL;
}


void do_account( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    BANK_ACCOUNT *account;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: account <name> create\n\r"
	              "        account <name> remove\n\r"
	              "        account <name> set <new balance>\n\r"
	              "        account list\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    
    if ( !str_prefix(arg1, "list") )
    {
	for ( account = bank_accounts; account != NULL; account = account->next )
	{
	    sprintf( buf, "%-30s %ld\n\r", account->name, account->balance );
	    send_to_char( buf, ch );
	}
	send_to_char( "\n\r", ch );
	return;
    }
    
    if ( arg2[0] == '\0' )
    {
	do_account( ch, "" );
	return;
    }
    
    if ( !str_prefix(arg2, "create") )
    {
	if ( find_account(arg1) )
	{
	    send_to_char( "An account with that name already exists.\n\r", ch );
	    return;
	}
	
	account = new_account();
	
	free_string( account->name );
	account->name = str_dup( arg1 );
	
	account->next = bank_accounts;
	bank_accounts = account;
	
	send_to_char( "OK.\n\r", ch );
	dirty = TRUE;
	return;
    }

    if ( !str_prefix(arg2, "remove") )
    {
	if ( bank_accounts == NULL )
	{
	    send_to_char( "There are no bank accounts to remove.\n\r", ch );
	    return;
	}
	
	if ( !str_prefix(arg1, bank_accounts->name) )
	{
	    BANK_ACCOUNT *head = bank_accounts;
	    
	    bank_accounts = bank_accounts->next;
	    free_account( head );
	}
	else
	{
	    for ( account = bank_accounts; account != NULL; account = account->next )
	    {
		if ( account->next == NULL )
		{
		    send_to_char( "Account not found.\n\r", ch );
		    return;
		}
		
		if ( !str_prefix(arg1, account->next->name) )
		{
		    BANK_ACCOUNT *removed = account->next;
		    
		    account->next = account->next->next;
		    
		    free_account( removed );
		    break;
		}
	    }
	}
	
	send_to_char( "OK.\n\r", ch );
	dirty = TRUE;
	return;
    }
    else if ( !str_prefix(arg2, "set") )
    {
	for ( account = bank_accounts; account != NULL; account = account->next )
	{
	    if ( !str_prefix(arg1, account->name) )
	    {
		int balance;
		
		if ( !is_number(argument) )
		{
		    do_account( ch, "" );
		    return;
		}
		
		balance = atoi( argument );
		if ( balance < 0 )
		{
		    send_to_char( "New balance must be non-negative.\n\r", ch );
		    return;
		}
		
		account->balance = atoi( argument );
		
		send_to_char( "OK.\n\r", ch );
		dirty = TRUE;
		return;
	    }
	}
	
	if ( account == NULL )
	{
	    send_to_char( "Account not found.\n\r", ch );
	    return;
	}
    }
    
    do_account( ch, "" );
    return;
}


bool account_deposit( const char *account_name, long amount )
{
    BANK_ACCOUNT *account;
    long new_balance;
    
    account = find_account( account_name );
    if ( account == NULL )
	return FALSE;

    new_balance = account->balance + amount;
    if ( new_balance < 0 )
	new_balance = LONG_MAX;
    account->balance = new_balance; 
    
    dirty = TRUE;
    return TRUE;
}


bool account_withdraw( const char *account_name, long amount )
{
    BANK_ACCOUNT *account;
    
    account = find_account( account_name );
    if ( account == NULL )
	return FALSE;

    account->balance -= amount;
    if ( account->balance < 0 )
    {
	bug( "account_withdraw: negative balance in account %s.", (long) account_name );
	account->balance = 0;
    }
    
    dirty = TRUE;
    return TRUE;
}


long account_balance( const char *account_name )
{
    BANK_ACCOUNT *account;
    
    account = find_account( account_name );
    if ( account == NULL )
	return -1;

    return account->balance;
}


void do_adeposit( CHAR_DATA *ch, char *argument )
{
    char account_arg[MAX_INPUT_LENGTH];
    char amount_arg[MAX_INPUT_LENGTH];
    char currency_arg[MAX_INPUT_LENGTH];
    CHAR_DATA *banker;
    long amount;

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players may hold bank balances.", ch );
	return;
    }

    for ( banker = ch->in_room->people; 
	  banker != NULL; 
	  banker = banker->next_in_room )
    {
	if ( IS_NPC(banker) && IS_SET(banker->act,ACT_IS_BANKER) )
	    break;
    }

    if ( banker == NULL || !can_see(ch, banker) )
    {
	send_to_char( "You can't do that here.\n\r", ch );
	return;
    }

    argument = one_argument( argument, account_arg );
    if ( account_arg[0] == '\0' )
    {
	send_to_char( "Deposit into which account?\n\r", ch );
	return;
    }

    argument = one_argument( argument, amount_arg );
    if ( amount_arg[0] == '\0' || !is_number(amount_arg) )
    {
	act( "$n asks you 'Deposit how much?'",
	    banker, NULL, ch, TO_VICT );
	update_reply( ch, banker );
	return;
    }
    amount = atoi( amount_arg );

    one_argument( argument, currency_arg );
    if ( (amount < 500 && (currency_arg[0] == '\0' || !str_prefix(currency_arg, "silver")))
    ||   amount < 5 )
    {
	act( "$n tells you 'We don't deal in such small amounts.'",
	    banker, NULL, ch, TO_VICT );
	update_reply( ch, banker );
	return;
    }

    if ( currency_arg[0] == '\0' || !str_prefix(currency_arg, "silver") )
    {
	if ( ch->silver < amount )
	{
	    act( "$n tells you 'You don't seem to be carrying that much silver.'",
		banker, NULL, ch, TO_VICT );
	    update_reply( ch, banker );
	    return;
	}
	
	if ( account_deposit(account_arg, amount * 9/10) )
	    ch->silver -= amount;
	else
	{
	    act( "$n tells you, 'That organization does not possess a bank account here.'",
	        banker, NULL, ch, TO_VICT );
	    return;
	}
    }
    else if ( !str_prefix(currency_arg, "gold") )
    {
	if ( ch->gold < amount )
	{
	    act( "$n tells you 'You don't seem to be carrying that much gold.'",
		banker, NULL, ch, TO_VICT );
	    update_reply( ch, banker );
	    return;
	}

	if ( account_deposit(account_arg, amount * 90) )
	    ch->gold -= amount;
	else
	{
	    act( "$n tells you, 'That organization does not possess a bank account here.'",
	        banker, NULL, ch, TO_VICT );
	    return;
	}
    }
    else
    {
	act( "$n tells you 'You can only deposit gold or silver.'",
	    banker, NULL, ch, TO_VICT );
	update_reply( ch, banker );
	return;
    }

    act( "$N takes your money and says, 'Thank you for your business.'",
	ch, NULL, banker, TO_CHAR );
    act( "$N takes $n's money and says, 'Thank you for your business.'",
	ch, NULL, banker, TO_ROOM );
}


void do_abalance( CHAR_DATA *ch, char *argument )
{
    char account_arg[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *banker;

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players may hold bank balances.", ch );
	return;
    }

    for ( banker = ch->in_room->people; 
	  banker != NULL; 
	  banker = banker->next_in_room )
    {
	if ( IS_NPC(banker) && IS_SET(banker->act,ACT_IS_BANKER) )
	    break;
    }

    if ( banker == NULL || !can_see(ch, banker) )
    {
	send_to_char( "You can't do that here.\n\r", ch );
	return;
    }

    argument = one_argument( argument, account_arg );
    if ( account_arg[0] == '\0' )
    {
	send_to_char( "Query which account?\n\r", ch );
	return;
    }
    if ( find_account(account_arg) == NULL )
    {
	act( "$n tells you, 'That organization does not possess a bank account here.'",
	    banker, NULL, ch, TO_VICT );
	return;
    }

    sprintf( buf, 
	"$n tells you, 'Your balance is %ld pieces of silver.'",
	account_balance(account_arg) );
    act( buf, banker, NULL, ch, TO_VICT );
    update_reply( ch, banker );
}


void do_awithdraw( CHAR_DATA *ch, char *argument, char *to )
{
    char account_arg[MAX_INPUT_LENGTH];
    char amount_arg[MAX_INPUT_LENGTH];
    char currency_arg[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *banker;
    long amount;

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players may hold bank balances.", ch );
	return;
    }

    for ( banker = ch->in_room->people; 
	  banker != NULL; 
	  banker = banker->next_in_room )
    {
	if ( IS_NPC(banker) && IS_SET(banker->act,ACT_IS_BANKER) )
	    break;
    }

    if ( banker == NULL || !can_see(ch, banker) )
    {
	send_to_char( "You can't do that here.\n\r", ch );
	return;
    }

    argument = one_argument( argument, account_arg );
    if ( account_arg[0] == '\0' )
    {
	send_to_char( "Withdraw from which account?\n\r", ch );
	return;
    }
    if ( !find_account(account_arg) )
    {
	act( "$n tells you, 'That organization does not possess a bank account here.'",
	    banker, NULL, ch, TO_VICT );
	return;
    }

    argument = one_argument( argument, amount_arg );
    if ( amount_arg[0] == '\0' || !is_number(amount_arg) )
    {
	act( "$n asks you 'Withdraw how much?'",
	    banker, NULL, ch, TO_VICT );
	update_reply( ch, banker );
	return;
    }

    amount = atoi( amount_arg );
    if ( amount <= 0 )
    {
	act( "$n tells you 'You can't withdraw that much.'",
	    banker, NULL, ch, TO_VICT );
	update_reply( ch, banker );
	return;
    }

    one_argument( argument, currency_arg );
    if ( currency_arg[0] == '\0' || !str_prefix(currency_arg, "silver") )
    {
	if ( account_balance(account_arg) < amount )
	{
	    act( "$n tells you 'There isn't that much in the account.'",
		banker, NULL, ch, TO_VICT );
	    update_reply( ch, banker );
	    return;
	}

	account_withdraw( account_arg, amount );
    	ch->silver += amount;

        sprintf( buf, "%ld silver", amount );
    }
    else if ( !str_prefix(currency_arg, "gold") )
    {
	if ( account_balance(account_arg) / 100 < amount )
	{
	    act( "$n tells you 'There isn't that much in the account.'",
		banker, NULL, ch, TO_VICT );
	    update_reply( ch, banker );
	    return;
	}

	account_withdraw( account_arg, amount * 100 );
	ch->gold += amount;

        sprintf( buf, "%ld gold", amount );
    }
    else
    {
	act( "$n tells you 'You can only withdraw gold or silver.'",
	    banker, NULL, ch, TO_VICT );
	update_reply( ch, banker );
	return;
    }

    act( "$N passes you your money and says, 'Thank you for your business.'",
	ch, NULL, banker, TO_CHAR );
    act( "$N passes $n some money and says, 'Thank you for your business.'",
	ch, NULL, banker, TO_ROOM );

    /* Send a note if to is non-null */
    if ( !IS_NULLSTR(to) )
    {
        char buf2[MAX_STRING_LENGTH];
        NOTE_DATA *pnote;
        char *strtime;

	strtime				= ctime( &current_time );
	strtime[strlen(strtime)-1]	= '\0';

	sprintf( buf2, 
	    "%s has withdrawn %s from the %s bank account.",
            ch->name,
            buf,
            account_arg );

	pnote			= new_note();
	pnote->next		= NULL;
	pnote->type		= NOTE_NOTE;
	pnote->sender		= str_dup( "Andaras" );
	pnote->to_list		= str_dup( to );
	pnote->date		= strtime;
	pnote->date_stamp	= current_time;
	pnote->subject		= str_dup( "Bank statement." );
	pnote->text		= str_dup( buf2 );
    
	append_note(pnote);
    }
}


void do_cdeposit( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can deposit into a clan account.\n\r", ch );
	return;
    }

    if ( !is_clan(ch) )
    {
	send_to_char( "You need to be in a clan to do that.\n\r", ch );
	return;
    }
    
    if ( clan_table[ch->clan].independent )
    {
	send_to_char( "The 'LONER clan' cannot possess bank accounts.\n\r", ch );
	return;
    }

    sprintf( buf, "%s %s", clan_table[ch->clan].name, argument );
    do_adeposit( ch, buf );
}


void do_cbalance( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can query a clan account.\n\r", ch );
	return;
    }

    if ( !is_clan(ch) )
    {
	send_to_char( "You need to be in a clan to do that.\n\r", ch );
	return;
    }
    
    if ( clan_table[ch->clan].independent )
    {
	send_to_char( "The 'LONER clan' cannot possess bank accounts.\n\r", ch );
	return;
    }

    if ( ch->pcdata->leader == 0 )
    {
	send_to_char( "Only the clan leader or deputies may do that.\n\r", ch );
	return;
    }

    sprintf( buf, "%s %s", clan_table[ch->clan].name, argument );
    do_abalance( ch, buf );
}



void do_cwithdraw( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can withdraw from a clan account.\n\r", ch );
	return;
    }

    if ( !is_clan(ch) )
    {
	send_to_char( "You need to be in a clan to do that.\n\r", ch );
	return;
    }
    
    if ( clan_table[ch->clan].independent )
    {
	send_to_char( "The 'LONER clan' cannot possess bank accounts.\n\r", ch );
	return;
    }

    if ( ch->pcdata->leader < 5 )
    {
	send_to_char( "Only the clan leader may do that.\n\r", ch );
	return;
    }

    sprintf( buf, "%s %s", clan_table[ch->clan].name, argument );
    do_awithdraw( ch, buf, "" );
}


void do_tdeposit( CHAR_DATA *ch, char *argument )
{
    char territory_arg[MAX_INPUT_LENGTH];
    int territory;

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can deposit into a territory account.\n\r", ch );
	return;
    }

    one_argument( argument, territory_arg );
    if ( territory_arg[0] == '\0' )
    {
	send_to_char( "Deposit into which territories bank account?\n\r", ch );
	return;
    }

    territory = territory_lookup( territory_arg );
    if ( territory == -1 )
    {
	send_to_char( "There is no such territory.\n\r", ch );
	return;
    }
    
    if ( ch->pcdata->social_classes[territory] != MAX_SOCIAL_CLASS - 1
    &&   !has_territory_flag(ch, territory, TERR_TREASURER) )
    {
	send_to_char( "You are not the ruler or tresurer of that territory.\n\r", ch );
	return;
    }

    do_adeposit( ch, argument );
}


void do_tbalance( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    int territory;

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can query a territory account.\n\r", ch );
	return;
    }

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	send_to_char( "Query which territories bank account?\n\r", ch );
	return;
    }

    territory = territory_lookup( arg );
    if ( territory == -1 )
    {
	send_to_char( "There is no such territory.\n\r", ch );
	return;
    }
    
    if ( ch->pcdata->social_classes[territory] != MAX_SOCIAL_CLASS - 1
    &&   !has_territory_flag(ch, territory, TERR_TREASURER) )
    {
	send_to_char( "You are not the ruler or tresurer of that territory.\n\r", ch );
	return;
    }

    do_abalance( ch, argument );
}


void do_twithdraw( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    int territory;

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can withdraw from a territory account.\n\r", ch );
	return;
    }

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	send_to_char( "Withdraw from which territories bank account?\n\r", ch );
	return;
    }

    territory = territory_lookup( arg );
    if ( territory == -1 )
    {
	send_to_char( "There is no such territory.\n\r", ch );
	return;
    }
    
    if ( ch->pcdata->social_classes[territory] != MAX_SOCIAL_CLASS - 1
    &&   !has_territory_flag(ch, territory, TERR_TREASURER) )
    {
	send_to_char( "You are not the ruler or treasurer of that territory.\n\r", ch );
	return;
    }

    sprintf( buf, "ruler:%s", territories[territory].name );
    do_awithdraw( ch, argument, buf );
}

