/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "merc.h"
#include "recycle.h"


#define LIST_DAYS		3
#define ARRAY_ITEMS(array)	(sizeof(array) / sizeof(char *))

static const char *plague_messages[] =
{
    "$n dies from the plague. Outbreak feared!",
    "Plague kills $n. Citizens worried!",
    "Another outbreak of the plague feared after $n dies from sudden illness!"
};

static MUD_EVENT *mud_events;
static int num_mud_events;


DECLARE_DO_FUN( do_yell	);

static const char *damage_message( int dam_type, bool killer_first )
{
    if ( dam_type == DAM_FIRE )
    {
	switch ( number_range(1, 4) )
	{
	case 1: return killer_first ? "toasts"	: "toasted";
	case 2: return killer_first ? "burns"	: "burned";
	case 3: return killer_first ? "roasts"	: "roasted";
	case 4: return killer_first ? "crisps"  : "crisped";
	}
    }
    else if ( dam_type == DAM_COLD )
	return killer_first ? "freezes" : "frozen";
    else if ( dam_type == DAM_LIGHTNING )
	return killer_first ? "electrocutes" : "electrocuted";
    else
    {
	switch ( number_range(1, 4) )
	{
	case 1: return killer_first ? "murders"		: "murdered";
	case 2: return killer_first ? "slays"		: "slain";
	case 3: return killer_first ? "slaughters"	: "slaughtered";
	case 4: return killer_first ? "kills"		: "killed";
	}
    }
    
    return "<???>";
}


void load_messages()
{
    MUD_EVENT *event;
    char letter;
    FILE *fp;

    if ( (fp = fopen(CRIER_FILE, "r")) == NULL )
    {
	bug( "load_messages: fopen", 0 );
	perror( CRIER_FILE );
	return;
    }
    
    while ( 1 ) {
    	letter = fread_letter( fp );
    	
    	if ( letter == '#' ) {
    	    break;
    	}
    	
    	ungetc( letter, fp );
    	
    	event 		= new_mud_event();
    	event->date	= fread_number( fp );
    	
    	free_string( event->message );
    	event->message	= fread_string( fp );
    	event->next	= mud_events;
    	mud_events	= event;
    	
    	num_mud_events++;
    }

    fclose( fp );
}


static void save_messages()
{
    MUD_EVENT *message;
    FILE *fp;

    fclose( fpReserve );
    if ( (fp = fopen(CRIER_FILE, "w")) == NULL )
    {
	bug( "add_message: fopen", 0 );
	perror( CRIER_FILE );
    }
    
    for ( message = mud_events; message != NULL; message = message->next )
	fprintf( fp, "%ld %s~\n", (long) message->date, message->message );
    fprintf( fp, "#\n" );
    
    fclose( fp );
    fpReserve = fopen( NULL_FILE, "r" ); 
}


void add_message( const char *msg, void *arg1, void *arg2, bool perm )
{
    static char * const he_she  [] = { "it",  "he",  "she" };
    static char * const him_her [] = { "it",  "him", "her" };
    static char * const his_her [] = { "its", "his", "her" };
 
    char buf[MAX_STRING_LENGTH];
    
    CHAR_DATA *ch1 = (CHAR_DATA *) arg1;
    CHAR_DATA *ch2 = (CHAR_DATA *) arg2;
    OBJ_DATA *obj1 = (OBJ_DATA *) arg1;
    OBJ_DATA *obj2 = (OBJ_DATA *) arg2;
    MUD_EVENT *message;
    const char *str;
    const char *i;
    char *point;
        
    point   = buf;
    str     = msg;
    while ( *str != '\0' )
    {
	if ( *str == '~' )
	{
	    str++;
	    *point++ = '-';
	    continue;
	}
	
	if ( *str != '$' )
	{
	    *point++ = *str++;
	    continue;
	}
	++str;
 
	if ( arg2 == NULL && *str >= 'A' && *str <= 'Z' )
	{
	    bug( "add_message: missing arg2 for code %d.", *str );
	    i = " <@@@> ";
	}
	else
	{
	    switch ( *str )
	    {
		default:  bug( "add_message: bad code %d.", *str );
			  i = " <@@@> ";                                break;
		case 't': i = (char *) arg1;                            break;
		case 'T': i = (char *) arg2;                            break;
		case 'n': i = PERS_TRUE( ch1 );                         break;
		case 'N': i = PERS_TRUE( ch2 );                         break;
		case 'e': i = he_she  [URANGE(0, ch1->sex, 2)];         break;
		case 'E': i = he_she  [URANGE(0, ch2->sex, 2)];         break;
		case 'm': i = him_her [URANGE(0, ch1->sex, 2)];         break;
		case 'M': i = him_her [URANGE(0, ch2->sex, 2)];         break;
		case 's': i = his_her [URANGE(0, ch1->sex, 2)];         break;
		case 'S': i = his_her [URANGE(0, ch2->sex, 2)];         break;
 
		case 'p':
		    i = obj1->short_descr;
		    break;
 
		case 'P':
		    i = obj2->short_descr;
		    break;
 	    }
	}
 
	++str;
	while ( ( *point = *i ) != '\0' )
	    ++point, ++i;
    }
 
    *point   = '\0';
    buf[0]   = UPPER( buf[0] );

    message 		= new_mud_event();
    message->message	= str_dup( buf );
    message->date	= perm ? 0 : time( NULL );
    message->next	= mud_events;
    mud_events		= message;
    num_mud_events++;
    
    save_messages();
}


void add_kill_message( CHAR_DATA *ch, CHAR_DATA *victim, int dam_type )
{
    char buf[MAX_STRING_LENGTH];

    if ( !IS_NPC(ch) && ch == victim )
    {
	if ( dam_type == DAM_DISEASE )
	    add_message(
		plague_messages[number_range(0, ARRAY_ITEMS(plague_messages) - 1)],
		ch, NULL, FALSE );
		
	return;
    }
	
    if ( (!IS_NPC(ch) && !IS_NPC(victim))
    ||   ( IS_NPC(ch) && !IS_NPC(victim)
    &&     number_percent() < 10 + victim->level) )
    {
	char postfix[MAX_STRING_LENGTH];
	const char *message;
	bool killer_first;
	
	if ( victim->in_room != NULL
	&&   victim->in_room->area->territory != 0
	&&   number_bits(1) )
	    sprintf( postfix, " in brutal %s slaying!",
	        territories[victim->in_room->area->territory].name );
	else
	    sprintf( postfix, "!" );
	
	killer_first = number_bits( 1 );
	if ( killer_first )
	    message = "$n %s $N%s";
	else
	    message = "$N %s by $n%s";

	sprintf( buf, message, damage_message(dam_type, killer_first), postfix );
	
	add_message( buf, ch, victim, FALSE );	
    }
    else if ( !IS_NPC(ch) && IS_NPC(victim) && victim->level - 10 >= ch->level )
    {
	const char *message;
	bool killer_first;
		
	killer_first = number_bits( 1 );
	if ( killer_first )
	    message = "Against all odds $n %s $N.";
	else
	    message = "$N %s by $n; odds-makers amazed!";

	sprintf( buf, message, damage_message(dam_type, killer_first) );
	
	add_message( buf, ch, victim, FALSE );	
    }
}


void do_message( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_STRING_LENGTH];
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0' )
    {
	send_to_char( "Syntax: message list\n\r"
	              "        message add <text>\n\r"
	              "        message remove <number>\n\r", ch );
	return;
    }
    
    if ( !str_prefix(arg, "list") )
    {
	MUD_EVENT *message;
	BUFFER *output;
	int i;
	
	output = new_buf();
	
	for ( i = 1, message = mud_events; 
	      message != NULL; 
	      i++, message = message->next )
	{
	    char buf[MAX_STRING_LENGTH];
	    
	    sprintf( buf, "%2d: %s\n\r", i, message->message );
	    add_buf( output, buf );
	}

	page_to_char( buf_string(output), ch );
	free_buf( output );
	
	if ( mud_events == NULL )
	{
	    send_to_char( "No messages to list.\n\r", ch );
	    return;
	}
	return;
    }
    else if ( !str_prefix(arg, "add") )
    {
	if ( argument[0] == '\0' )
	{
	    do_message( ch, "" );
	    return;
	}
	
	add_message( argument, NULL, NULL, FALSE );
	send_to_char( "OK.\n\r", ch );
	return;
    }
    else if ( !str_prefix(arg, "remove") )
    {
	MUD_EVENT *message, *previous;
	int i;
	
	if ( !is_number(argument) )
	{
	    do_message( ch, "" );
	    return;
	}
	
	i = atoi( argument );
	
	for ( previous = message = mud_events;
	      message != NULL;
	      previous = message, message = message->next )
	{	    
	    if ( --i == 0 )
	    {
		if ( previous == message )
		    mud_events = mud_events->next;
		else
		    previous->next = message->next;
		
		free_mud_event( message );
		num_mud_events--;
		
		save_messages();
		
		send_to_char( "OK.\n\r", ch );
		return;
	    }
	}
	
	send_to_char( "Message not found.\n\r", ch );
	return;
    }
    
    do_message( ch, "" );
}


void update_crier_messages()
{
    MUD_EVENT *message, *previous;
    
    for ( message = previous = mud_events;
          message != NULL;  )
    {
	if ( message->date == 0
	||   message->date + (60 * 60 * 24 * LIST_DAYS) >= time(NULL) )
	{
	    previous = message;
	    message  = message->next;
	    continue;
	}
	    
	if ( message == previous )
	{
	    message = mud_events = mud_events->next;
	    free_mud_event( previous );
	    previous = message;
	}
	else
	{
	    previous->next = message->next;
	    free_mud_event( message );
	    message = previous->next;
	}
	
	num_mud_events--;
	
	if ( message == NULL )
	    break;
    }
    
    save_messages();
}


void cry( CHAR_DATA *crier )
{
    MUD_EVENT *message;
    int i, msg;
    
    if ( num_mud_events == 0 )
	return;
    
    for ( msg = i = number_range(1, num_mud_events), message = mud_events;
          message != NULL;
          message = message->next )
    {
	if ( --i == 0 )
	{
	    bool no_shout = FALSE;

	    if ( IS_SET(crier->comm, COMM_NOSHOUT) )
		no_shout = TRUE;
	    REMOVE_BIT( crier->comm, COMM_NOSHOUT );
	    do_yell( crier, message->message );
	    if ( no_shout )
		SET_BIT( crier->comm, COMM_NOSHOUT );
		
	    return;
	}
    }
    
    bug( "cry: message %d not found.", msg );
    return;
}

