/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "magic.h"
#include "recycle.h"
#include "tables.h"

/* command procedures needed */
DECLARE_DO_FUN(do_return	);



/*
 * Local functions.
 */
void	affect_modify	args( ( CHAR_DATA *ch, AFFECT_DATA *paf, bool fAdd ) );
bool	bad_fit		args( ( OBJ_DATA *obj, CHAR_DATA *ch ) );


/* friend stuff -- for NPC's mostly */
bool is_friend(CHAR_DATA *ch,CHAR_DATA *victim)
{
    if (is_same_group(ch,victim))
	return TRUE;

    
    if (!IS_NPC(ch))
	return FALSE;

    if (!IS_NPC(victim))
    {
	if (IS_SET(ch->off_flags,ASSIST_PLAYERS))
	    return TRUE;
	else
	    return FALSE;
    }

    if (IS_AFFECTED(ch,AFF_CHARM))
	return FALSE;

    if (IS_SET(ch->off_flags,ASSIST_ALL))
	return TRUE;

    if (ch->group && ch->group == victim->group)
	return TRUE;

    if (IS_SET(ch->off_flags,ASSIST_VNUM) 
    &&  ch->pIndexData == victim->pIndexData)
	return TRUE;

    if (IS_SET(ch->off_flags,ASSIST_RACE) && CHAR_RACE(ch) == CHAR_RACE(victim))
	return TRUE;
     
    if (IS_SET(ch->off_flags,ASSIST_ALIGN)
    &&  !IS_SET(ch->act,ACT_NOALIGN) && !IS_SET(victim->act,ACT_NOALIGN)
    &&  ((IS_GOOD(ch) && IS_GOOD(victim))
    ||   (IS_EVIL(ch) && IS_EVIL(victim))
    ||   (IS_NEUTRAL(ch) && IS_NEUTRAL(victim))))
	return TRUE;

    return FALSE;
}

/* returns number of people on an object */
int count_users(OBJ_DATA *obj)
{
    CHAR_DATA *fch;
    int count = 0;

    if (obj->in_room == NULL)
	return 0;

    for (fch = obj->in_room->people; fch != NULL; fch = fch->next_in_room)
	if (fch->on == obj)
	    count++;

    return count;
}
     
/* returns material number */
int material_lookup (const char *name)
{
    return 0;
}

/* returns race number */
int race_lookup (const char *name)
{
   int race;

   for ( race = 0; race_table[race].name != NULL; race++)
   {
	if (LOWER(name[0]) == LOWER(race_table[race].name[0])
	&&  !str_prefix( name,race_table[race].name))
	    return race;
   }

   return 0;
}

int religion_lookup(const char *name)
{
    int religion;

    for (religion = 0; religion_table[religion].name != NULL; religion++)
    {
	if (LOWER(name[0]) == LOWER(religion_table[religion].name[0])
	&&  !str_prefix(name, religion_table[religion].name))
	    return religion;
    }
 
    return -1;
}

int hometown_lookup (const char *name)
{
   int ht;
        
   for ( ht = 1; hometown_table[ht].ht_name != NULL; ht++)
   {
	if (LOWER(name[0]) == LOWER(hometown_table[ht].ht_name[0])
	&&  !str_prefix( name,hometown_table[ht].ht_name))
	    return ht;
   }

   return 0;
}

int liq_lookup (const char *name)
{
    int liq;

    for ( liq = 0; liq_table[liq].liq_name != NULL; liq++)
    {
	if (LOWER(name[0]) == LOWER(liq_table[liq].liq_name[0])
	&& !str_prefix(name,liq_table[liq].liq_name))
	    return liq;
    }

    return -1;
}

int weapon_lookup (const char *name)
{
    int type;

    for (type = 0; weapon_table[type].name != NULL; type++)
    {
	if (LOWER(name[0]) == LOWER(weapon_table[type].name[0])
	&&  !str_prefix(name,weapon_table[type].name))
	    return type;
    }
 
    return -1;
}

int advantage_lookup( const struct advantage_type *table, const char *name, CHAR_DATA *ch )
{
    int i;
    for ( i = 0; !IS_NULLSTR(table[i].name); ++i )
    {
        if ( LOWER(name[0]) == LOWER(table[i].name[0])
        &&   !str_prefix(name, table[i].name) )
        {
            if ( table[i].restrictions == 0 || !char_matches(ch, table[i].restrictions) )
                return i;
        }
    }
    return -1;
}


int weapon_type (const char *name)
{
    int type;
 
    for (type = 0; weapon_table[type].name != NULL; type++)
    {
        if (LOWER(name[0]) == LOWER(weapon_table[type].name[0])
        &&  !str_prefix(name,weapon_table[type].name))
            return weapon_table[type].type;
    }
 
    return WEAPON_EXOTIC;
}


int poison_type (const char *name)
{
    int type;
 
    for (type = 0; poison_table[type].name != NULL; type++)
    {
        if (LOWER(name[0]) == LOWER(poison_table[type].name[0])
        &&  !str_prefix(name, poison_table[type].name))
            return poison_table[type].type;
    }
 
    return -1;
}


int item_lookup(const char *name)
{
    int type;

    for (type = 0; item_table[type].name != NULL; type++)
    {
        if (LOWER(name[0]) == LOWER(item_table[type].name[0])
        &&  !str_prefix(name,item_table[type].name))
            return item_table[type].type;
    }
 
    return -1;
}

char *item_name(int item_type)
{
    int type;

    for (type = 0; item_table[type].name != NULL; type++)
	if (item_type == item_table[type].type)
	    return item_table[type].name;
    return "none";
}

char *weapon_name( int weapon_type)
{
    int type;
 
    for (type = 0; weapon_table[type].name != NULL; type++)
        if (weapon_type == weapon_table[type].type)
            return weapon_table[type].name;
    return "exotic";
}

int attack_lookup  (const char *name)
{
    int att;

    for ( att = 0; attack_table[att].name != NULL; att++)
    {
	if (LOWER(name[0]) == LOWER(attack_table[att].name[0])
	&&  !str_prefix(name,attack_table[att].name))
	    return att;
    }

    return 0;
}

/* returns a flag for wiznet */
long wiznet_lookup (const char *name)
{
    int flag;

    for (flag = 0; wiznet_table[flag].name != NULL; flag++)
    {
	if (LOWER(name[0]) == LOWER(wiznet_table[flag].name[0])
	&& !str_prefix(name,wiznet_table[flag].name))
	    return flag;
    }

    return -1;
}

/* returns class number */
int class_lookup (const char *name)
{
   int class;
 
   for ( class = 0; class < MAX_CLASS; class++)
   {
        if (LOWER(name[0]) == LOWER(class_table[class].name[0])
        &&  !str_prefix( name,class_table[class].name))
            return class;
   }
 
   return -1;
}

/* for immunity, vulnerabiltiy, and resistant
   the 'globals' (magic and weapons) may be overriden
   three other cases -- wood, silver, and iron -- are checked in fight.c */

int check_immune(CHAR_DATA *ch, int dam_type)
{
    int immune, def;
    int bit;

    immune = -1;
    def = IS_NORMAL;

    if (dam_type == DAM_NONE)
	return immune;

    if (dam_type <= 3)
    {
	if (IS_SET(CHAR_IMM(ch),IMM_WEAPON))
	    def = IS_IMMUNE;
	else if (IS_SET(CHAR_RES(ch),RES_WEAPON))
	    def = IS_RESISTANT;
	else if (IS_SET(CHAR_VULN(ch),VULN_WEAPON))
	    def = IS_VULNERABLE;
    }
    else /* magical attack */
    {
	if (IS_SET(CHAR_IMM(ch),IMM_MAGIC))
	    def = IS_IMMUNE;
	else if (IS_SET(CHAR_RES(ch),RES_MAGIC))
	    def = IS_RESISTANT;
	else if (IS_SET(CHAR_VULN(ch),VULN_MAGIC))
	    def = IS_VULNERABLE;
    }

    /* set bits to check -- VULN etc. must ALL be the same or this will fail */
    switch (dam_type)
    {
	case(DAM_BASH):		bit = IMM_BASH;		break;
	case(DAM_PIERCE):	bit = IMM_PIERCE;	break;
	case(DAM_SLASH):	bit = IMM_SLASH;	break;
	case(DAM_FIRE):		bit = IMM_FIRE;		break;
	case(DAM_COLD):		bit = IMM_COLD;		break;
	case(DAM_LIGHTNING):	bit = IMM_LIGHTNING;	break;
	case(DAM_ACID):		bit = IMM_ACID;		break;
	case(DAM_POISON):	bit = IMM_POISON;	break;
	case(DAM_NEGATIVE):	bit = IMM_NEGATIVE;	break;
	case(DAM_HOLY):		bit = IMM_HOLY;		break;
	case(DAM_ENERGY):	bit = IMM_ENERGY;	break;
	case(DAM_MENTAL):	bit = IMM_MENTAL;	break;
	case(DAM_DISEASE):	bit = IMM_DISEASE;	break;
	case(DAM_DROWNING):	bit = IMM_DROWNING;	break;
	case(DAM_LIGHT):	bit = IMM_LIGHT;	break;
	case(DAM_CHARM):	bit = IMM_CHARM;	break;
	case(DAM_SOUND):	bit = IMM_SOUND;	break;
	default:		return def;
    }

    if ( IS_SET(CHAR_IMM(ch), bit) )
	immune = IS_IMMUNE;
    if ( IS_SET(CHAR_RES(ch), bit) && immune != IS_IMMUNE )
	immune = IS_RESISTANT;
    if ( IS_SET(CHAR_VULN(ch), bit) )
    {
	if (immune == IS_IMMUNE)
	    immune = IS_RESISTANT;
	else if (immune == IS_RESISTANT)
	    immune = IS_NORMAL;
	else
	    immune = IS_VULNERABLE;
    }

    if (immune == -1)
	return def;
    else
	return immune;
}

bool is_clan(CHAR_DATA *ch)
{
    return ch->clan ? TRUE : FALSE;
}

bool is_same_clan(CHAR_DATA *ch, CHAR_DATA *victim)
{
    if (clan_table[ch->clan].independent)
	return FALSE;
    else 
	return (ch->clan == victim->clan);
}

/* checks mob format */
bool is_old_mob(CHAR_DATA *ch)
{
    if (ch->pIndexData == NULL)
	return FALSE;
    else if (ch->pIndexData->new_format)
	return FALSE;
    return TRUE;
}
 
/* for returning skill information */
int get_skill(CHAR_DATA *ch, int sn)
{
    int skill;

    if (sn == -1) /* shorthand for level based skills */
    {
	skill = ch->level * 5 / 2;
    }

    else if (sn < -1 || sn > max_player_skill)
    {
	bug("Bad sn %d in get_skill.",sn);
	skill = 0;
    }

    else if (!IS_NPC(ch) && !ch->polymorphed)
    {
	if ( sn == gsn_sneak 
	&&   IS_SET(race_table[CHAR_RACE(ch)].aff, AFF_SNEAK) )
	    skill = 100;
	else if ( ch->level < skill_level(ch, sn) )
	    skill = 0;
	else
	    skill = ch->pcdata->learned[sn] + ch->pcdata->skill_mods[sn];

        if ( skill_table[sn].sclass == SCLASS_WEAPON &&
             has_customization(ch, ADVANTAGE_NATURAL_FIGHTER) )
            skill += 10;
        if ( skill_table[sn].sclass == SCLASS_SUBTERFUGE &&
             has_customization(ch, ADVANTAGE_CRAFTY) )
            skill += 10;
        if ( sn == gsn_fast_healing &&
             has_customization(ch, ADVANTAGE_QUICK_HEALER) )
            skill += 50;
        if ( sn == gsn_haggle &&
             has_customization(ch, ANCESTOR_ANDARAS) )
            skill += 10;
        if ( (sn == gsn_music
        ||   skill_table[sn].bard_song) &&
             has_customization(ch, ANCESTOR_DMOON) )
            skill += 10;
    } 
    else /* mobiles */
    {
        long act, off;
        if ( ch->polymorphed )
        {
            act = ch->polymorphed->act;
            off = ch->polymorphed->off_flags;
        }
        else
        {
            act = ch->act;
            off = ch->off_flags;
            /* Polymorphed player maintains some of their former knowledge */
            if ( !IS_NPC(ch) )
            {
                if ( IS_SET(class_table[ch->class].flags, CLASS_SNEAKY) )
                    act |= ACT_THIEF;
                if ( IS_SET(class_table[ch->class].flags, CLASS_RELIGIOUS) )
                    act |= ACT_CLERIC;
            }
        }

        if (skill_table[sn].spell_fun != spell_null)
            /* Calculate spell skill level. Players maintain former knowledge
             * for spells. Eg. for mage drake, liches, etc. We use other flags
             * to turn spells on/off as a whole or in part in the polymorph
             * affect data.
             */
            if ( IS_NPC(ch) )
	        skill = 40 + 2 * ch->level;
            else if ( ch->level < skill_level(ch, sn) )
                skill = 0;
            else
                skill = ch->pcdata->learned[sn] + ch->pcdata->skill_mods[sn];

	else if (sn == gsn_sneak || sn == gsn_hide)
            skill = ch->level * 2 + 20;
	
        else if ((sn == gsn_dodge && IS_SET(off,OFF_DODGE))
	||       (sn == gsn_parry && IS_SET(off,OFF_PARRY)))
	    skill = ch->level * 2;
        
	else if (sn == gsn_shield_block)
	    skill = 10 + 2 * ch->level;

	else if (sn == gsn_arms_mastery 
	&& (IS_SET(act,ACT_WARRIOR) || IS_SET(act,ACT_THIEF)))
	    skill = 10 + 3 * ch->level;

	else if (sn == gsn_weapon_brawling && IS_SET(act,ACT_WARRIOR))
	    skill = 4 * ch->level - 40;

	else if (sn == gsn_hand_to_hand)
	    skill = 40 + 2 * ch->level;

 	else if (sn == gsn_trip && IS_SET(off,OFF_TRIP))
	    skill = 10 + 3 * ch->level;

 	else if (sn == gsn_bash && IS_SET(off,OFF_BASH))
	    skill = 10 + 3 * ch->level;

	else if (sn == gsn_disarm 
	     &&  (IS_SET(off,OFF_DISARM) 
	     ||   IS_SET(act,ACT_WARRIOR)
	     ||   IS_SET(act,ACT_THIEF)))
	    skill = 20 + 3 * ch->level;

	else if (sn == gsn_berserk && IS_SET(off,OFF_BERSERK))
	    skill = 3 * ch->level;

	else if (sn == gsn_kick)
	    skill = 10 + 3 * ch->level;

	else if (sn == gsn_backstab && IS_SET(act,ACT_THIEF))
	    skill = 20 + 2 * ch->level;

	else if (sn == gsn_circle && IS_SET(act,ACT_THIEF))
	    skill = 20 + 2 * ch->level;

	else if ( sn == gsn_rescue
	     &&   (IS_SET(off, OFF_RESCUE)
	     ||    IS_SET(act, ACT_WARRIOR)) )
	    skill = 40 + ch->level; 

	else if (sn == gsn_unmount)
	    skill = 40 + ch->level; 

	else if (sn == gsn_recall)
	    skill = 40 + ch->level;

	else if (sn == gsn_dirt)
	    skill = 10 + 3 * ch->level;
	
	else if (sn == gsn_ride)
	    skill = 2 * ch->level;

	else if (sn == gsn_sword
	||  sn == gsn_bow
	||  sn == gsn_dagger
	||  sn == gsn_spear
	||  sn == gsn_mace
	||  sn == gsn_axe
	||  sn == gsn_flail
	||  sn == gsn_whip
	||  sn == gsn_polearm)
	    skill = 40 + 5 * ch->level / 2;

	else 
	   skill = 0;
    }

    if (ch->daze > 0)
    {
	if (skill_table[sn].spell_fun != spell_null)
	    skill /= 2;
	else
	    skill = 2 * skill / 3;
    }

    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK]  > 10 )
	skill = 9 * skill / 10;

    if ( IS_AFFECTED(ch, AFF_CHARM) && !IS_AFFECTED2(ch, AFF2_DOMINATION) )
	skill /= 2;

    return URANGE(0,skill,100);
}

/* for returning weapon information */
int get_weapon_sn(CHAR_DATA *ch)
{
    OBJ_DATA *wield;
    int sn;

    wield = get_eq_char( ch, WEAR_WIELD );
    if (wield == NULL || wield->item_type != ITEM_WEAPON)
        sn = gsn_hand_to_hand;
    else switch (wield->value[0])
    {
        default :               sn = -1;                break;
        case(WEAPON_SWORD):     sn = gsn_sword;         break;
        case(WEAPON_BOW):	sn = gsn_bow;        break;
        case(WEAPON_DAGGER):    sn = gsn_dagger;        break;
        case(WEAPON_SPEAR):     sn = gsn_spear;         break;
        case(WEAPON_MACE):      sn = gsn_mace;          break;
        case(WEAPON_AXE):       sn = gsn_axe;           break;
        case(WEAPON_FLAIL):     sn = gsn_flail;         break;
        case(WEAPON_WHIP):      sn = gsn_whip;          break;
        case(WEAPON_POLEARM):   sn = gsn_polearm;       break;
   }
   return sn;
}

/* for returning weapon information */
int get_secondary_sn(CHAR_DATA *ch)
{
    OBJ_DATA *wield;
    int sn;

    wield = get_eq_char( ch, WEAR_SECONDARY );
    if (wield == NULL || wield->item_type != ITEM_WEAPON)
        sn = gsn_hand_to_hand;
    else switch (wield->value[0])
    {
        default :               sn = -1;                break;
        case(WEAPON_SWORD):     sn = gsn_sword;         break;
        case(WEAPON_BOW):	sn = gsn_bow;           break;
        case(WEAPON_DAGGER):    sn = gsn_dagger;        break;
        case(WEAPON_SPEAR):     sn = gsn_spear;         break;
        case(WEAPON_MACE):      sn = gsn_mace;          break;
        case(WEAPON_AXE):       sn = gsn_axe;           break;
        case(WEAPON_FLAIL):     sn = gsn_flail;         break;
        case(WEAPON_WHIP):      sn = gsn_whip;          break;
        case(WEAPON_POLEARM):   sn = gsn_polearm;       break;
   }
   return sn;
}

int get_weapon_skill(CHAR_DATA *ch, int sn)
{
     int skill;

     /* -1 is exotic */
    if (IS_NPC(ch))
    {
	if (sn == -1)
	    skill = 3 * ch->level;
	else if (sn == gsn_hand_to_hand)
	    skill = 40 + 2 * ch->level;
	else 
	    skill = 40 + 5 * ch->level / 2;
    }
    
    else
    {
	if (sn == -1)
	    skill = 3 * ch->level;
	else
	    skill = ch->pcdata->learned[sn] + ch->pcdata->skill_mods[sn];
    }

    return URANGE(0,skill,100);
} 


/* used to de-screw characters */
void reset_char(CHAR_DATA *ch)
{
     int loc,mod,stat;
     OBJ_DATA *obj;
     AFFECT_DATA *af;
     int i;

     if (IS_NPC(ch))
	return;

    if (ch->pcdata->perm_hit == 0 
    ||	ch->pcdata->perm_mana == 0
    ||  ch->pcdata->perm_move == 0
    ||	ch->pcdata->last_level == 0)
    {
    /* do a FULL reset */
	for (loc = 0; loc < MAX_WEAR; loc++)
	{
	    obj = get_eq_char(ch,loc);
	    if (obj == NULL || IS_OBJ_STAT(obj, ITEM_DISPELLED))
		continue;
	    if (!obj->enchanted)
	    for ( af = obj->pIndexData->affected; af != NULL; af = af->next )
	    {
		mod = af->modifier;
		switch(af->location)
		{
		    case APPLY_SEX:	ch->sex		-= mod;
					if (ch->sex < 0 || ch->sex >2)
					    ch->sex = IS_NPC(ch) ?
						0 :
						ch->pcdata->true_sex;
									break;
		    case APPLY_MANA:	ch->max_mana	-= mod;		break;
		    case APPLY_HIT:	ch->max_hit	-= mod;		break;
		    case APPLY_MOVE:	ch->max_move	-= mod;		break;
		}
	    }

            for ( af = obj->affected; af != NULL; af = af->next )
            {
                mod = af->modifier;
                switch(af->location)
                {
                    case APPLY_SEX:     ch->sex         -= mod;         break;
                    case APPLY_MANA:    ch->max_mana    -= mod;         break;
                    case APPLY_HIT:     ch->max_hit     -= mod;         break;
                    case APPLY_MOVE:    ch->max_move    -= mod;         break;
                }
            }
	}
	/* now reset the permanent stats */
	ch->pcdata->perm_hit 	= ch->max_hit;
	ch->pcdata->perm_mana 	= ch->max_mana;
	ch->pcdata->perm_move	= ch->max_move;
	ch->pcdata->last_level	= ch->played/3600;
	if (ch->pcdata->true_sex < 0 || ch->pcdata->true_sex > 2)
        {
            if (ch->sex > 0 && ch->sex < 3)
                ch->pcdata->true_sex    = ch->sex;
            else
                ch->pcdata->true_sex    = 0;
        }
    }

    /* now restore the character to his/her true condition */
    for (stat = 0; stat < MAX_STATS; stat++)
	ch->mod_stat[stat] = 0;

    if (ch->pcdata->true_sex < 0 || ch->pcdata->true_sex > 2)
	ch->pcdata->true_sex = 0; 
    ch->sex		= ch->pcdata->true_sex;
    ch->max_hit 	= ch->pcdata->perm_hit;
    ch->max_mana	= ch->pcdata->perm_mana;
    ch->max_move	= ch->pcdata->perm_move;
   
    for (i = 0; i < 4; i++)
    	ch->armor[i]	= 100;

    ch->hitroll		= 0;
    ch->damroll		= 0;
    ch->saving_throw	= 0;

    /* now start adding back the effects */
    for (loc = 0; loc < MAX_WEAR; loc++)
    {
        obj = get_eq_char(ch,loc);
        if (obj == NULL)
            continue;
	for (i = 0; i < 4; i++)
	    ch->armor[i] -= apply_ac( obj, loc, i );

	if ( IS_OBJ_STAT(obj, ITEM_DISPELLED) )
	    continue;

        if (!obj->enchanted)
	for ( af = obj->pIndexData->affected; af != NULL; af = af->next )
        {
            mod = af->modifier;
            switch(af->location)
            {
		case APPLY_STR:		ch->mod_stat[STAT_STR]	+= mod;	break;
		case APPLY_DEX:		ch->mod_stat[STAT_DEX]	+= mod; break;
		case APPLY_INT:		ch->mod_stat[STAT_INT]	+= mod; break;
		case APPLY_WIS:		ch->mod_stat[STAT_WIS]	+= mod; break;
		case APPLY_CON:		ch->mod_stat[STAT_CON]	+= mod; break;
		case APPLY_PER:		ch->mod_stat[STAT_PER]  += mod; break;
		case APPLY_CHA:		ch->mod_stat[STAT_CHA]  += mod; break;

		case APPLY_SEX:		ch->sex			+= mod; break;
		case APPLY_MANA:	ch->max_mana		+= mod; break;
		case APPLY_HIT:		ch->max_hit		+= mod; break;
		case APPLY_MOVE:	ch->max_move		+= mod; break;

		case APPLY_AC:		
		    for (i = 0; i < 4; i ++)
			ch->armor[i] += mod; 
		    break;
		case APPLY_HITROLL:	ch->hitroll		+= mod; break;
		case APPLY_DAMROLL:	ch->damroll		+= mod; break;
	
		case APPLY_SAVES:		ch->saving_throw += mod; break;
		case APPLY_SAVING_ROD: 		ch->saving_throw += mod; break;
		case APPLY_SAVING_PETRI:	ch->saving_throw += mod; break;
		case APPLY_SAVING_BREATH: 	ch->saving_throw += mod; break;
		case APPLY_SAVING_SPELL:	ch->saving_throw += mod; break;
	    }
        }
 
        for ( af = obj->affected; af != NULL; af = af->next )
        {
            mod = af->modifier;
            switch(af->location)
            {
                case APPLY_STR:         ch->mod_stat[STAT_STR]  += mod; break;
                case APPLY_DEX:         ch->mod_stat[STAT_DEX]  += mod; break;
                case APPLY_INT:         ch->mod_stat[STAT_INT]  += mod; break;
                case APPLY_WIS:         ch->mod_stat[STAT_WIS]  += mod; break;
                case APPLY_CON:         ch->mod_stat[STAT_CON]  += mod; break;
                case APPLY_PER:         ch->mod_stat[STAT_PER]  += mod; break;
                case APPLY_CHA:         ch->mod_stat[STAT_CHA]  += mod; break;
 
                case APPLY_SEX:         ch->sex                 += mod; break;
                case APPLY_MANA:        ch->max_mana            += mod; break;
                case APPLY_HIT:         ch->max_hit             += mod; break;
                case APPLY_MOVE:        ch->max_move            += mod; break;
 
                case APPLY_AC:
                    for (i = 0; i < 4; i ++)
                        ch->armor[i] += mod;
                    break;
		case APPLY_HITROLL:     ch->hitroll             += mod; break;
                case APPLY_DAMROLL:     ch->damroll             += mod; break;
 
                case APPLY_SAVES:         ch->saving_throw += mod; break;
                case APPLY_SAVING_ROD:          ch->saving_throw += mod; break;
                case APPLY_SAVING_PETRI:        ch->saving_throw += mod; break;
                case APPLY_SAVING_BREATH:       ch->saving_throw += mod; break;
                case APPLY_SAVING_SPELL:        ch->saving_throw += mod; break;
            }
	}
    }
  
    /* now add back spell effects */
    for (af = ch->affected; af != NULL; af = af->next)
    {
        mod = af->modifier;
        switch(af->location)
        {
                case APPLY_STR:         ch->mod_stat[STAT_STR]  += mod; break;
                case APPLY_DEX:         ch->mod_stat[STAT_DEX]  += mod; break;
                case APPLY_INT:         ch->mod_stat[STAT_INT]  += mod; break;
                case APPLY_WIS:         ch->mod_stat[STAT_WIS]  += mod; break;
                case APPLY_CON:         ch->mod_stat[STAT_CON]  += mod; break;
                case APPLY_PER:         ch->mod_stat[STAT_PER]  += mod; break;
                case APPLY_CHA:         ch->mod_stat[STAT_CHA]  += mod; break;
 
                case APPLY_SEX:         ch->sex                 += mod; break;
                case APPLY_MANA:        ch->max_mana            += mod; break;
                case APPLY_HIT:         ch->max_hit             += mod; break;
                case APPLY_MOVE:        ch->max_move            += mod; break;
 
                case APPLY_AC:
                    for (i = 0; i < 4; i ++)
                        ch->armor[i] += mod;
                    break;
                case APPLY_HITROLL:     ch->hitroll             += mod; break;
                case APPLY_DAMROLL:     ch->damroll             += mod; break;
 
                case APPLY_SAVES:         ch->saving_throw += mod; break;
                case APPLY_SAVING_ROD:          ch->saving_throw += mod; break;
                case APPLY_SAVING_PETRI:        ch->saving_throw += mod; break;
                case APPLY_SAVING_BREATH:       ch->saving_throw += mod; break;
                case APPLY_SAVING_SPELL:        ch->saving_throw += mod; break;
        } 
    }

    /* make sure sex is RIGHT!!!! */
    if (ch->sex < 0 || ch->sex > 2)
	ch->sex = ch->pcdata->true_sex;
}


/*
 * Retrieve a character's trusted level for permission checking.
 */
int get_trust( CHAR_DATA *ch )
{
    if ( ch->desc != NULL && ch->desc->original != NULL )
	ch = ch->desc->original;

    if (ch->trust)
	return ch->trust;

    if ( IS_NPC(ch) && ch->level >= LEVEL_HERO )
	return LEVEL_HERO - 1;
    else
	return ch->level;
}


/*
 * Retrieve a character's age.
 */
int get_age( CHAR_DATA *ch )
{
    return 17 + pc_race_table[ch->race].race_age + ( ch->played + (int) (current_time - ch->logon) ) / (TICKS_IN_HOUR * 72000);
}

int get_mob_stat( int stat, int level, MOB_INDEX_DATA *pIndexData )
{
    int value = UMIN( 100, 44 + level );
    switch ( stat )
    {
        case STAT_STR:
            if ( IS_SET(pIndexData->act, ACT_WARRIOR) )
                value += 12;
            if ( IS_SET(pIndexData->act, ACT_CLERIC) )
                value += 4;
            value += 2 * (pIndexData->size - SIZE_MEDIUM);
            break;
        case STAT_INT:
            if ( IS_SET(pIndexData->act, ACT_WARRIOR) )
                value -= 4;
            if ( IS_SET(pIndexData->act, ACT_THIEF) )
                value += 4;
            if ( IS_SET(pIndexData->act, ACT_MAGE) )
                value += 12;
            break;
        case STAT_WIS:
            if ( IS_SET(pIndexData->act, ACT_THIEF) )
                value -= 4;
            if ( IS_SET(pIndexData->act, ACT_CLERIC) )
                value += 12;
            break;
        case STAT_DEX:
            if ( IS_SET(pIndexData->act, ACT_THIEF) )
                value += 12;
            if ( IS_SET(pIndexData->act, ACT_CLERIC) )
                value -= 4;
            if ( IS_SET(pIndexData->act, ACT_MAGE) )
                value += 4;
            if ( IS_SET(pIndexData->off_flags, OFF_FAST) )
                value += 8;
            break;
        case STAT_CON:
            if ( IS_SET(pIndexData->act, ACT_WARRIOR) )
                value += 8;
            value += 2 * (pIndexData->size - SIZE_MEDIUM);
            break;
        default:;
    }
    return value;
}

/* command for retrieving stats */
int get_curr_stat( CHAR_DATA *ch, int stat )
{
//    char buf[MAX_STRING_LENGTH];
    int max;
    int customization_mod = 0;

    if ( !IS_NPC(ch) && ch->pcdata->customizations )
    {
        switch ( stat )
        {
            case STAT_STR:
                if ( has_customization(ch, DISADVANTAGE_ELDERLY) )
                    customization_mod -= 4;
                break;
            case STAT_INT:
                if ( has_customization(ch, ADVANTAGE_EIDETIC_MEMORY) )
                    customization_mod += 8;
                if ( has_customization(ch, DISADVANTAGE_ELDERLY) )
                    customization_mod -= 4;
                break;
            case STAT_WIS:
                if ( has_customization(ch, ADVANTAGE_STRONG_WILL) )
                    customization_mod += 4;
                if ( has_customization(ch, DISADVANTAGE_ELDERLY) )
                    customization_mod -= 4;
                if ( has_customization(ch, DISADVANTAGE_WEAK_WILLED) )
                    customization_mod -= 4;
                break;
            case STAT_DEX:
                if ( has_customization(ch, DISADVANTAGE_KLUTZ) )
                    customization_mod -= 4;
                if ( has_customization(ch, DISADVANTAGE_ELDERLY) )
                    customization_mod -= 4;
                break;
            case STAT_CON:
                if ( has_customization(ch, ADVANTAGE_TOUGH) )
                    customization_mod += 4;
                if ( has_customization(ch, DISADVANTAGE_BAD_HEALTH) )
                    customization_mod -= 4;
                if ( has_customization(ch, DISADVANTAGE_ELDERLY) )
                    customization_mod -= 4;
                break;
            case STAT_CHA:
                if ( has_customization(ch, ADVANTAGE_RENSHAIS_BLESSING) )
                    customization_mod += 4;
                if ( has_customization(ch, ADVANTAGE_BLAND) )
                    customization_mod -= 4;
                if ( has_customization(ch, ADVANTAGE_DANGEROUS_BEAUTY) )
                    customization_mod += 8;
                if ( has_customization(ch, ADVANTAGE_LARGE) )
                    customization_mod -= 4;
                if ( has_customization(ch, DISADVANTAGE_INTROVERSION) )
                    customization_mod -= 16;
                if ( has_customization(ch, DISADVANTAGE_RENSHAIS_CURSE) )
                    customization_mod -= 4;
                if ( has_customization(ch, DISADVANTAGE_SCARED) )
                    customization_mod -= 4;
                if ( has_customization(ch, DISADVANTAGE_HIDEOUSLY_SCARED) )
                    customization_mod -= 12;
                if ( has_customization(ch, ANCESTOR_BATON) )
                    customization_mod += 4;
                if ( has_customization(ch, ANCESTOR_SUMANDAR) )
                    customization_mod += 4;
                break;
            case STAT_PER:
                if ( has_customization(ch, DISADVANTAGE_BAD_EYESIGHT) )
                    customization_mod -= 4;
                if ( has_customization(ch, DISADVANTAGE_ELDERLY) )
                    customization_mod -= 4;
                break;
            default:;
        }
    }

    if (IS_NPC(ch) || ch->level > LEVEL_IMMORTAL || ch->polymorphed)
	max = 100;

    else
    {
	max = pc_race_table[ch->race].max_stats[stat] + 16;

/* Humans have been giving an extra +6 to their max stat so that they
can wear more magical equipment. However, their prime attribute stats
only gets at +2 increase. After all, Humans are "jack of all trades".
Other races, now get a +6 to their prime attribute stat, so other
races are now really specialised in whatever class they are. Furthermo,
the max trainable stat of Humans as been lowered in cost.c so to get
really good stats, humans have to wear more equipment. */

	if (class_table[ch->class].attr_prime == stat) {
	  if (ch->race == race_lookup("human")) {
	    max += 2;
	  }
	  else {
	    max += 6;
	  }
	}
/* Bonus to human's max stat for wearing magical items. Larger max stats. 
*/
	if ( ch->race == race_lookup("human"))
	    max += 8;

        max += customization_mod;

	max = UMIN(max,100);
    }
  
//    sprintf (buf, "Max = %d ", max);
//    wiznet(buf, NULL,NULL,WIZ_SACCING,0,0);
//    sprintf (buf, "Mod stat = %d ", (ch->mod_stat[stat]));
//    wiznet(buf, NULL,NULL,WIZ_SACCING,0,0);
//    sprintf (buf, "Stat #%d = %d", stat, URANGE(12, CHAR_PERM_STAT(ch, stat) + ch->mod_stat[stat], max) );
//    wiznet(buf, NULL,NULL,WIZ_SACCING,0,0);
    return URANGE(12, CHAR_PERM_STAT(ch, stat) + ch->mod_stat[stat] + customization_mod, max);
}

int get_real_stat( CHAR_DATA *ch, int stat )
{
    MOB_INDEX_DATA *polymorph_save = ch->polymorphed;
    int result;

    ch->polymorphed = NULL;
    result = get_curr_stat( ch, stat );
    ch->polymorphed= polymorph_save;
    return result;
}

const char *get_stat_word( CHAR_DATA *ch, int stat )
{
    int max, avg, word, value;
    
    value = get_curr_stat( ch, stat );
    
    avg  = IS_NPC(ch) ? 50 : pc_race_table[ch->race].stats[stat];

    if ( value <= avg )
	word = AVG_STAT_NAME * UMAX(0, value) / (avg + 1);
    else
    {
	if ( IS_NPC(ch) || ch->polymorphed )
	    max = 100;

	else
	{
	    max = pc_race_table[ch->race].max_stats[stat] + 16;

	    if (class_table[ch->class].attr_prime == stat)
		max += 8;

	    if ( ch->race == race_lookup("human"))
		max += 4;

	    if ( ch->level >= LEVEL_IMMORTAL )
		max = UMAX( max, 100 );
	}

	word = (MAX_STAT_NAME - AVG_STAT_NAME) * (value - avg) / (max - avg)
	       + (AVG_STAT_NAME - 1);
	word = URANGE( 0, word, (MAX_STAT_NAME - 1) );
    }

    switch( stat )
    {
    default:	
	bug( "get_stat_word: bad stat (%d).", stat );
	return "";
    case STAT_STR:	return str_names[ word ];
    case STAT_INT:	return int_names[ word ];
    case STAT_WIS:	return wis_names[ word ];
    case STAT_DEX:	return dex_names[ word ];
    case STAT_CON:	return con_names[ word ];
    case STAT_PER:	return per_names[ word ];
    case STAT_CHA:	return cha_names[ word ];
    }
}

/* command for returning max training score */
int get_max_train( CHAR_DATA *ch, int stat )
{
    int max;

    if (IS_NPC(ch) || ch->level > LEVEL_IMMORTAL)
	return 100;

    max = pc_race_table[ch->race].max_stats[stat];
    if (class_table[ch->class].attr_prime == stat)
    {
	if (ch->race == race_lookup("human"))
	   max += 12;
	else
	   max += 8;
    }

    return UMIN(max,100);
}
   
	
/*
 * Retrieve a character's carry capacity.
 */
int can_carry_n( CHAR_DATA *ch )
{
    int result;

    if ( !IS_NPC(ch) && ch->level >= LEVEL_IMMORTAL )
	return 1000;

    if ( IS_NPC(ch) && IS_SET(ch->act, ACT_PET) )
	return 0;

    result = MAX_WEAR + get_curr_stat(ch,STAT_DEX)/2 + ch->level;
    if ( has_customization(ch, DISADVANTAGE_BAD_BACK) )
        result /= 2;
    return result;
}



/*
 * Retrieve a character's carry capacity.
 */
int can_carry_w( CHAR_DATA *ch )
{
    if ( !IS_NPC(ch) && ch->level >= LEVEL_IMMORTAL )
	return 10000000;

    if ( IS_NPC(ch) && IS_SET(ch->act, ACT_PET) )
	return 0;

    return str_app[get_curr_stat(ch,STAT_STR)/4].carry * 10 + ch->level * 25;
}


/*
 * Get modifier for a stat
 */
int get_stat_mod( CHAR_DATA *ch, int stat )
{

    int modifier;

    modifier = 0;

    if (stat == STAT_WIS)
     modifier = wis_app[get_real_stat(ch, STAT_WIS)/4].wis_spell_modifier;
    else if (stat == STAT_INT)
     modifier = int_app[get_real_stat(ch, STAT_INT)/4].int_spell_modifier;

    return modifier;
}



/*
 * See if a string is one of the names of an object.
 */

bool is_name ( char *str, char *namelist )
{
    char name[MAX_INPUT_LENGTH], part[MAX_INPUT_LENGTH];
    char *list, *string;

    /* fix crash on NULL namelist */
    if (namelist == NULL || namelist[0] == '\0')
    	return FALSE;

    /* fixed to prevent is_name on "" returning TRUE */
    if (str[0] == '\0')
	return FALSE;

    string = str;
    /* we need ALL parts of string to match part of namelist */
    for ( ; ; )  /* start parsing string */
    {
	str = one_argument(str,part);

	if (part[0] == '\0' )
	    return TRUE;

	/* check to see if this is part of namelist */
	list = namelist;
	for ( ; ; )  /* start parsing namelist */
	{
	    list = one_argument(list,name);
	    if (name[0] == '\0')  /* this name was not found */
		return FALSE;

	    if (!str_prefix(string,name))
		return TRUE; /* full pattern match */

	    if (!str_prefix(part,name))
		break;
	}
    }
}

bool is_exact_name(char *str, char *namelist )
{
    char name[MAX_INPUT_LENGTH];

    if (namelist == NULL)
	return FALSE;

    for ( ; ; )
    {
	namelist = one_argument( namelist, name );
	if ( name[0] == '\0' )
	    return FALSE;
	if ( !str_cmp( str, name ) )
	    return TRUE;
    }
}

char *true_name( CHAR_DATA *ch )
{
    if ( IS_NPC(ch)
    &&   ch->desc
    &&   ch->desc->original )
	return ch->desc->original->name;
    else
	return ch->name;
}

/* enchanted stuff for eq */
void affect_enchant(OBJ_DATA *obj)
{
    /* okay, move all the old flags into new vectors if we have to */
    if (!obj->enchanted)
    {
        AFFECT_DATA *paf, *af_new;
        obj->enchanted = TRUE;

        for (paf = obj->pIndexData->affected;
             paf != NULL; paf = paf->next)
        {
	    af_new = new_affect();

            af_new->next = obj->affected;
            obj->affected = af_new;
 
	    af_new->where	= paf->where;
            af_new->type        = UMAX(0,paf->type);
            af_new->level       = paf->level;
            af_new->duration    = paf->duration;
            af_new->location    = paf->location;
            af_new->modifier    = paf->modifier;
            af_new->bitvector   = paf->bitvector;
        }
    }
}
           

/*
 * Apply or remove an affect to a character.
 */
void affect_modify( CHAR_DATA *ch, AFFECT_DATA *paf, bool fAdd )
{
    OBJ_DATA *wield;
    int mod,i;

    mod = paf->modifier;

    if ( fAdd )
    {
	switch (paf->where)
	{
	case TO_AFFECTS:
	    SET_BIT(ch->affected_by, paf->bitvector);
	    break;
	case TO_AFFECTS2:
	    SET_BIT(ch->affected_by2, paf->bitvector);
	    break;
	case TO_IMMUNE:
            CHAR_SET_IMM(ch, paf->bitvector);
	    break;
	case TO_RESIST:
	    CHAR_SET_RES(ch,paf->bitvector);
	    break;
	case TO_VULN:
	    CHAR_SET_VULN(ch,paf->bitvector);
	    break;
        case TO_SKILL:
            if ( !IS_NPC(ch) )
                ch->pcdata->skill_mods[paf->location] += mod;
            return;
        case TO_POLYMORPH:
            ch->polymorphed = get_mob_index( paf->modifier );
            break;
	}
	
	if ( paf->appearance[0] != '\0'
	&&   (ch->appearance[0] == '\0'
	||    AFFECT_PRIORITY(paf->flags) > ch->priority) )
	{
	    free_string( ch->appearance );
	    ch->appearance = str_dup( paf->appearance );
	    
	    ch->priority = AFFECT_PRIORITY( paf->flags );
	}
    }
    else
    {
        switch (paf->where)
        {
        case TO_AFFECTS:
            REMOVE_BIT(ch->affected_by, paf->bitvector);
            break;
        case TO_AFFECTS2:
            REMOVE_BIT(ch->affected_by2, paf->bitvector);
            break;
        case TO_IMMUNE:
            CHAR_REM_IMM(ch,paf->bitvector);
            break;
        case TO_RESIST:
            CHAR_REM_RES(ch,paf->bitvector);
            break;
        case TO_VULN:
            CHAR_REM_VULN(ch,paf->bitvector);
            break;
        case TO_SKILL:
            if ( !IS_NPC(ch) )
                ch->pcdata->skill_mods[paf->location] -= mod;
            return;
        case TO_POLYMORPH:
            ch->polymorphed = NULL;
        }
        
	mod = 0 - mod;
	
	if ( !strcmp(paf->appearance, ch->appearance) )
	{
	    free_string( ch->appearance );
	    ch->appearance = str_dup( "" );
	    
	    ch->priority = 0;
	}
    }

    switch ( paf->location )
    {
    default:
	bug( "Affect_modify: unknown location %d.", paf->location );
	return;

    case APPLY_NONE:						break;
    case APPLY_STR:           ch->mod_stat[STAT_STR]	+= mod;	break;
    case APPLY_DEX:           ch->mod_stat[STAT_DEX]	+= mod;	break;
    case APPLY_INT:           ch->mod_stat[STAT_INT]	+= mod;	break;
    case APPLY_WIS:           ch->mod_stat[STAT_WIS]	+= mod;	break;
    case APPLY_CON:           ch->mod_stat[STAT_CON]	+= mod;	break;
    case APPLY_PER:           ch->mod_stat[STAT_PER]	+= mod;	break;
    case APPLY_CHA:           ch->mod_stat[STAT_CHA]	+= mod;	break;
    case APPLY_SEX:           ch->sex			+= mod;	break;
    case APPLY_CLASS:						break;
    case APPLY_LEVEL:						break;
    case APPLY_AGE:						break;
    case APPLY_HEIGHT:						break;
    case APPLY_WEIGHT:						break;
    case APPLY_MANA:          ch->max_mana		+= mod;	break;
    case APPLY_HIT:           ch->max_hit		+= mod;	break;
    case APPLY_MOVE:          ch->max_move		+= mod;	break;
    case APPLY_GOLD:						break;
    case APPLY_EXP:						break;
    case APPLY_AC:
        for (i = 0; i < 4; i ++)
            ch->armor[i] += mod;
        break;
    case APPLY_HITROLL:       ch->hitroll		+= mod;	break;
    case APPLY_DAMROLL:       ch->damroll		+= mod;	break;
    case APPLY_SAVES:         ch->saving_throw          += mod; break;
    case APPLY_SAVING_ROD:    ch->saving_throw		+= mod;	break;
    case APPLY_SAVING_PETRI:  ch->saving_throw		+= mod;	break;
    case APPLY_SAVING_BREATH: ch->saving_throw		+= mod;	break;
    case APPLY_SAVING_SPELL:  ch->saving_throw		+= mod;	break;
    case APPLY_SPELL_AFFECT:  					break;
    }

    /*
     * Check for weapon wielding.
     * Guard against recursion (for weapons with affects).
     */
    if ( !IS_NPC(ch) && ( wield = get_eq_char( ch, WEAR_WIELD ) ) != NULL
    &&   get_obj_weight(wield) > 
         (str_app[get_curr_stat(ch,STAT_STR)/4].wield*10))
    {
	static int depth;

	if ( depth == 0 )
	{
	    depth++;
	    act( "You drop $p.", ch, wield, NULL, TO_CHAR );
	    act( "$n drops $p.", ch, wield, NULL, TO_ROOM );
	    obj_from_char( wield );
	    obj_to_room( wield, ch->in_room );
	    depth--;
	}
    }

    if ( !IS_NPC(ch) && ( wield = get_eq_char( ch, WEAR_SECONDARY ) ) != NULL
    &&   get_obj_weight(wield) > 
         (str_app[get_curr_stat(ch,STAT_STR)/4].wield*5))
    {
	static int depth;

	if ( depth == 0 )
	{
	    depth++;
	    act( "You drop $p.", ch, wield, NULL, TO_CHAR );
	    act( "$n drops $p.", ch, wield, NULL, TO_ROOM );
	    obj_from_char( wield );
	    obj_to_room( wield, ch->in_room );
	    depth--;
	}
    }

    return;
}


/* find an effect in an affect list */
AFFECT_DATA  *affect_find(AFFECT_DATA *paf, int sn)
{
    AFFECT_DATA *paf_find;
    
    for ( paf_find = paf; paf_find != NULL; paf_find = paf_find->next )
    {
        if ( paf_find->type == sn )
	return paf_find;
    }

    return NULL;
}

/* fix object affects when removing one */
void affect_check(CHAR_DATA *ch,int where,int vector)
{
    AFFECT_DATA *paf;
    OBJ_DATA *obj;
    bool found;

    if ( where == TO_OBJECT
    ||   where == TO_WEAPON
    ||   where == TO_SKILL
    ||   vector == 0 )
	return;

    found = FALSE;

    for ( paf = ch->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->appearance[0] != '\0'
	&&   (ch->appearance[0] == '\0'
	||    AFFECT_PRIORITY(paf->flags) > ch->priority) )
	{
	    free_string( ch->appearance );
	    ch->appearance = str_dup( paf->appearance );
	    
	    ch->priority = AFFECT_PRIORITY( paf->flags );
	}

	if ( paf->where == where && paf->bitvector == vector )
	{
	    switch (where)
	    {
	        case TO_AFFECTS:
		    SET_BIT(ch->affected_by,vector);
		    break;
                case TO_AFFECTS2:
                    SET_BIT(ch->affected_by2, vector);
                    break;
	        case TO_IMMUNE:
		    CHAR_SET_IMM(ch,vector);   
		    break;
	        case TO_RESIST:
		    CHAR_SET_RES(ch,vector);
		    break;
	        case TO_VULN:
		    CHAR_SET_VULN(ch,vector);
		    break;
	    }
	    
	    found = TRUE;
	}
    }

    if ( found )
	return;

    for (obj = ch->carrying; obj != NULL; obj = obj->next_content)
    {
	if (obj->wear_loc == -1 || IS_OBJ_STAT(obj, ITEM_DISPELLED))
	    continue;

            for (paf = obj->affected; paf != NULL; paf = paf->next)
            if (paf->where == where && paf->bitvector == vector)
            {
                switch (where)
                {
                    case TO_AFFECTS:
                        SET_BIT(ch->affected_by,vector);
                        break;
                    case TO_AFFECTS2:
                        SET_BIT(ch->affected_by2,vector);
                        break;
                    case TO_IMMUNE:
                        CHAR_SET_IMM(ch,vector);
                        break;
                    case TO_RESIST:
                        CHAR_SET_RES(ch,vector);
                        break;
                    case TO_VULN:
                        CHAR_SET_VULN(ch,vector);
                  
                }
                return;
            }

        if (obj->enchanted)
	    continue;

        for (paf = obj->pIndexData->affected; paf != NULL; paf = paf->next)
            if (paf->where == where && paf->bitvector == vector)
            {
                switch (where)
                {
                    case TO_AFFECTS:
                        SET_BIT(ch->affected_by,vector);
                        break;
                    case TO_AFFECTS2:
                        SET_BIT(ch->affected_by2,vector);
                        break;
                    case TO_IMMUNE:
                        CHAR_SET_IMM(ch,vector);
                        break;
                    case TO_RESIST:
                        CHAR_SET_RES(ch,vector);
                        break;
                    case TO_VULN:
                        CHAR_SET_VULN(ch,vector);
                        break;
                }
                return;
            }
    }
}

/*
 * Give an affect to a char.
 */
void affect_to_char( CHAR_DATA *ch, AFFECT_DATA *paf )
{
    AFFECT_DATA *paf_new;

    paf_new = new_affect();

    *paf_new		= *paf;
    paf_new->name	= str_dup( paf->name );
    paf_new->appearance	= str_dup( paf->appearance );

    paf_new->next	= ch->affected;
    ch->affected	= paf_new;

    affect_modify( ch, paf_new, TRUE );

    return;
}


void affect_to_char_after( CHAR_DATA *ch, AFFECT_DATA *paf, AFFECT_DATA *after )
{
    AFFECT_DATA *paf_new;
    
    paf_new = new_affect();
    
    *paf_new		= *paf;
    paf_new->name	= str_dup( paf->name );
    paf_new->appearance	= str_dup( paf->appearance );

    paf_new->next	= after->next;
    after->next		= paf_new;

    affect_modify( ch, paf_new, TRUE );

    return;
}


/* give an affect to an object */
void affect_to_obj(OBJ_DATA *obj, AFFECT_DATA *paf)
{
    AFFECT_DATA *paf_new;

    paf_new = new_affect();

    *paf_new		= *paf;
    paf_new->name	= str_dup( "" );
    paf_new->appearance = str_dup( paf->appearance );

    paf_new->next	= obj->affected;
    obj->affected	= paf_new;

    /* apply any affect vectors to the object's extra_flags */
    if (paf->bitvector)
        switch (paf->where)
        {
        case TO_OBJECT:
    	    SET_BIT(obj->extra_flags,paf->bitvector);
            if ( obj->item_type == ITEM_LIGHT
            &&   obj->wear_loc == WEAR_LIGHT
            &&   obj->carried_by->in_room != NULL
            &&   obj->carried_by->in_room->light )
                obj->carried_by->in_room->light--;
	    break;
        case TO_WEAPON:
	    if (obj->item_type == ITEM_WEAPON)
	        SET_BIT(obj->value[4],paf->bitvector);
	    break;
        }
    

    return;
}


/*
 * Remove an affect from a char.
 */
void affect_remove( CHAR_DATA *ch, AFFECT_DATA *paf )
{
    int where;
    int vector;

    if ( ch->affected == NULL )
    {
	bug( "Affect_remove: no affect.", 0 );
	return;
    }

    affect_modify( ch, paf, FALSE );
    where = paf->where;
    vector = paf->bitvector;

    if ( paf == ch->affected )
    {
	ch->affected	= paf->next;
    }
    else
    {
	AFFECT_DATA *prev;

	for ( prev = ch->affected; prev != NULL; prev = prev->next )
	{
	    if ( prev->next == paf )
	    {
		prev->next = paf->next;
		break;
	    }
	}

	if ( prev == NULL )
	{
	    bug( "Affect_remove: cannot find paf.", 0 );
	    return;
	}
    }

    if ( IS_SET(paf->flags, AFF_PROC_FINISHED) )
        affect_procs[paf->procNo].fn( ch, NULL, paf, AFF_PROC_FINISHED );

    free_affect(paf);

    affect_check(ch,where,vector);
    return;
}

void affect_remove_obj( OBJ_DATA *obj, AFFECT_DATA *paf)
{
    int where, vector;
    if ( obj->affected == NULL )
    {
        bug( "Affect_remove_object: no affect.", 0 );
        return;
    }

    if ( obj->carried_by != NULL
    &&   obj->wear_loc != -1 
    &&   !IS_OBJ_STAT(obj, ITEM_DISPELLED) )
	affect_modify( obj->carried_by, paf, FALSE );

    where = paf->where;
    vector = paf->bitvector;

    /* remove flags from the object if needed */
    if (paf->bitvector)
	switch( paf->where)
        {
        case TO_OBJECT:
            REMOVE_BIT(obj->extra_flags,paf->bitvector);
            if ( obj->item_type == ITEM_LIGHT
            &&   obj->wear_loc == WEAR_LIGHT
            &&   obj->carried_by->in_room != NULL )
                obj->carried_by->in_room->light++;
            break;
        case TO_WEAPON:
            if (obj->item_type == ITEM_WEAPON)
                REMOVE_BIT(obj->value[4],paf->bitvector);
            break;
        }

    if ( paf == obj->affected )
    {
        obj->affected    = paf->next;
    }
    else
    {
        AFFECT_DATA *prev;

        for ( prev = obj->affected; prev != NULL; prev = prev->next )
        {
            if ( prev->next == paf )
            {
                prev->next = paf->next;
                break;
            }
        }

        if ( prev == NULL )
        {
            bug( "Affect_remove_object: cannot find paf.", 0 );
            return;
        }
    }

    if ( where == TO_OBJECT
    &&   vector == ITEM_DISPELLED
    &&   obj->carried_by != NULL
    &&   obj->wear_loc != WEAR_NONE )
    {
	AFFECT_DATA *lpaf;
	if ( !obj->enchanted )
	    for ( lpaf = obj->pIndexData->affected; lpaf != NULL; lpaf = lpaf->next )
		if ( lpaf->location != APPLY_SPELL_AFFECT )
		    affect_modify( obj->carried_by, lpaf, TRUE );
	for ( lpaf = obj->affected; lpaf != NULL; lpaf = lpaf->next )
	    if ( lpaf->location == APPLY_SPELL_AFFECT )
    		affect_to_char ( obj->carried_by, lpaf );
	    else
		affect_modify( obj->carried_by, lpaf, TRUE );
    }

    if ( IS_SET(paf->flags, AFF_PROC_FINISHED) )
        affect_procs[paf->procNo].fn( NULL, obj, paf, AFF_PROC_FINISHED );

    free_affect(paf);

    if (obj->carried_by != NULL && obj->wear_loc != -1)
	affect_check( obj->carried_by, where, vector );
    return;
}



/*
 * Strip all affects of a given sn.
 */
void affect_strip( CHAR_DATA *ch, int sn )
{
    AFFECT_DATA *paf;
    AFFECT_DATA *paf_next;

    for ( paf = ch->affected; paf != NULL; paf = paf_next )
    {
	paf_next = paf->next;
	if ( paf->type == sn )
	    affect_remove( ch, paf );
    }

    return;
}



/*
 * Return true if a char is affected by a spell.
 */
bool is_affected( CHAR_DATA *ch, int sn )
{
    AFFECT_DATA *paf;

    for ( paf = ch->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->type == sn )
	    return TRUE;
    }

    return FALSE;
}



/*
 * Add or enhance an affect.
 */
void affect_join( CHAR_DATA *ch, AFFECT_DATA *paf )
{
    AFFECT_DATA *paf_old;
    bool found;

    found = FALSE;
    for ( paf_old = ch->affected; paf_old != NULL; paf_old = paf_old->next )
    {
	if ( paf_old->type == paf->type && paf_old->location == paf->location )
	{
	    paf->level = (paf->level += paf_old->level) / 2;
	    paf->duration += paf_old->duration;
	    paf->modifier += paf_old->modifier;
	    affect_remove( ch, paf_old );
	    break;
	}
    }

    affect_to_char( ch, paf );
    return;
}



/*
 * Move a char out of a room.
 */
void char_from_room( CHAR_DATA *ch, bool unmount )
{
    OBJ_DATA *obj;

    if ( ch->in_room == NULL )
    {
	bug( "Char_from_room: NULL.", 0 );
	return;
    }

    if ( !IS_NPC(ch) )
	--ch->in_room->area->nplayer;
	
    if ( unmount )
    {
	if ( ch->riding != NULL )
	{
	    ch->riding->mounted_by	= NULL;
	    ch->riding			= NULL;
	}
	else if ( ch->mounted_by != NULL )
	{
	    ch->mounted_by->riding	= NULL;
	    ch->mounted_by		= NULL;
	}
    }

    if ( ( obj = get_eq_char( ch, WEAR_HOLD /*WEAR_LIGHT*/ ) ) != NULL
    &&   obj->item_type == ITEM_LIGHT
    &&   obj->value[2] != 0
    &&   ch->in_room->light > 0
    &&   !IS_OBJ_STAT(obj, ITEM_EXTINGUISHED) )
	--ch->in_room->light;

    if ( ch == ch->in_room->people )
    {
	ch->in_room->people = ch->next_in_room;
    }
    else
    {
	CHAR_DATA *prev;

	for ( prev = ch->in_room->people; prev; prev = prev->next_in_room )
	{
	    if ( prev->next_in_room == ch )
	    {
		prev->next_in_room = ch->next_in_room;
		break;
	    }
	}

	if ( prev == NULL )
	    bug( "Char_from_room: ch not found.", 0 );
    }

    ch->in_room      = NULL;
    ch->next_in_room = NULL;
    ch->on 	     = NULL;  /* sanity check! */
    return;
}



/*
 * Move a char into a room.
 */
void char_to_room( CHAR_DATA *ch, ROOM_INDEX_DATA *pRoomIndex )
{
    char buf[MAX_STRING_LENGTH];
    OBJ_DATA *obj;

    if ( pRoomIndex == NULL )
    {
	ROOM_INDEX_DATA *room;

	bug( "Char_to_room: NULL.", 0 );
	
	if ((room = get_room_index(ROOM_VNUM_TEMPLE)) != NULL)
	    char_to_room(ch,room);
	
	return;
    }

    ch->in_room		= pRoomIndex;
    ch->next_in_room	= pRoomIndex->people;
    pRoomIndex->people	= ch;

    if ( !IS_NPC(ch) )
    {
	if (ch->in_room->area->empty)
	{
	    ch->in_room->area->empty = FALSE;
	    ch->in_room->area->age = 0;
	}
	++ch->in_room->area->nplayer;
    }

    if ( ( obj = get_eq_char( ch, WEAR_HOLD /*WEAR_LIGHT*/ ) ) != NULL
    &&   obj->item_type == ITEM_LIGHT
    &&   obj->value[2] != 0
    &&   !IS_OBJ_STAT(obj, ITEM_EXTINGUISHED) )
	++ch->in_room->light;
	
    if (IS_AFFECTED(ch,AFF_PLAGUE))
    {
        AFFECT_DATA *af, plague, plague_carrier;
        CHAR_DATA *vch;
        
        for ( af = ch->affected; af != NULL; af = af->next )
        {
            if (af->type == gsn_plague)
                break;
        }
        
        if (af == NULL)
        {
            REMOVE_BIT(ch->affected_by,AFF_PLAGUE);
            return;
        }
        
        if (af->level == 1)
            return;
        
	plague.where		= TO_AFFECTS;
	plague.type		= gsn_plague;
	plague.level		= af->level - 1; 
	plague.duration 	= number_range(1,2 * plague.level);
	plague.bitvector 	= AFF_PLAGUE;
	plague.location		= APPLY_STR;
	plague.modifier 	= -20;
	plague.procNo		= affect_proc_lookup("plague");
	plague.flags		= AFF_CONT_UPDATE|AFF_INVIS|AFF_PRI_MEDIUM;
	plague.name		= "";

	plague_carrier.where		= TO_AFFECTS;
	plague_carrier.type		= gsn_plague;
	plague_carrier.level		= af->level / 2; 
	plague_carrier.duration		= number_range( 1,
	                                      2 * plague_carrier.level );
	plague_carrier.bitvector	= AFF_PLAGUE;
	plague_carrier.location		= APPLY_NONE;
	plague_carrier.modifier		= 0;
	plague_carrier.procNo		= 0;
	plague_carrier.flags		= AFF_INVIS;
	plague_carrier.name		= "";
	plague_carrier.appearance	= "";
        
        for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
        {
            if (!saves_spell(plague.level - 2,vch,DAM_DISEASE) 
	    &&  !IS_IMMORTAL(vch) &&
            	!IS_AFFECTED(vch,AFF_PLAGUE) && number_bits(6) == 0)
            {
		if ( number_bits(2) == 0 )
		{
		    /* Victim is merely an unaware carrier! */
		    affect_join(vch,&plague_carrier);
		}
		else
		{
		    if ( IS_NPC(vch) )
			plague.appearance = "";
		    else
			plague.appearance =
			    "%n %p, a fevered sweat covering %s body.";
		    send_to_char("You feel hot and feverish.\n\r",vch);
		    act("$n shivers and looks very ill.",vch,NULL,NULL,TO_ROOM);
		    affect_join(vch,&plague);
		}
            }
        }
    }

    if ( !IS_NULLSTR(pRoomIndex->area->sound_track)
    &&   !IS_SET(pRoomIndex->room_flags, ROOM_NO_AMUSIC)
    &&   IS_SET(ch->act, PLR_MUSIC) )
    {
        sprintf( buf, "!!MUSIC(%s C=1 T=music)\n\r",
            pRoomIndex->area->sound_track );
        send_to_char( buf, ch );
    }
    else if ( IS_SET(ch->act, PLR_MUSIC)
    &&   (IS_NULLSTR(pRoomIndex->area->sound_track)
    ||    IS_SET(pRoomIndex->room_flags, ROOM_NO_AMUSIC)) )
    {
        sprintf( buf, "!!MUSIC(off)\n\r" );
        send_to_char( buf, ch );
    }

    return;
}


/*
 * Check to see if an object will fit a player.
 */
bool bad_fit( OBJ_DATA *obj, CHAR_DATA *ch )
{
    if ( obj->pIndexData->fit_flags == 0 )
	return FALSE;

    if ( IS_SET(race_table[CHAR_RACE(ch)].char_flags, obj->pIndexData->fit_flags) )
	return TRUE;

    switch ( CHAR_SIZE(ch) )
    {
	case SIZE_TINY:
	    return IS_SET(obj->pIndexData->fit_flags, FIT_NO_TINY);
	case SIZE_SMALL:
	    return IS_SET(obj->pIndexData->fit_flags, FIT_NO_SMALL);
	case SIZE_MEDIUM:
	    return IS_SET(obj->pIndexData->fit_flags, FIT_NO_MEDIUM);
	case SIZE_LARGE:
	    return IS_SET(obj->pIndexData->fit_flags, FIT_NO_LARGE);
	case SIZE_HUGE:
	    return IS_SET(obj->pIndexData->fit_flags, FIT_NO_HUGE);
	case SIZE_GIANT:
	    return IS_SET(obj->pIndexData->fit_flags, FIT_NO_GIANT);
	default:
	    bug( "Bad_fit: invalid character size.", 0 );
	    return FALSE;
    }
}


/*
 * Check if a char matches a set of restriction/aggression flags.
 */
bool char_matches( CHAR_DATA *ch, int flags )
{
    if ( (ch->sex == 1 && IS_SET(flags, CHAR_MALE))
    ||   (ch->sex == 2 && IS_SET(flags, CHAR_FEMALE))
    ||   (ch->sex == 3 && IS_SET(flags, CHAR_EUNICH)) )
	return TRUE;

    if ( !IS_AFFECTED2(ch, AFF2_DELUDE)
    &&   ((IS_GOOD(ch) && IS_SET(flags, CHAR_GOOD))
    ||    (IS_EVIL(ch) && IS_SET(flags, CHAR_EVIL))
    ||    (IS_NEUTRAL(ch) && IS_SET(flags, CHAR_NEUTRAL))) )
	return TRUE;

    if ( IS_SET(race_table[CHAR_RACE(ch)].char_flags, flags) )
	return TRUE;
    
    if ( IS_NPC(ch) )
    {
	if ( (IS_SET(ch->act, ACT_MAGE) && IS_SET(flags, CHAR_CAST_MAGE))
	||   (IS_SET(ch->act, ACT_CLERIC) && IS_SET(flags, CHAR_CAST_CLERIC)) )
	    return TRUE;
    }
    else
    {
	if ( (IS_SET(class_table[ch->class].flags, CLASS_NEEDS_SYMBOL)
	&&    IS_SET(flags, CHAR_CAST_CLERIC))
	||   (!IS_SET(class_table[ch->class].flags, CLASS_NEEDS_SYMBOL)
	&&    IS_SET(flags, CHAR_CAST_MAGE)) )
	    return TRUE;
		
	if ( (IS_SET(flags, CHAR_PURE_CASTER)
	&&    class_table[ch->class].pMana >= PURE_MAGIC)
	||   (IS_SET(flags, CHAR_NON_CASTER)
	&&    class_table[ch->class].pMana < NON_MAGIC)
	||   (IS_SET(flags, CHAR_SEMI_CASTER)
	&&    class_table[ch->class].pMana < PURE_MAGIC
	&&    class_table[ch->class].pMana >= NON_MAGIC) )
	    return TRUE;
	
	if ( (IS_SET(flags, CHAR_NON_RULER)
	&&    ch->pcdata->social_classes[ch->in_room->area->territory] != MAX_SOCIAL_CLASS - 1)
	||   (IS_SET(flags, CHAR_NOBLE)
	&&    ch->pcdata->social_classes[ch->in_room->area->territory] >= 7)
	||   (IS_SET(flags, CHAR_PEASANT)
	&&    ch->pcdata->social_classes[ch->in_room->area->territory] < 5) )
	    return TRUE;
    }
    
    return FALSE;
}


/*
 * Give an obj to a char.
 */
void obj_to_char( OBJ_DATA *obj, CHAR_DATA *ch )
{
    obj->next_content	 = ch->carrying;
    ch->carrying	 = obj;
    obj->carried_by	 = ch;
    obj->in_room	 = NULL;
    obj->in_obj		 = NULL;
    ch->carry_number	+= get_obj_number( obj );
    ch->carry_weight	+= get_obj_weight( obj );
}



/*
 * Take an obj from its character.
 */
void obj_from_char( OBJ_DATA *obj )
{
    CHAR_DATA *ch;

    if ( ( ch = obj->carried_by ) == NULL )
    {
	bug( "Obj_from_char: null ch.", 0 );
	return;
    }

    if ( obj->wear_loc != WEAR_NONE )
	unequip_char( ch, obj );

    if ( ch->carrying == obj )
    {
	ch->carrying = obj->next_content;
    }
    else
    {
	OBJ_DATA *prev;

	for ( prev = ch->carrying; prev != NULL; prev = prev->next_content )
	{
	    if ( prev->next_content == obj )
	    {
		prev->next_content = obj->next_content;
		break;
	    }
	}

	if ( prev == NULL )
	    bug( "Obj_from_char: obj not in list.", 0 );
    }

    obj->carried_by	 = NULL;
    obj->next_content	 = NULL;
    ch->carry_number	-= get_obj_number( obj );
    ch->carry_weight	-= get_obj_weight( obj );
    return;
}



/*
 * Find the ac value of an obj, including position effect.
 */
int apply_ac( OBJ_DATA *obj, int iWear, int type )
{
    if ( obj->item_type != ITEM_ARMOR )
	return 0;

    switch ( iWear )
    {
    case WEAR_BODY:	return 3 * obj->value[type];
    case WEAR_HEAD:	return 2 * obj->value[type];
    case WEAR_LEGS:	return 2 * obj->value[type];
    case WEAR_FEET:	return     obj->value[type];
    case WEAR_HANDS:	return     obj->value[type];
    case WEAR_ARMS:	return     obj->value[type];
    case WEAR_SHIELD:	return     obj->value[type];
    case WEAR_NECK_1:	return     obj->value[type];
    case WEAR_NECK_2:	return     obj->value[type];
    case WEAR_ABOUT:	return 2 * obj->value[type];
    case WEAR_WAIST:	return     obj->value[type];
    case WEAR_WRIST_L:	return     obj->value[type];
    case WEAR_WRIST_R:	return     obj->value[type];
    case WEAR_HOLD:	return     obj->value[type];
    }

    return 0;
}



/*
 * Find a piece of eq on a character.
 */
OBJ_DATA *get_eq_char( CHAR_DATA *ch, int iWear )
{
    OBJ_DATA *obj;

    if (ch == NULL)
	return NULL;

    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc == iWear )
	    return obj;
    }

    return NULL;
}



/*
 * Equip a char with an obj.
 */
void equip_char( CHAR_DATA *ch, OBJ_DATA *obj, int iWear )
{
    AFFECT_DATA *paf;
    int i;

    if ( get_eq_char(ch, iWear) != NULL )
    {
	bug( "Equip_char: already equipped (%d).", iWear );
	return;
    }
    
    if ( bad_fit(obj, ch) )
    {
	send_to_char( "You can't seem to fit it on.", ch );
	act( "$n can't seem to fit $p on.",
	    ch, obj, NULL, TO_ROOM );
	return;
    }

    if ( char_matches(ch, obj->pIndexData->restrictions) )
    {
	if ( !IS_SET(obj->pIndexData->restrictions, RESTRICT_WASTING)
	&&   !IS_SET(obj->pIndexData->restrictions, RESTRICT_CORRUPTING) )
	{
	    /*
	     * Thanks to Morgenes for the bug fix here!
	     */
	    if ( IS_SET(obj->pIndexData->restrictions, RESTRICT_HARMFUL) )
	    {
		act( "You are burnt by $p and drop it!", 
		    ch, obj, NULL, TO_CHAR );
		act( "$n screams as $e is burnt by $p.",
		    ch, obj, NULL, TO_ROOM );

	        obj_from_char( obj );
	        obj_to_room( obj, ch->in_room );

		damage( ch, ch, number_range(1, obj->level), 
		    0, DAM_FIRE, FALSE, NULL );
	    }
	    else
	    {
		act( "You are zapped by $p and drop it.",
		    ch, obj, NULL, TO_CHAR );
		act( "$n is zapped by $p and drops it.",
		    ch, obj, NULL, TO_ROOM );

	        obj_from_char( obj );
	        obj_to_room( obj, ch->in_room );
	    }
	    
	    return;
	}
    }
    
    if ( !IS_NPC(ch)
    &&   obj->pIndexData->area != NULL
    &&   IS_SET(obj->pIndexData->area->flags, AREA_VIRTUAL)
    &&   ch->level < LEVEL_IMMORTAL )
    {
	act( "$p is too insubstantial to wear.", ch, obj, NULL, TO_CHAR );
	return;
    }

    for (i = 0; i < 4; i++)
    	ch->armor[i]	-= apply_ac( obj, iWear,i );
    obj->wear_loc	 = iWear;

    if ( !IS_OBJ_STAT(obj, ITEM_DISPELLED) )
    {
	if (!obj->enchanted)
	    for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
		if ( paf->location != APPLY_SPELL_AFFECT )
		    affect_modify( ch, paf, TRUE );
	for ( paf = obj->affected; paf != NULL; paf = paf->next )
	    if ( paf->location == APPLY_SPELL_AFFECT )
    		affect_to_char ( ch, paf );
	    else
		affect_modify( ch, paf, TRUE );
    }

    if ( obj->item_type == ITEM_LIGHT
    &&   obj->value[2] != 0
    &&   ch->in_room != NULL
    &&   !IS_OBJ_STAT(obj, ITEM_EXTINGUISHED) )
	++ch->in_room->light;

    if ( obj->item_type == ITEM_ARMOR )
	ch->armor_weight += obj->weight;

    return;
}



/*
 * Unequip a char with an obj.
 */
void unequip_char( CHAR_DATA *ch, OBJ_DATA *obj )
{
    AFFECT_DATA *paf = NULL;
    AFFECT_DATA *lpaf = NULL;
    AFFECT_DATA *lpaf_next = NULL;
    int i;

    if ( obj->wear_loc == WEAR_NONE )
    {
	bug( "Unequip_char: already unequipped.", 0 );
	return;
    }

    for (i = 0; i < 4; i++)
    	ch->armor[i]	+= apply_ac( obj, obj->wear_loc,i );
    obj->wear_loc	 = -1;

    if ( !IS_OBJ_STAT(obj, ITEM_DISPELLED) )
    {
    if (!obj->enchanted)
    {
	for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
        {
	    if ( paf->location == APPLY_SPELL_AFFECT )
	    {
	        for ( lpaf = ch->affected; lpaf != NULL; lpaf = lpaf_next )
	        {
		    lpaf_next = lpaf->next;
		    if ((lpaf->type == paf->type) &&
		        (lpaf->level == paf->level) &&
		        (lpaf->location == APPLY_SPELL_AFFECT))
		    {
		        affect_remove( ch, lpaf );
			lpaf_next = NULL;
		    }
	        }
	    }
	    else
	    {
	        affect_modify( ch, paf, FALSE );
		affect_check(ch,paf->where,paf->bitvector);
	    }
        }
    }

    for ( paf = obj->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->location == APPLY_SPELL_AFFECT )
	{
	    bug ( "Norm-Apply: %d", 0 );
	    for ( lpaf = ch->affected; lpaf != NULL; lpaf = lpaf_next )
	    {
		lpaf_next = lpaf->next;
		if ((lpaf->type == paf->type) &&
		    (lpaf->level == paf->level) &&
		    (lpaf->location == APPLY_SPELL_AFFECT))
		{
		    bug ( "location = %d", lpaf->location );
		    bug ( "type = %d", lpaf->type );
		    affect_remove( ch, lpaf );
		    lpaf_next = NULL;
		}
	    }
	}
	else
	{
	    affect_modify( ch, paf, FALSE );
	    affect_check(ch,paf->where,paf->bitvector);	
	}
    }
    }

    if ( obj->item_type == ITEM_LIGHT
    &&   obj->value[2] != 0
    &&   ch->in_room != NULL
    &&   ch->in_room->light > 0
    &&   !IS_OBJ_STAT(obj, ITEM_EXTINGUISHED) )
	--ch->in_room->light;

    if ( obj->item_type == ITEM_ARMOR )
	ch->armor_weight -= obj->weight;

    return;
}



/*
 * Count occurrences of an obj in a list.
 */
int count_obj_list( OBJ_INDEX_DATA *pObjIndex, OBJ_DATA *list )
{
    OBJ_DATA *obj;
    int nMatch;

    nMatch = 0;
    for ( obj = list; obj != NULL; obj = obj->next_content )
    {
	if ( obj->pIndexData == pObjIndex )
	    nMatch++;
    }

    return nMatch;
}



/*
 * Move an obj out of a room.
 */
void obj_from_room( OBJ_DATA *obj )
{
    ROOM_INDEX_DATA *in_room;
    CHAR_DATA *ch;

    if ( ( in_room = obj->in_room ) == NULL )
    {
	bug( "obj_from_room: NULL.", 0 );
	return;
    }

    for (ch = in_room->people; ch != NULL; ch = ch->next_in_room)
	if (ch->on == obj)
	    ch->on = NULL;

    if ( obj == in_room->contents )
    {
	in_room->contents = obj->next_content;
    }
    else
    {
	OBJ_DATA *prev;

	for ( prev = in_room->contents; prev; prev = prev->next_content )
	{
	    if ( prev->next_content == obj )
	    {
		prev->next_content = obj->next_content;
		break;
	    }
	}

	if ( prev == NULL )
	{
	    bug( "Obj_from_room: obj not found.", 0 );
	    return;
	}
    }

    obj->in_room      = NULL;
    obj->next_content = NULL;
    return;
}



/*
 * Move an obj into a room.
 */
void obj_to_room( OBJ_DATA *obj, ROOM_INDEX_DATA *pRoomIndex )
{
    obj->next_content		= pRoomIndex->contents;
    pRoomIndex->contents	= obj;
    obj->in_room		= pRoomIndex;
    obj->carried_by		= NULL;
    obj->in_obj			= NULL;
    return;
}



/*
 * Move an object into an object.
 */
void obj_to_obj( OBJ_DATA *obj, OBJ_DATA *obj_to )
{
    obj->next_content		= obj_to->contains;
    obj_to->contains		= obj;
    obj->in_obj			= obj_to;
    obj->in_room		= NULL;
    obj->carried_by		= NULL;
    if (obj_to->pIndexData->vnum == OBJ_VNUM_PIT)
        obj->cost = 0; 

    for ( ; obj_to != NULL; obj_to = obj_to->in_obj )
    {
	if ( obj_to->carried_by != NULL )
	{
	    obj_to->carried_by->carry_number += get_obj_number( obj );
	    obj_to->carried_by->carry_weight += get_obj_weight( obj )
		* WEIGHT_MULT(obj_to) / 100;
	}
    }

    return;
}



/*
 * Move an object out of an object.
 */
void obj_from_obj( OBJ_DATA *obj )
{
    OBJ_DATA *obj_from;

    if ( ( obj_from = obj->in_obj ) == NULL )
    {
	bug( "Obj_from_obj: null obj_from.", 0 );
	return;
    }

    if ( obj == obj_from->contains )
    {
	obj_from->contains = obj->next_content;
    }
    else
    {
	OBJ_DATA *prev;

	for ( prev = obj_from->contains; prev; prev = prev->next_content )
	{
	    if ( prev->next_content == obj )
	    {
		prev->next_content = obj->next_content;
		break;
	    }
	}

	if ( prev == NULL )
	{
	    bug( "Obj_from_obj: obj not found.", 0 );
	    return;
	}
    }

    obj->next_content = NULL;
    obj->in_obj       = NULL;

    for ( ; obj_from != NULL; obj_from = obj_from->in_obj )
    {
	if ( obj_from->carried_by != NULL )
	{
	    obj_from->carried_by->carry_number -= get_obj_number( obj );
	    obj_from->carried_by->carry_weight -= get_obj_weight( obj ) 
		* WEIGHT_MULT(obj_from) / 100;
	}
    }

    return;
}



/*
 * Extract an obj from the world.
 */
void extract_obj( OBJ_DATA *obj )
{
    OBJ_DATA *obj_content;
    OBJ_DATA *obj_next;

    if ( obj->in_room != NULL )
	obj_from_room( obj );
    else if ( obj->carried_by != NULL )
	obj_from_char( obj );
    else if ( obj->in_obj != NULL )
	obj_from_obj( obj );

    for ( obj_content = obj->contains; obj_content; obj_content = obj_next )
    {
	obj_next = obj_content->next_content;
	extract_obj( obj_content );
    }

    if ( obj->pIndexData->instances == obj )
	obj->pIndexData->instances = obj->next_instance;
    else
    {
	OBJ_DATA *current;
	
	for ( current = obj->pIndexData->instances; 
	      current != NULL; 
	      current = current->next_instance )
	{
	    if ( current->next_instance == NULL )
	    {
		bug( "Extract_obj: object not found in instance list.", 0 );
		break;
	    }

	    if ( current->next_instance == obj )
	    {
		current->next_instance = obj->next_instance;
		break;
	    }
	}
    }

    if ( object_list == obj )
    {
	object_list = obj->next;
    }
    else
    {
	OBJ_DATA *prev;

	for ( prev = object_list; prev != NULL; prev = prev->next )
	{
	    if ( prev->next == obj )
	    {
		prev->next = obj->next;
		break;
	    }
	}

	if ( prev == NULL )
	{
	    bug( "Extract_obj: obj %d not found.", obj->pIndexData->vnum );
	    return;
	}
    }

    if ( HAS_TRIGGER(obj, OTRIG_RANDOM) && periodic_object_checks != NULL )
    {
	if ( (OBJ_DATA *) (periodic_object_checks->data) == obj )
	{
	    PERIODIC_CHECK_DATA *tail = periodic_object_checks->next;
	    free_periodic_check( periodic_object_checks );
	    periodic_object_checks = tail;
	}
	else
	{
	    PERIODIC_CHECK_DATA *prev;

	    for ( prev = periodic_object_checks; prev != NULL; prev = prev->next )
	    {
		if ( (OBJ_DATA *) (prev->next->data) == obj )
		{
		    prev->next = prev->next->next;
		    break;
		}
	    }

	    if ( prev == NULL )
	    {
		bug( "Extract_obj: periodic check not found.", 0 );
		return;
	    }
	}
    }

    --obj->pIndexData->count;

    obj->next = extracted_objs;
    extracted_objs = obj;
    INVALIDATE( obj );

    return;
}



/*
 * Extract a char from the world.
 */
void extract_char( CHAR_DATA *ch, bool fPull )
{
    CHAR_DATA *wch;
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;

/*
 *  if ( ch->in_room == NULL )
 *  {
 *	bug( "Extract_char: NULL.", 0 );
 *	return;
 *  }
 */

    if ( ch == actor )
    {
	bug( "Extract_char: tried to extract actor!", 0 );
	return;
    }

    nuke_pets( ch );
    ch->pet = NULL; /* just in case */

    if ( fPull )
	die_follower( ch );
    
    stop_fighting( ch, TRUE );

    for ( obj = ch->carrying; obj != NULL; obj = obj_next )
    {
	obj_next = obj->next_content;
	extract_obj( obj );
    }

    if ( ch->in_room != NULL )
	char_from_room( ch, TRUE );

    /* Death room is set in the clan tabe now */
    if ( !fPull )
    {
	char_to_room( ch, get_room_index(clan_table[ch->clan].hall) );
	SET_BIT( ch->act, PLR_KILLED );
	return;
    }

    if ( IS_NPC(ch) )
    {
	CHAR_DATA *current;
	
	if ( ch->pIndexData->instances == ch )
	    ch->pIndexData->instances = ch->next_instance;
	else
	{
	    for ( current = ch->pIndexData->instances; 
	          current != NULL; 
	          current = current->next_instance )
	    {
		if ( current->next_instance == NULL )
		{
		    bug( "Extract_char: char not found in instance list.", 0 );
		    break;
		}

		if ( current->next_instance == ch )
		{
		    current->next_instance = ch->next_instance;
		    break;
		}
	    }
	}
	
	--ch->pIndexData->count;
    }

    if ( ch->desc != NULL && ch->desc->original != NULL )
    {
	do_return( ch, "" );
	ch->desc = NULL;
    }

    for ( wch = char_list; wch != NULL; wch = wch->next )
    {
	if ( wch->reply == ch )
	    wch->reply = NULL;

	if ( ch->mprog_target == wch )
	    wch->mprog_target = NULL;

	if ( !IS_NPC(ch) && wch->memory )
	{
	    MEM_DATA *memory;

	    for ( memory = wch->memory; memory != NULL; memory = memory->next )
	    {
		if ( memory->reaction == MEM_HOSTILE
		&&   memory->id == ch->id )
		{
		    remove_memory( wch, memory );
		    break;
		}
	    }
	}
    }

    if ( ch == char_list )
    {
       char_list = ch->next;
    }
    else
    {
	CHAR_DATA *prev;

	for ( prev = char_list; prev != NULL; prev = prev->next )
	{
	    if ( prev->next == ch )
	    {
		prev->next = ch->next;
		break;
	    }
	}

	if ( prev == NULL )
	{
	    bug( "Extract_char: char not found.", 0 );
	    return;
	}
    }

    if ( ch->desc != NULL )
	ch->desc->character = NULL;

    ch->next = extracted_chars;
    extracted_chars = ch;
    INVALIDATE( ch );
    
    return;
}



/*
 * Find a char in the room.
 */
CHAR_DATA *get_char_room( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *rch;
    int number;
    int count;

    number = number_argument( argument, arg );
    count  = 0;
    if ( !str_cmp( arg, "self" ) )
	return ch;
    for ( rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room )
    {
	if ( !can_see( ch, rch ) || !is_name( arg, CHAR_NAME(rch) ) )
	    continue;
	if ( ++count == number )
	    return rch;
    }

    return NULL;
}




/*
 * Find a char in the world.
 */
CHAR_DATA *get_char_world( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *wch;
    int number;
    int count;

    if ( ( wch = get_char_room( ch, argument ) ) != NULL )
	return wch;

    number = number_argument( argument, arg );
    count  = 0;
    for ( wch = char_list; wch != NULL ; wch = wch->next )
    {
	if ( wch->in_room == NULL || !can_see( ch, wch ) 
	||   !is_name( arg, CHAR_NAME(wch) ) )
	    continue;
	if ( ++count == number )
	    return wch;
    }

    return NULL;
}



/*
 * Find some object with a given index data.
 * Used by area-reset 'P' command.
 */
OBJ_DATA *get_obj_type( OBJ_INDEX_DATA *pObjIndex )
{
/*    OBJ_DATA *obj;*/

    return pObjIndex->instances;
/*
 *  for ( obj = object_list; obj != NULL; obj = obj->next )
 *  {
 *	if ( obj->pIndexData == pObjIndex )
 *	    return obj;
 *  }
 *
 *  return NULL;
 */
}


/*
 * Find an obj in a list.
 */
OBJ_DATA *get_obj_list( CHAR_DATA *ch, char *argument, OBJ_DATA *list )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;

    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = list; obj != NULL; obj = obj->next_content )
    {
	if ( can_see_obj( ch, obj ) && is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }

    return NULL;
}



/*
 * Find an obj in player's inventory.
 */
OBJ_DATA *get_obj_carry( CHAR_DATA *ch, char *argument, CHAR_DATA *viewer )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;

    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc == WEAR_NONE
	&&   (can_see_obj( viewer, obj ) ) 
	&&   is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }

    return NULL;
}



/*
 * Find an obj in player's equipment.
 */
OBJ_DATA *get_obj_wear( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;

    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc != WEAR_NONE
	&&   can_see_obj( ch, obj )
	&&   is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }

    return NULL;
}


/*
 * Find an obj in another player's equipment.
 */
OBJ_DATA *get_obj_wear_other( CHAR_DATA *ch, char *argument, CHAR_DATA *victim )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;

    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = victim->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc != WEAR_NONE
	&&   can_see_obj( ch, obj )
	&&   is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }

    return NULL;
}





/*
 * Find an obj in the room or in inventory.
 */
OBJ_DATA *get_obj_here( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj;

    obj = get_obj_list( ch, argument, ch->in_room->contents );
    if ( obj != NULL )
	return obj;

    if ( ( obj = get_obj_carry( ch, argument, ch ) ) != NULL )
	return obj;

    if ( ( obj = get_obj_wear( ch, argument ) ) != NULL )
	return obj;

    return NULL;
}



/*
 * Find an obj in the world.
 */
OBJ_DATA *get_obj_world( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;

    if ( ( obj = get_obj_here( ch, argument ) ) != NULL )
	return obj;

    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = object_list; obj != NULL; obj = obj->next )
    {
	if ( can_see_obj( ch, obj ) && is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }

    return NULL;
}

/* deduct cost from a character */

void deduct_cost(CHAR_DATA *ch, int cost)
{
    int silver = 0, gold = 0;

    silver = UMIN(ch->silver,cost); 

    if (silver < cost)
    {
	gold = ((cost - silver + 99) / 100);
	silver = cost - 100 * gold;
    }

    ch->gold -= gold;
    ch->silver -= silver;

    if (ch->gold < 0)
    {
	bug("deduct costs: gold %d < 0",ch->gold);
	ch->gold = 0;
    }
    if (ch->silver < 0)
    {
	bug("deduct costs: silver %d < 0",ch->silver);
	ch->silver = 0;
    }
}   
/*
 * Create a 'money' obj.
 */
OBJ_DATA *create_money( int gold, int silver )
{
    char buf[MAX_STRING_LENGTH];
    OBJ_DATA *obj;

    if ( gold < 0 || silver < 0 || (gold == 0 && silver == 0) )
    {
	bug( "Create_money: zero or negative money.",UMIN(gold,silver));
	gold = UMAX(1,gold);
	silver = UMAX(1,silver);
    }

    if (gold == 0 && silver == 1)
    {
	obj = create_object( get_obj_index( OBJ_VNUM_SILVER_ONE ), 0, FALSE );
    }
    else if (gold == 1 && silver == 0)
    {
	obj = create_object( get_obj_index( OBJ_VNUM_GOLD_ONE), 0, FALSE );
    }
    else if (silver == 0)
    {
        obj = create_object( get_obj_index( OBJ_VNUM_GOLD_SOME ), 0, FALSE );
        sprintf( buf, obj->short_descr, gold );
        free_string( obj->short_descr );
        obj->short_descr        = str_dup( buf );
        obj->value[1]           = gold;
        obj->cost               = gold;
	obj->weight		= gold/5;
    }
    else if (gold == 0)
    {
        obj = create_object( get_obj_index( OBJ_VNUM_SILVER_SOME ), 0, FALSE );
        sprintf( buf, obj->short_descr, silver );
        free_string( obj->short_descr );
        obj->short_descr        = str_dup( buf );
        obj->value[0]           = silver;
        obj->cost               = silver;
	obj->weight		= silver/20;
    }
 
    else
    {
	obj = create_object( get_obj_index( OBJ_VNUM_COINS ), 0, FALSE );
	sprintf( buf, obj->short_descr, silver, gold );
	free_string( obj->short_descr );
	obj->short_descr	= str_dup( buf );
	obj->value[0]		= silver;
	obj->value[1]		= gold;
	obj->cost		= 100 * gold + silver;
	obj->weight		= gold / 5 + silver / 20;
    }

    return obj;
}



/*
 * Return # of objects which an object counts as.
 * Thanks to Tony Chamberlain for the correct recursive code here.
 */
int get_obj_number( OBJ_DATA *obj )
{
    int number;
 
    if (obj->item_type == ITEM_CONTAINER || obj->item_type == ITEM_MONEY
    ||  obj->item_type == ITEM_GEM || obj->item_type == ITEM_JEWELRY)
        number = 0;
    else
        number = 1;
 
    for ( obj = obj->contains; obj != NULL; obj = obj->next_content )
        number += get_obj_number( obj );
 
    return number;
}


/*
 * Return weight of an object, including weight of contents.
 */
int get_obj_weight( OBJ_DATA *obj )
{
    int weight;
    OBJ_DATA *tobj;

    weight = obj->weight;
    for ( tobj = obj->contains; tobj != NULL; tobj = tobj->next_content )
	weight += get_obj_weight( tobj ) * WEIGHT_MULT(obj) / 100;

    return weight;
}

int get_true_weight(OBJ_DATA *obj)
{
    int weight;
 
    weight = obj->weight;
    for ( obj = obj->contains; obj != NULL; obj = obj->next_content )
        weight += get_obj_weight( obj );
 
    return weight;
}

/*
 * True if room is dark.
 */
bool room_is_dark( CHAR_DATA *ch, ROOM_INDEX_DATA *pRoomIndex )
{
    if ( pRoomIndex->light > 0 )
	return FALSE;

    if ( IS_AFFECTED(ch, AFF_DARK_VISION) )
        return FALSE;

    if ( IS_SET(pRoomIndex->room_flags, ROOM_DARK) )
	return TRUE;

    if ( pRoomIndex->sector_type == SECT_INSIDE
    ||   pRoomIndex->sector_type == SECT_CITY )
	return FALSE;

    if ( weather_info.sunlight != SUN_DARK )
	return FALSE;

    if ( LUNAR_PHASE == MOON_FULL
    &&   weather_info.sky != SKY_LIGHTNING
    &&   weather_info.sky != SKY_RAINING )
	return FALSE;

    if ( (LUNAR_PHASE == MOON_WAXING_NEAR_FULL
    ||    LUNAR_PHASE == MOON_WANING_NEAR_FULL)
    &&   weather_info.sky == SKY_CLOUDLESS )
	return FALSE;

    return TRUE;
}


bool is_room_owner(CHAR_DATA *ch, ROOM_INDEX_DATA *room)
{
    if (room->owner == NULL || room->owner[0] == '\0')
	return FALSE;

    return is_name(ch->name,room->owner);
}

/*
 * True if room is private.
 */
bool room_is_private( ROOM_INDEX_DATA *pRoomIndex )
{
    CHAR_DATA *rch;
    int count;


    if (pRoomIndex->owner != NULL && pRoomIndex->owner[0] != '\0')
	return TRUE;

    count = 0;
    for ( rch = pRoomIndex->people; rch != NULL; rch = rch->next_in_room )
	count++;

    if ( IS_SET(pRoomIndex->room_flags, ROOM_PRIVATE)  && count >= 2 )
	return TRUE;

    if ( IS_SET(pRoomIndex->room_flags, ROOM_SOLITARY) && count >= 1 )
	return TRUE;
    
    if ( IS_SET(pRoomIndex->room_flags, ROOM_IMP_ONLY) )
	return TRUE;

    return FALSE;
}

/* visibility on a room -- for entering and exits */
bool can_see_room( CHAR_DATA *ch, ROOM_INDEX_DATA *pRoomIndex )
{
    if (IS_SET(pRoomIndex->room_flags, ROOM_IMP_ONLY) 
    &&  get_trust(ch) < MAX_LEVEL)
	return FALSE;

    if (IS_SET(pRoomIndex->room_flags, ROOM_GODS_ONLY)
    &&  !IS_IMMORTAL(ch))
	return FALSE;

    if (IS_SET(pRoomIndex->room_flags, ROOM_HEROES_ONLY)
    &&  !IS_IMMORTAL(ch))
	return FALSE;

    if (IS_SET(pRoomIndex->room_flags,ROOM_NEWBIES_ONLY)
    &&  ch->level > 5 && !IS_IMMORTAL(ch))
	return FALSE;

    if (!IS_IMMORTAL(ch) && pRoomIndex->clan && ch->clan != pRoomIndex->clan)
	return FALSE;

    return TRUE;
}



/*
 * True if char can see victim.
 */
bool can_see( CHAR_DATA *ch, CHAR_DATA *victim )
{
/* RT changed so that WIZ_INVIS has levels */
    if ( ch == victim )
	return TRUE;
    
    if ( get_trust(ch) < victim->invis_level)
	return FALSE;

    if (get_trust(ch) < victim->incog_level && ch->in_room != victim->in_room)
	return FALSE;

    if ( has_holylight(ch)
    ||   (IS_NPC(ch) && IS_IMMORTAL(ch)))
	return TRUE;

    if ( IS_AFFECTED(ch, AFF_BLIND) )
	return FALSE;

    if ( !IS_AFFECTED(ch, AFF_DARK_VISION)
    &&   IS_AFFECTED2(victim, AFF2_DARKNESS_CLOUD) )
	return FALSE;

    if ( IS_AFFECTED(victim, AFF_FAERIE_FIRE) )
	return TRUE;

    if ( room_is_dark( ch, ch->in_room )
    &&   (!IS_AFFECTED(ch, AFF_INFRARED)
    ||    IS_SET(CHAR_FORM(victim), FORM_UNDEAD)
    ||    IS_SET(CHAR_FORM(victim), FORM_COLD_BLOOD))
    &&   !IS_AFFECTED(ch, AFF_DARK_VISION) )
	return FALSE;

    if ( IS_AFFECTED(victim, AFF_INVISIBLE)
    &&   !IS_AFFECTED(ch, AFF_DETECT_INVIS) )
	return FALSE;

    if ( IS_AFFECTED2(victim, AFF2_IMPROVED_INVIS)
    &&   !IS_AFFECTED(ch, AFF2_DETECT_ILLUSION)
    &&   !IS_AFFECTED(ch, AFF_DETECT_INVIS) )
	return FALSE;

    /* sneaking */
    if ( IS_AFFECTED(victim, AFF_SNEAK)
    &&   !IS_AFFECTED(ch,AFF_DETECT_HIDDEN)
    &&   victim->fighting == NULL)
    {
	int chance;
	chance = get_skill(victim,gsn_sneak);
	chance = chance + get_curr_stat(victim,STAT_DEX) * 3/8;
 	chance = chance - get_curr_stat(ch,STAT_PER) * 6/10;
	chance -= ch->level - effective_level(victim, gsn_sneak) * 3/2;

	if (number_percent() < chance)
	    return FALSE;
    }

    if ( IS_AFFECTED(victim, AFF_HIDE)
    &&   !IS_AFFECTED(ch, AFF_DETECT_HIDDEN)
    &&   victim->fighting == NULL)
	return FALSE;

    return TRUE;
}



/*
 * True if char can see obj.
 */
bool can_see_obj( CHAR_DATA *ch, OBJ_DATA *obj )
{
    if ( has_holylight(ch) )
	return TRUE;

    if ( obj->item_type == ITEM_TRAP )
	return FALSE;

    if ( ch == actor )
	return TRUE;

    if ( IS_SET(obj->extra_flags,ITEM_VIS_DEATH))
	return FALSE;

    if ( IS_AFFECTED(ch, AFF_BLIND) 
    &&   (obj->item_type != ITEM_POTION
    ||    obj->carried_by != ch) )
	return FALSE;

    if ( obj->item_type == ITEM_LIGHT && obj->value[2] != 0 && !IS_OBJ_STAT(obj, ITEM_EXTINGUISHED) )
	return TRUE;

    if ( IS_SET(obj->extra_flags, ITEM_INVIS)
    &&   !IS_AFFECTED(ch, AFF_DETECT_INVIS) )
        return FALSE;

    if ( get_trust(ch) < LEVEL_IMMORTAL
    &&   obj->pIndexData->area != NULL
    &&   IS_SET(obj->pIndexData->area->flags, AREA_VIRTUAL) )
	return FALSE;

    if ( IS_OBJ_STAT(obj,ITEM_GLOW))
	return TRUE;

    if ( room_is_dark( ch, ch->in_room ) 
    &&   !IS_AFFECTED(ch, AFF_INFRARED)
    &&   !IS_AFFECTED(ch, AFF_DARK_VISION) )
	return FALSE;

    return TRUE;
}



/*
 * True if char can drop obj.
 */
bool can_drop_obj( CHAR_DATA *ch, OBJ_DATA *obj )
{
    if ( !IS_SET(obj->extra_flags, ITEM_NODROP) )
	return TRUE;

    if ( !IS_NPC(ch) && ch->level >= LEVEL_IMMORTAL )
	return TRUE;

    return FALSE;
}


/*
 * Return ascii name of an affect location.
 */
char *affect_loc_name( int location )
{
    switch ( location )
    {
    case APPLY_NONE:		return "none";
    case APPLY_STR:		return "strength";
    case APPLY_DEX:		return "dexterity";
    case APPLY_INT:		return "intelligence";
    case APPLY_WIS:		return "wisdom";
    case APPLY_CON:		return "constitution";
    case APPLY_PER:		return "perception";
    case APPLY_CHA:		return "charisma";
    case APPLY_SEX:		return "sex";
    case APPLY_CLASS:		return "class";
    case APPLY_LEVEL:		return "level";
    case APPLY_AGE:		return "age";
    case APPLY_MANA:		return "mana";
    case APPLY_HIT:		return "hp";
    case APPLY_MOVE:		return "moves";
    case APPLY_GOLD:		return "gold";
    case APPLY_EXP:		return "experience";
    case APPLY_AC:		return "armor class";
    case APPLY_HITROLL:		return "hit roll";
    case APPLY_DAMROLL:		return "damage roll";
    case APPLY_SAVES:		return "saves";
    case APPLY_SAVING_ROD:	return "save vs rod";
    case APPLY_SAVING_PETRI:	return "save vs petrification";
    case APPLY_SAVING_BREATH:	return "save vs breath";
    case APPLY_SAVING_SPELL:	return "save vs spell";
    case APPLY_SPELL_AFFECT:	return "none";
    }

    bug( "Affect_location_name: unknown location %d.", location );
    return "(unknown)";
}



/*
 * Return ascii name of an poison type.
 */
char *poison_type_name( int type )
{
    switch ( type )
    {
	 case POISON_NORMAL:		return "default";
	 case POISON_WEAKNESS:	return "weakness";
	 case POISON_NAUSEA:		return "nausea";
	 case POISON_BLINDNESS:	return "blindness";
	 case POISON_BLEEDING:	return "bleeding";
	 case POISON_PARALYSIS:	return "paralysis";
	 case POISON_COMA:		return "coma";
	 case POISON_MADNESS:	return "madness";
	 case POISON_WITHER:		return "wither";
	 case POISON_DEATH:		return "death";
    }

    bug( "Poison_type: unknown type %d.", type );
    return "(unknown)";
}



/*
 * Return ascii name of an affect bit vector.
 */
char *affect_bit_name( int vector )
{
    static char buf[512];

    buf[0] = '\0';
    if ( vector & AFF_BLIND         ) strcat( buf, " blind"         );
    if ( vector & AFF_INVISIBLE     ) strcat( buf, " invisible"     );
    if ( vector & AFF_DETECT_EVIL   ) strcat( buf, " detect_evil"   );
    if ( vector & AFF_DETECT_GOOD   ) strcat( buf, " detect_good"   );
    if ( vector & AFF_DETECT_INVIS  ) strcat( buf, " detect_invis"  );
    if ( vector & AFF_DETECT_MAGIC  ) strcat( buf, " detect_magic"  );
    if ( vector & AFF_DETECT_HIDDEN ) strcat( buf, " detect_hidden" );
    if ( vector & AFF_SANCTUARY     ) strcat( buf, " sanctuary"     );
    if ( vector & AFF_FAERIE_FIRE   ) strcat( buf, " faerie_fire"   );
    if ( vector & AFF_INFRARED      ) strcat( buf, " infrared"      );
    if ( vector & AFF_CURSE         ) strcat( buf, " curse"         );
    if ( vector & AFF_POISON        ) strcat( buf, " poison"        );
    if ( vector & AFF_PROTECT_EVIL  ) strcat( buf, " prot_evil"     );
    if ( vector & AFF_PROTECT_GOOD  ) strcat( buf, " prot_good"     );
    if ( vector & AFF_SLEEP         ) strcat( buf, " sleep"         );
    if ( vector & AFF_SNEAK         ) strcat( buf, " sneak"         );
    if ( vector & AFF_HIDE          ) strcat( buf, " hide"          );
    if ( vector & AFF_CHARM         ) strcat( buf, " charm"         );
    if ( vector & AFF_FLYING        ) strcat( buf, " flying"        );
    if ( vector & AFF_PASS_DOOR     ) strcat( buf, " pass_door"     );
    if ( vector & AFF_BERSERK	    ) strcat( buf, " berserk"	    );
    if ( vector & AFF_CALM	    ) strcat( buf, " calm"	    );
    if ( vector & AFF_HASTE	    ) strcat( buf, " haste"	    );
    if ( vector & AFF_SLOW          ) strcat( buf, " slow"          );
    if ( vector & AFF_PLAGUE	    ) strcat( buf, " plague" 	    );
    if ( vector & AFF_DARK_VISION   ) strcat( buf, " dark_vision"   );
    if ( vector & AFF_PARALYSIS     ) strcat( buf, " paralysis"     );
    if ( vector & AFF_ANIMAL_CALL   ) strcat( buf, " animal_call"   );
    if ( vector & AFF_REGENERATION  ) strcat( buf, " regeneration"  );
    if ( vector & AFF_DETECT_TRAPS  ) strcat( buf, " detect_traps"  );
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


char *affect2_bit_name( int vector )
{
    static char buf[512];

    buf[0] = '\0';
    if ( vector & AFF2_DARKNESS_CLOUD	) strcat( buf, " darkness_cloud"	);
    if ( vector & AFF2_FIX_SOUL		) strcat( buf, " fix_soul"		);
    if ( vector & AFF2_MARK_TREE	) strcat( buf, " mark_tree"		);
    if ( vector & AFF2_WATER_SPEECH	) strcat( buf, " water_speech"		);
    if ( vector & AFF2_WATER_BREATHING	) strcat( buf, " water_breathing"	);
    if ( vector & AFF2_WATERS_WAYS	) strcat( buf, " waters_ways"		);
    if ( vector & AFF2_SMOKE_PARASITE	) strcat( buf, " smoke_parasite"	);
    if ( vector & AFF2_SPELL_STEAL	) strcat( buf, " spell_steal"		);
    if ( vector & AFF2_STOLEN_SPELL	) strcat( buf, " stolen_spell"		);
    if ( vector & AFF2_REFLECTION	) strcat( buf, " reflection"		);
    if ( vector & AFF2_STABILIZE_MANA	) strcat( buf, " stabilize_mana"	);
    if ( vector & AFF2_DOMINATION	) strcat( buf, " domination"		);
    if ( vector & AFF2_HALLUCINATION	) strcat( buf, " hallucination"		);
    if ( vector & AFF2_DETECT_ILLUSION	) strcat( buf, " detect_illusion"	);
    if ( vector & AFF2_PACIFIST		) strcat( buf, " pacifist"		);
    if ( vector & AFF2_MIRROR_IMAGE	) strcat( buf, " mirror_image"		);
    if ( vector & AFF2_DELUDE		) strcat( buf, " delude"		);
    if ( vector & AFF2_IMPROVED_INVIS	) strcat( buf, " improved_invis"	);
    if ( vector & AFF2_ALKAR		) strcat( buf, " alkar"			);
    if ( vector & AFF2_DEATH_TRANCE	) strcat( buf, " death_trance"		);
    if ( vector & AFF2_CORPSE_FORM	) strcat( buf, " corpse_form"		);
    if ( vector & AFF2_CORPSE_FORM	) strcat( buf, " corpse_form"		);
    if ( vector & AFF2_FIRE_TRACK	) strcat( buf, " fire_track"		);
    if ( vector & AFF2_TRACELESS_TRAVEL ) strcat( buf, " traceless_travel"      );
    if ( vector & AFF2_CONTINGENCY      ) strcat( buf, " contingency"           );
    if ( vector & AFF2_BACKLASH         ) strcat( buf, " backlash"              );
            
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


/*
 * Return ascii name of extra flags vector.
 */
char *extra_bit_name( int extra_flags )
{
    static char buf[512];

    buf[0] = '\0';
    if ( extra_flags & ITEM_GLOW         ) strcat( buf, " glow"         );
    if ( extra_flags & ITEM_HUM          ) strcat( buf, " hum"          );
    if ( extra_flags & ITEM_DARK         ) strcat( buf, " dark"         );
    if ( extra_flags & ITEM_LOCK         ) strcat( buf, " lock"         );
    if ( extra_flags & ITEM_EVIL         ) strcat( buf, " evil"         );
    if ( extra_flags & ITEM_INVIS        ) strcat( buf, " invis"        );
    if ( extra_flags & ITEM_MAGIC        ) strcat( buf, " magic"        );
    if ( extra_flags & ITEM_NODROP       ) strcat( buf, " nodrop"       );
    if ( extra_flags & ITEM_BLESS        ) strcat( buf, " bless"        );
    if ( extra_flags & ITEM_MIRROR       ) strcat( buf, " mirror"       );
    if ( extra_flags & ITEM_NOREMOVE     ) strcat( buf, " noremove"     );
    if ( extra_flags & ITEM_INVENTORY    ) strcat( buf, " inventory"    );
    if ( extra_flags & ITEM_NOPURGE	 ) strcat( buf, " nopurge"	);
    if ( extra_flags & ITEM_VIS_DEATH	 ) strcat( buf, " vis_death"	);
    if ( extra_flags & ITEM_ROT_DEATH	 ) strcat( buf, " rot_death"	);
    if ( extra_flags & ITEM_NOLOCATE	 ) strcat( buf, " no_locate"	);
    if ( extra_flags & ITEM_SELL_EXTRACT ) strcat( buf, " sell_extract" );
    if ( extra_flags & ITEM_BURN_PROOF	 ) strcat( buf, " burn_proof"	);
    if ( extra_flags & ITEM_NOUNCURSE	 ) strcat( buf, " no_uncurse"	);
    if ( extra_flags & ITEM_NO_SAC	 ) strcat( buf, " no_sac"	);
    if ( extra_flags & ITEM_NONMETAL     ) strcat( buf, " nonmetal"	);
    if ( extra_flags & ITEM_DISPELLED    ) strcat( buf, " dispelled"	);
    if ( extra_flags & ITEM_LODGED	 ) strcat( buf, " lodged"	);
    if ( extra_flags & ITEM_EXTINGUISHED ) strcat( buf, " extinguished" );
    if ( extra_flags & ITEM_FIRST_SOCKET ) strcat( buf, " first_socket" );
    if ( extra_flags & ITEM_SECOND_SOCKET) strcat( buf, " second_socket");
    if ( extra_flags & ITEM_THIRD_SOCKET ) strcat( buf, " third_socket" );
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

/* return ascii name of an act vector */
char *act_bit_name( int act_flags )
{
    static char buf[512];

    buf[0] = '\0';

    if (IS_SET(act_flags,ACT_IS_NPC))
    { 
 	strcat(buf," npc");
    	if (act_flags & ACT_SENTINEL 	) strcat(buf, " sentinel");
    	if (act_flags & ACT_SCAVENGER	) strcat(buf, " scavenger");
	if (act_flags & ACT_AGGRESSIVE	) strcat(buf, " aggressive");
	if (act_flags & ACT_STAY_AREA	) strcat(buf, " stay_area");
	if (act_flags & ACT_WIMPY	) strcat(buf, " wimpy");
	if (act_flags & ACT_PET		) strcat(buf, " pet");
	if (act_flags & ACT_TRAIN	) strcat(buf, " train");
	if (act_flags & ACT_PRACTICE	) strcat(buf, " practice");
	if (act_flags & ACT_UNDEAD	) strcat(buf, " undead");
	if (act_flags & ACT_CLERIC	) strcat(buf, " cleric");
	if (act_flags & ACT_MAGE	) strcat(buf, " mage");
	if (act_flags & ACT_THIEF	) strcat(buf, " thief");
	if (act_flags & ACT_WARRIOR	) strcat(buf, " warrior");
	if (act_flags & ACT_NOALIGN	) strcat(buf, " noalign");
	if (act_flags & ACT_NOPURGE	) strcat(buf, " nopurge");
	if (act_flags & ACT_IS_HEALER	) strcat(buf, " healer");
	if (act_flags & ACT_IS_CHANGER  ) strcat(buf, " changer");
	if (act_flags & ACT_GAIN	) strcat(buf, " gain");
	if (act_flags & ACT_UPDATE_ALWAYS) strcat(buf," update_always");
	if (act_flags & ACT_IS_BANKER	) strcat(buf, " banker");
	if (act_flags & ACT_MOUNT	) strcat(buf, " mount");
	if (act_flags & ACT_INDOORS     ) strcat(buf, " indoors");
	if (act_flags & ACT_OUTDOORS    ) strcat(buf, " outdoors");
	if (act_flags & ACT_META	) strcat(buf, " meta-physician");
	if (act_flags & ACT_CRIER	) strcat(buf, " crier");
	if (act_flags & ACT_SURRENDERS	) strcat(buf, " surrenders");
	if (act_flags & ACT_NO_SCAN	) strcat(buf, " no_scan");
	if (act_flags & ACT_GUARD	) strcat(buf, " guard");
    }
    else
    {
	strcat(buf," player");
	if (act_flags & PLR_JUSTICAR	) strcat(buf, " justicar");
	if (act_flags & PLR_AUTOASSIST	) strcat(buf, " autoassist");
	if (act_flags & PLR_AUTOEXIT	) strcat(buf, " autoexit");
	if (act_flags & PLR_AUTOLOOT	) strcat(buf, " autoloot");
	if (act_flags & PLR_AUTOSAC	) strcat(buf, " autosac");
	if (act_flags & PLR_AUTOGOLD	) strcat(buf, " autogold");
	if (act_flags & PLR_AUTOSPLIT	) strcat(buf, " autosplit");
	if (act_flags & PLR_NODELETE    ) strcat(buf, " no_delete" );
	if (act_flags & PLR_HOLYLIGHT	) strcat(buf, " holy_light");
	if (act_flags & PLR_CANLOOT	) strcat(buf, " loot_corpse");
	if (act_flags & PLR_NOSUMMON	) strcat(buf, " no_summon");
	if (act_flags & PLR_HOMEHIDE	) strcat(buf, " homehide");
	if (act_flags & PLR_NOFOLLOW	) strcat(buf, " no_follow");
	if (act_flags & PLR_NOMERCY	) strcat(buf, " no_mercy");
	if (act_flags & PLR_NOCANCEL	) strcat(buf, " no_cancel");
	if (act_flags & PLR_FREEZE	) strcat(buf, " frozen");
        if (act_flags & PLR_SOUNDS      ) strcat(buf, " sounds");
        if (act_flags & PLR_MUSIC       ) strcat(buf, " music");
	/*if (act_flags & PLR_THIEF	) strcat(buf, " thief");*/
	/*if (act_flags & PLR_KILLER	) strcat(buf, " killer");*/
    }
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *comm_bit_name(int comm_flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (comm_flags & COMM_QUIET		) strcat(buf, " quiet");
    if (comm_flags & COMM_DEAF		) strcat(buf, " deaf");
    if (comm_flags & COMM_NOWIZ		) strcat(buf, " no_wiz");
    if (comm_flags & COMM_NOIMP		) strcat(buf, " no_imp");
    if (comm_flags & COMM_NOAUCTION	) strcat(buf, " no_auction");
    if (comm_flags & COMM_NOGOSSIP	) strcat(buf, " no_gossip");
    if (comm_flags & COMM_NOQUESTION	) strcat(buf, " no_question");
    if (comm_flags & COMM_NOMUSIC	) strcat(buf, " no_music");
    if (comm_flags & COMM_NOCLAN	) strcat(buf, " no_clan");
    if (comm_flags & COMM_NOQUOTE	) strcat(buf, " no_quote");
    if (comm_flags & COMM_NOORATE	) strcat(buf, " no_orate");
    if (comm_flags & COMM_NORELIGION	) strcat(buf, " no_religion" );
    if (comm_flags & COMM_SHOUTSOFF	) strcat(buf, " shouts_off");
    if (comm_flags & COMM_COMPACT	) strcat(buf, " compact");
    if (comm_flags & COMM_BRIEF		) strcat(buf, " brief");
    if (comm_flags & COMM_PROMPT	) strcat(buf, " prompt");
    if (comm_flags & COMM_COMBINE	) strcat(buf, " combine");
    if (comm_flags & COMM_NOGRATS	) strcat(buf, " no_grats");
    if (comm_flags & COMM_NOEMOTE	) strcat(buf, " no_emote");
    if (comm_flags & COMM_NOSHOUT	) strcat(buf, " no_shout");
    if (comm_flags & COMM_NOTELL	) strcat(buf, " no_tell");
    if (comm_flags & COMM_PRIORITY	) strcat(buf, " priority" );
    if (comm_flags & COMM_NOCHANNELS	) strcat(buf, " no_channels");
    if (comm_flags & COMM_SNOOP_PROOF	) strcat(buf, " snoop_proof");
    

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *territory_bit_name(int terr_flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (terr_flags & TERR_CAPTAIN_GUARD	) strcat(buf, " captain_guard");
    if (terr_flags & TERR_GUARD		) strcat(buf, " guard");
    if (terr_flags & TERR_TREASURER	) strcat(buf, " treasurer");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *imm_bit_name(int imm_flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (imm_flags & IMM_SUMMON		) strcat(buf, " summon");
    if (imm_flags & IMM_CHARM		) strcat(buf, " charm");
    if (imm_flags & IMM_MAGIC		) strcat(buf, " magic");
    if (imm_flags & IMM_WEAPON		) strcat(buf, " weapon");
    if (imm_flags & IMM_BASH		) strcat(buf, " blunt");
    if (imm_flags & IMM_PIERCE		) strcat(buf, " piercing");
    if (imm_flags & IMM_SLASH		) strcat(buf, " slashing");
    if (imm_flags & IMM_FIRE		) strcat(buf, " fire");
    if (imm_flags & IMM_COLD		) strcat(buf, " cold");
    if (imm_flags & IMM_LIGHTNING	) strcat(buf, " lightning");
    if (imm_flags & IMM_ACID		) strcat(buf, " acid");
    if (imm_flags & IMM_POISON		) strcat(buf, " poison");
    if (imm_flags & IMM_NEGATIVE	) strcat(buf, " negative");
    if (imm_flags & IMM_HOLY		) strcat(buf, " holy");
    if (imm_flags & IMM_ENERGY		) strcat(buf, " energy");
    if (imm_flags & IMM_MENTAL		) strcat(buf, " mental");
    if (imm_flags & IMM_DISEASE		) strcat(buf, " disease");
    if (imm_flags & IMM_DROWNING	) strcat(buf, " drowning");
    if (imm_flags & IMM_LIGHT		) strcat(buf, " light");
    if (imm_flags & VULN_IRON		) strcat(buf, " iron");
    if (imm_flags & VULN_WOOD		) strcat(buf, " wood");
    if (imm_flags & VULN_SILVER		) strcat(buf, " silver");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *wear_bit_name(int wear_flags)
{
    static char buf[512];

    buf [0] = '\0';
    if (wear_flags & ITEM_TAKE		) strcat(buf, " take");
    if (wear_flags & ITEM_WEAR_FINGER	) strcat(buf, " finger");
    if (wear_flags & ITEM_WEAR_NECK	) strcat(buf, " neck");
    if (wear_flags & ITEM_WEAR_BODY	) strcat(buf, " torso");
    if (wear_flags & ITEM_WEAR_HEAD	) strcat(buf, " head");
    if (wear_flags & ITEM_WEAR_LEGS	) strcat(buf, " legs");
    if (wear_flags & ITEM_WEAR_FEET	) strcat(buf, " feet");
    if (wear_flags & ITEM_WEAR_HANDS	) strcat(buf, " hands");
    if (wear_flags & ITEM_WEAR_ARMS	) strcat(buf, " arms");
    if (wear_flags & ITEM_WEAR_SHIELD	) strcat(buf, " shield");
    if (wear_flags & ITEM_WEAR_ABOUT	) strcat(buf, " body");
    if (wear_flags & ITEM_WEAR_WAIST	) strcat(buf, " waist");
    if (wear_flags & ITEM_WEAR_WRIST	) strcat(buf, " wrist");
    if (wear_flags & ITEM_WIELD		) strcat(buf, " wield");
    if (wear_flags & ITEM_HOLD		) strcat(buf, " hold");
    if (wear_flags & ITEM_OLD_NO_SAC	) strcat(buf, " (old) nosac");
    if (wear_flags & ITEM_WEAR_FLOAT	) strcat(buf, " float");
    if (wear_flags & ITEM_WEAR_HAIR     ) strcat(buf, " hair");
    if (wear_flags & ITEM_WEAR_FACE	) strcat(buf, " face");
    if (wear_flags & ITEM_WEAR_EARS	) strcat(buf, " ears");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *form_bit_name(int form_flags)
{
    static char buf[512];

    buf[0] = '\0';
    if (form_flags & FORM_POISON	) strcat(buf, " poison");
    else if (form_flags & FORM_EDIBLE	) strcat(buf, " edible");
    if (form_flags & FORM_MAGICAL	) strcat(buf, " magical");
    if (form_flags & FORM_INSTANT_DECAY	) strcat(buf, " instant_rot");
    if (form_flags & FORM_OTHER		) strcat(buf, " other");
    if (form_flags & FORM_ANIMAL	) strcat(buf, " animal");
    if (form_flags & FORM_SENTIENT	) strcat(buf, " sentient");
    if (form_flags & FORM_UNDEAD	) strcat(buf, " undead");
    if (form_flags & FORM_CONSTRUCT	) strcat(buf, " construct");
    if (form_flags & FORM_MIST		) strcat(buf, " mist");
    if (form_flags & FORM_INTANGIBLE	) strcat(buf, " intangible");
    if (form_flags & FORM_BIPED		) strcat(buf, " biped");
    if (form_flags & FORM_CENTAUR	) strcat(buf, " centaur");
    if (form_flags & FORM_INSECT	) strcat(buf, " insect");
    if (form_flags & FORM_SPIDER	) strcat(buf, " spider");
    if (form_flags & FORM_CRUSTACEAN	) strcat(buf, " crustacean");
    if (form_flags & FORM_WORM		) strcat(buf, " worm");
    if (form_flags & FORM_BLOB		) strcat(buf, " blob");
    if (form_flags & FORM_MAMMAL	) strcat(buf, " mammal");
    if (form_flags & FORM_BIRD		) strcat(buf, " bird");
    if (form_flags & FORM_REPTILE	) strcat(buf, " reptile");
    if (form_flags & FORM_SNAKE		) strcat(buf, " snake");
    if (form_flags & FORM_DRAGON	) strcat(buf, " dragon");
    if (form_flags & FORM_AMPHIBIAN	) strcat(buf, " amphibian");
    if (form_flags & FORM_FISH		) strcat(buf, " fish");
    if (form_flags & FORM_COLD_BLOOD 	) strcat(buf, " cold_blooded");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *part_bit_name(int part_flags)
{
    static char buf[512];

    buf[0] = '\0';
    if (part_flags & PART_HEAD		) strcat(buf, " head");
    if (part_flags & PART_ARMS		) strcat(buf, " arms");
    if (part_flags & PART_LEGS		) strcat(buf, " legs");
    if (part_flags & PART_HEART		) strcat(buf, " heart");
    if (part_flags & PART_BRAINS	) strcat(buf, " brains");
    if (part_flags & PART_GUTS		) strcat(buf, " guts");
    if (part_flags & PART_HANDS		) strcat(buf, " hands");
    if (part_flags & PART_FEET		) strcat(buf, " feet");
    if (part_flags & PART_FINGERS	) strcat(buf, " fingers");
    if (part_flags & PART_EAR		) strcat(buf, " ears");
    if (part_flags & PART_EYE		) strcat(buf, " eyes");
    if (part_flags & PART_LONG_TONGUE	) strcat(buf, " long_tongue");
    if (part_flags & PART_EYESTALKS	) strcat(buf, " eyestalks");
    if (part_flags & PART_TENTACLES	) strcat(buf, " tentacles");
    if (part_flags & PART_FINS		) strcat(buf, " fins");
    if (part_flags & PART_WINGS		) strcat(buf, " wings");
    if (part_flags & PART_TAIL		) strcat(buf, " tail");
    if (part_flags & PART_CLAWS		) strcat(buf, " claws");
    if (part_flags & PART_FANGS		) strcat(buf, " fangs");
    if (part_flags & PART_HORNS		) strcat(buf, " horns");
    if (part_flags & PART_SCALES	) strcat(buf, " scales");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *weapon_bit_name(int weapon_flags)
{
    static char buf[512];

    buf[0] = '\0';
    if (weapon_flags & WEAPON_FLAMING	) strcat(buf, " flaming");
    if (weapon_flags & WEAPON_FROST	) strcat(buf, " frost");
    if (weapon_flags & WEAPON_VAMPIRIC	) strcat(buf, " vampiric");
    if (weapon_flags & WEAPON_SHARP	) strcat(buf, " sharp");
    if (weapon_flags & WEAPON_VORPAL	) strcat(buf, " vorpal");
    if (weapon_flags & WEAPON_TWO_HANDS	) strcat(buf, " two-handed");
    if (weapon_flags & WEAPON_SHOCKING	) strcat(buf, " shocking");
    if (weapon_flags & WEAPON_FADING	) strcat(buf, " fading");
    if (weapon_flags & WEAPON_POISON	) strcat(buf, " poison");
    if (weapon_flags & WEAPON_DEFENDER  ) strcat(buf, " defender");
    if (weapon_flags & WEAPON_CURSE     ) strcat(buf, " curse");
    if (weapon_flags & WEAPON_DISPEL    ) strcat(buf, " dispel");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *cont_bit_name( int cont_flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (cont_flags & CONT_CLOSEABLE	) strcat(buf, " closable");
    if (cont_flags & CONT_PICKPROOF	) strcat(buf, " pickproof");
    if (cont_flags & CONT_CLOSED	) strcat(buf, " closed");
    if (cont_flags & CONT_LOCKED	) strcat(buf, " locked");

    return (buf[0] != '\0' ) ? buf+1 : "none";
}


char *off_bit_name(int off_flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (off_flags & OFF_AREA_ATTACK	) strcat(buf, " area attack");
    if (off_flags & OFF_BACKSTAB	) strcat(buf, " backstab");
    if (off_flags & OFF_BASH		) strcat(buf, " bash");
    if (off_flags & OFF_BERSERK		) strcat(buf, " berserk");
    if (off_flags & OFF_DISARM		) strcat(buf, " disarm");
    if (off_flags & OFF_DODGE		) strcat(buf, " dodge");
    if (off_flags & OFF_FADE		) strcat(buf, " fade");
    if (off_flags & OFF_FAST		) strcat(buf, " fast");
    if (off_flags & OFF_KICK		) strcat(buf, " kick");
    if (off_flags & OFF_KICK_DIRT	) strcat(buf, " kick_dirt");
    if (off_flags & OFF_PARRY		) strcat(buf, " parry");
    if (off_flags & OFF_RESCUE		) strcat(buf, " rescue");
    if (off_flags & OFF_TAIL		) strcat(buf, " tail");
    if (off_flags & OFF_TRIP		) strcat(buf, " trip");
    if (off_flags & OFF_CRUSH		) strcat(buf, " crush");
    if (off_flags & ASSIST_ALL		) strcat(buf, " assist_all");
    if (off_flags & ASSIST_ALIGN	) strcat(buf, " assist_align");
    if (off_flags & ASSIST_RACE		) strcat(buf, " assist_race");
    if (off_flags & ASSIST_PLAYERS	) strcat(buf, " assist_players");
    if (off_flags & ASSIST_GUARD	) strcat(buf, " assist_guard");
    if (off_flags & ASSIST_VNUM		) strcat(buf, " assist_vnum");
    if (off_flags & OFF_HUNTER		) strcat(buf, " hunter");
    if (off_flags & OFF_NASTY_HUNTER    ) strcat(buf, " nasty_hunter");
    if (off_flags & OFF_UNMOUNT		) strcat(buf, " unmount");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}

char *room_bit_name(int room_flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (room_flags & ROOM_DARK		) strcat(buf, " dark");
    if (room_flags & ROOM_COURT         ) strcat(buf, " court");
    if (room_flags & ROOM_NO_MOB	) strcat(buf, " no_mob");
    if (room_flags & ROOM_INDOORS	) strcat(buf, " indoors");
    if (room_flags & ROOM_PRIVATE	) strcat(buf, " private");
    if (room_flags & ROOM_SAFE		) strcat(buf, " safe");
    if (room_flags & ROOM_SOLITARY	) strcat(buf, " solitary");
    if (room_flags & ROOM_PET_SHOP	) strcat(buf, " pet_shop");
    if (room_flags & ROOM_NO_RECALL	) strcat(buf, " no_recall");
    if (room_flags & ROOM_IMP_ONLY	) strcat(buf, " imp_only");
    if (room_flags & ROOM_GODS_ONLY	) strcat(buf, " gods_only");
    if (room_flags & ROOM_HEROES_ONLY	) strcat(buf, " heroes_only");
    if (room_flags & ROOM_NEWBIES_ONLY	) strcat(buf, " newbies_only");
    if (room_flags & ROOM_LAW		) strcat(buf, " law");
    if (room_flags & ROOM_NOWHERE	) strcat(buf, " nowhere");
    if (room_flags & ROOM_NO_CAST_MAGE	) strcat(buf, " no_cast_mage");
    if (room_flags & ROOM_NO_CAST_CLERIC) strcat(buf, " no_cast_cleric");
    if (room_flags & ROOM_NO_SCAN	) strcat(buf, " no_scan");
    if (room_flags & ROOM_EQUIPMENT_LOADED)
                                          strcat(buf, " equipment_loaded");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}



char *furniture_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & STAND_IN		) strcat(buf, " stand_in");
    if (flags & STAND_ON		) strcat(buf, " stand_on");
    if (flags & STAND_AT		) strcat(buf, " stand_at");
    if (flags & SIT_IN			) strcat(buf, " sit_in");
    if (flags & SIT_ON			) strcat(buf, " sit_on");
    if (flags & SIT_AT			) strcat(buf, " sit_at");
    if (flags & REST_IN			) strcat(buf, " rest_in");
    if (flags & REST_ON			) strcat(buf, " rest_on");
    if (flags & REST_AT			) strcat(buf, " rest_at");
    if (flags & SLEEP_IN		) strcat(buf, " sleep_in");
    if (flags & SLEEP_ON		) strcat(buf, " sleep_on");
    if (flags & SLEEP_AT		) strcat(buf, " sleep_at");
    if (flags & PUT_IN			) strcat(buf, " put_in");
    if (flags & PUT_ON			) strcat(buf, " put_on");
    if (flags & PUT_AT			) strcat(buf, " put_at");
    if (flags & PUT_INSIDE		) strcat(buf, " put_inside");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


char *exit_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & EX_ISDOOR		) strcat(buf, " isdoor");
    if (flags & EX_CLOSED		) strcat(buf, " closed");
    if (flags & EX_LOCKED		) strcat(buf, " locked");
    if (flags & EX_PICKPROOF		) strcat(buf, " pickproof");
    if (flags & EX_NOPASS		) strcat(buf, " nopass");
    if (flags & EX_EASY			) strcat(buf, " easy");
    if (flags & EX_HARD			) strcat(buf, " hard");
    if (flags & EX_INFURIATING		) strcat(buf, " infuriating");
    if (flags & EX_NOCLOSE		) strcat(buf, " noclose");
    if (flags & EX_NOLOCK		) strcat(buf, " nolock");
    if (flags & EX_NGBACK               ) strcat(buf, " stay");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


char *portal_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & GATE_NORMAL_EXIT	) strcat(buf, " normal_exit");
    if (flags & GATE_NOCURSE		) strcat(buf, " nocurse");
    if (flags & GATE_GOWITH		) strcat(buf, " gowith");
    if (flags & GATE_BUGGY		) strcat(buf, " buggy");
    if (flags & GATE_RANDOM		) strcat(buf, " random");
    if (flags & GATE_SENDHOME		) strcat(buf, " sendhome");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


char *aggression_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & CHAR_CAST_MAGE		) strcat(buf, " cast_mage");
    if (flags & CHAR_CAST_CLERIC	) strcat(buf, " cast_cleric");
    if (flags & CHAR_NON_CASTER		) strcat(buf, " non_caster");
    if (flags & CHAR_SEMI_CASTER	) strcat(buf, " semi_caster");
    if (flags & CHAR_PURE_CASTER	) strcat(buf, " pure_caster");
    if (flags & CHAR_MALE		) strcat(buf, " male");
    if (flags & CHAR_FEMALE		) strcat(buf, " female");
    if (flags & CHAR_EUNICH		) strcat(buf, " eunich");
    if (flags & CHAR_GOOD		) strcat(buf, " good");
    if (flags & CHAR_NEUTRAL		) strcat(buf, " neutral");
    if (flags & CHAR_EVIL		) strcat(buf, " evil");
    if (flags & CHAR_HUMAN		) strcat(buf, " human");
    if (flags & CHAR_ELF		) strcat(buf, " elf");
    if (flags & CHAR_DWARF		) strcat(buf, " dwarf");
    if (flags & CHAR_GIANT		) strcat(buf, " giant");
    if (flags & CHAR_CENTAUR		) strcat(buf, " centaur");
    if (flags & CHAR_GNOME		) strcat(buf, " gnome");
    if (flags & CHAR_ORC		) strcat(buf, " orc");
    if (flags & CHAR_CAMBION		) strcat(buf, " cambion");
    if (flags & AGG_EVERYONE		) strcat(buf, " everyone");
    if (flags & AGG_CHICKEN		) strcat(buf, " chicken");
    if (flags & AGG_PSYCHO		) strcat(buf, " psycho");
    if (flags & CHAR_NON_RULER		) strcat(buf, " non-ruler");
    if (flags & CHAR_NOBLE		) strcat(buf, " noble");
    if (flags & CHAR_PEASANT		) strcat(buf, " peasant");


    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


char *restriction_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & CHAR_CAST_MAGE		) strcat(buf, " cast_mage");
    if (flags & CHAR_CAST_CLERIC	) strcat(buf, " cast_cleric");
    if (flags & CHAR_NON_CASTER		) strcat(buf, " non_caster");
    if (flags & CHAR_SEMI_CASTER	) strcat(buf, " semi_caster");
    if (flags & CHAR_PURE_CASTER	) strcat(buf, " pure_caster");
    if (flags & CHAR_MALE		) strcat(buf, " male");
    if (flags & CHAR_FEMALE		) strcat(buf, " female");
    if (flags & CHAR_EUNICH		) strcat(buf, " eunich");
    if (flags & CHAR_GOOD		) strcat(buf, " good");
    if (flags & CHAR_NEUTRAL		) strcat(buf, " neutral");
    if (flags & CHAR_EVIL		) strcat(buf, " evil");
    if (flags & CHAR_HUMAN		) strcat(buf, " human");
    if (flags & CHAR_ELF		) strcat(buf, " elf");
    if (flags & CHAR_DWARF		) strcat(buf, " dwarf");
    if (flags & CHAR_GIANT		) strcat(buf, " giant");
    if (flags & CHAR_CENTAUR		) strcat(buf, " centaur");
    if (flags & CHAR_GNOME		) strcat(buf, " gnome");
    if (flags & CHAR_ORC		) strcat(buf, " orc");
    if (flags & CHAR_CAMBION		) strcat(buf, " cambion");
    if (flags & RESTRICT_HARMFUL	) strcat(buf, " harmful");
    if (flags & RESTRICT_WASTING	) strcat(buf, " wasting");
    if (flags & RESTRICT_CORRUPTING	) strcat(buf, " corrupting");
    if (flags & CHAR_NON_RULER		) strcat(buf, " non-ruler");
    if (flags & CHAR_NOBLE		) strcat(buf, " noble");
    if (flags & CHAR_PEASANT		) strcat(buf, " peasant");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


char *fit_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & FIT_NO_TINY		) strcat(buf, " no_tiny");
    if (flags & FIT_NO_SMALL		) strcat(buf, " no_small");
    if (flags & FIT_NO_MEDIUM		) strcat(buf, " no_medium");
    if (flags & FIT_NO_LARGE		) strcat(buf, " no_large");
    if (flags & FIT_NO_HUGE		) strcat(buf, " no_huge");
    if (flags & FIT_NO_GIANT		) strcat(buf, " no_giant");
    if (flags & FIT_NO_HUMAN		) strcat(buf, " no_human");
    if (flags & FIT_NO_ELF		) strcat(buf, " no_elf");
    if (flags & FIT_NO_DWARF		) strcat(buf, " no_dwarf");
    if (flags & FIT_NO_GIANT_RACE	) strcat(buf, " no_giant_race");
    if (flags & FIT_NO_CENTAUR		) strcat(buf, " no_centaur");
    if (flags & FIT_NO_GNOME		) strcat(buf, " no_gnome");
    if (flags & FIT_NO_ORC		) strcat(buf, " no_orc");
    if (flags & FIT_NO_CAMBION		) strcat(buf, " no_cambion");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


char *area_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & AREA_ARENA		) strcat(buf, " arena");
    if (flags & AREA_DIRTY		) strcat(buf, " dirty");
    if (flags & AREA_EVERLOAD		) strcat(buf, " everload");
    if (flags & AREA_FREE		) strcat(buf, " free");
    if (flags & AREA_INVIS		) strcat(buf, " invis");
    /*if (flags & AREA_LAWLESS		) strcat(buf, " lawless");*/
    if (flags & AREA_OPEN_SLATHER	) strcat(buf, " open_slather");
    if (flags & AREA_QUIET		) strcat(buf, " quiet");
    if (flags & AREA_REVAMP		) strcat(buf, " revamp");
    if (flags & AREA_VIRTUAL		) strcat(buf, " virtual");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


bool has_customization( CHAR_DATA *ch, int type )
{
    CUST_DATA *current;

    if ( IS_NPC(ch) )
        return FALSE;

    for ( current = ch->pcdata->customizations;
          current != NULL;
          current = current->next )
    {
        if ( current->type == type )
            return TRUE;
    }

    return FALSE;
}

bool has_holylight( CHAR_DATA *ch )
{
    if ( IS_NPC(ch) )
	return FALSE;

    if ( !IS_SET(ch->act, PLR_HOLYLIGHT) )
	return FALSE;

    if ( IS_IMMORTAL(ch) )
	return TRUE;

    if ( ch->in_room != NULL && is_creator(ch, ch->in_room->area) )
	return TRUE;

    return FALSE;
}


int get_olc_level( CHAR_DATA *ch )
{
    if ( ch->desc != NULL && ch->desc->original != NULL )
	ch = ch->desc->original;

    if ( !IS_NPC(ch) )
	return ch->pcdata->olc_level;

    return 0;
}


void remove_memory( CHAR_DATA *ch, MEM_DATA *memory )
{
    MEM_DATA *previous;
    
    if ( ch->memory == memory )
	ch->memory = memory->next;
    else
    {
	for ( previous = ch->memory; previous != NULL; previous = previous->next )
	{
	    if ( previous->next == memory )
	    {
		previous->next = memory->next;
		break;
	    }
	}
	
	if ( previous == NULL )
	{
	    bug( "Remove_memory: memory not found.", 0 );
	    return;
	}
    }
    
    free_mem_data( memory );
}


char *trap_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & TRAP_OBJ		) strcat(buf, " object");
    if (flags & TRAP_ROOM		) strcat(buf, " room");
    if (flags & TRAP_ENTER		) strcat(buf, " enter");
    if (flags & TRAP_LEAVE		) strcat(buf, " leave");
    if (flags & TRAP_OPEN		) strcat(buf, " open");
    if (flags & TRAP_CLOSE		) strcat(buf, " close");
    if (flags & TRAP_UNLOCK		) strcat(buf, " unlock");
    if (flags & TRAP_PICK		) strcat(buf, " pick");
    if (flags & TRAP_GET		) strcat(buf, " get");
    if (flags & TRAP_PUT		) strcat(buf, " put");
    if (flags & TRAP_EXAMINE		) strcat(buf, " examine");
    if (flags & TRAP_N			) strcat(buf, " north");
    if (flags & TRAP_S			) strcat(buf, " south");
    if (flags & TRAP_E			) strcat(buf, " east");
    if (flags & TRAP_W			) strcat(buf, " west");
    if (flags & TRAP_U			) strcat(buf, " up");
    if (flags & TRAP_D			) strcat(buf, " down");
    if (flags & TRAP_EASY		) strcat(buf, " easy");
    if (flags & TRAP_HARD		) strcat(buf, " hard");
    if (flags & TRAP_INFURIATING	) strcat(buf, " infuriating");
    if (flags & TRAP_IMPOSSIBLE		) strcat(buf, " impossible");
    if (flags & TRAP_MAGICAL		) strcat(buf, " magical");
    if (flags & TRAP_NE			) strcat(buf, " northeast");
    if (flags & TRAP_SE			) strcat(buf, " southeast");
    if (flags & TRAP_SW			) strcat(buf, " southwest");
    if (flags & TRAP_NW			) strcat(buf, " northwest");
    if (flags & TRAP_CW			) strcat(buf, " clockwise");
    if (flags & TRAP_CCW		) strcat(buf, " counterclockwise");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


/*
char *territory_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & TER_ADVANCE_BY_COMBAT	) strcat(buf, " advance_by_combat");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
*/

char *find_use_msg( OBJ_DATA *obj, sh_int tag )
{
    USE_MSG_DATA *current;
    
    for ( current = obj->pIndexData->use_msgs; current != NULL; current = current->next )
    {
	if ( current->tag == tag )
	    return current->msg;
    }
    
    return NULL;
}


char *crime_bit_name(int flags)
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & CRIME_PKILLING              ) strcat(buf, " pkilling");
    if (flags & CRIME_PSTEALING             ) strcat(buf, " pstealing");
    if (flags & TERRITORY_JUSTICAR_ALIGNED  ) strcat(buf, " justicar_aligned");

    return ( buf[0] != '\0' ) ? buf+1 : "none";
}


ROOM_INDEX_DATA *get_random_room(CHAR_DATA *ch)
{
    ROOM_INDEX_DATA *room;

    for ( ; ; )
    {
        room = get_room_index( number_range( 0, 65535 ) );
        if ( room != NULL )
        if ( can_see_room(ch,room)
	&&   !room_is_private(room)
        &&   !IS_SET(room->room_flags, ROOM_PRIVATE)
        &&   !IS_SET(room->room_flags, ROOM_SOLITARY) 
	&&   !IS_SET(room->room_flags, ROOM_SAFE) 
	&&   !IS_SET(room->room_flags, ROOM_NO_RECALL)
	&&   (IS_NPC(ch) || IS_SET(ch->act,ACT_AGGRESSIVE) 
	||   !IS_SET(room->room_flags,ROOM_LAW)))
            break;
    }

    return room;
}


OBJ_DATA  *get_random_obj(CHAR_DATA *ch)
{
    OBJ_INDEX_DATA *pObjIndex;

    for ( ; ; )
    {
        pObjIndex = get_obj_index( number_range( 0, 65535 ) );
        if ( pObjIndex != NULL
        &&   pObjIndex->instances
        &&   can_see_obj(ch, pObjIndex->instances) )
            break;
    }

    return pObjIndex->instances;
}


CHAR_DATA *get_random_npc(CHAR_DATA *ch)
{
    MOB_INDEX_DATA *pMobIndex;

    for ( ; ; )
    {
        pMobIndex = get_mob_index( number_range( 0, 65535 ) );
        if ( pMobIndex != NULL
        &&   pMobIndex->instances
        &&   can_see(ch, pMobIndex->instances) )
            break;
    }

    return pMobIndex->instances;
}


const char *get_room_description( CHAR_DATA *ch )
{
    if ( IS_AFFECTED2(ch, AFF2_HALLUCINATION) )
        return get_random_room(ch)->description;
    return ch->in_room->description;
}


const char *get_room_name( CHAR_DATA *ch )
{
    if ( IS_AFFECTED2(ch, AFF2_HALLUCINATION) )
        return get_random_room(ch)->name;
    return ch->in_room->name;
}


int count_objects( OBJ_DATA *obj )
{
    if ( obj == NULL )
        return 0;
    if ( obj->next_content )
        return 1 + count_objects( obj->next_content );
    if ( obj->contains )
        return 1 + count_objects( obj->contains );
    return 1;
}
