/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,	   *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *									   *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael	   *
 *  Chastain, Michael Quan, and Mitchell Tse.				   *
 *									   *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc	   *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.						   *
 *									   *
 *  Much time and thought has gone into this software and you are	   *
 *  benefitting.  We hope that you share your changes too.  What goes	   *
 *  around, comes around.						   *
 ***************************************************************************/
 
/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "magic.h"
#include "recycle.h"


bool sn_in_group( const char *group_name, int sn )
{
    int group;
    int i;

    group = group_lookup( group_name );
    if ( group == -1 )
        return FALSE;

    for ( i = 0; i < MAX_IN_GROUP; ++i )
    {
        if ( IS_NULLSTR(group_table[group].spells[i]) )
            continue;

        if ( skill_lookup(group_table[group].spells[i]) == sn )
            return TRUE;
    }

    return FALSE;
}


bool gn_in_group( const char *group_name, int gn )
{
    int group;
    int i;

    group = group_lookup( group_name );
    if ( group == -1 )
        return FALSE;

    for ( i = 0; i < MAX_IN_GROUP; ++i )
    {
        if ( IS_NULLSTR(group_table[group].spells[i]) )
            continue;

        if ( group_lookup(group_table[group].spells[i]) == gn )
            return TRUE;
    }

    return FALSE;
}


bool skill_not_default( int class, int sn )
{
    return !sn_in_group( class_table[class].base_group, sn )
        && !sn_in_group( class_table[class].default_group, sn )
        && !sn_in_group( "rom basics", sn )
        && str_cmp( skill_table[sn].name, "recall" );
}


bool group_not_default( int class, int gn )
{
    return !gn_in_group( class_table[class].base_group, gn )
        && !gn_in_group( class_table[class].default_group, gn )
        && str_cmp( group_table[gn].name, "rom basics" );
}


bool disp_skills_in_group( BUFFER *output, const char *group, bool added )
{
    char buf[MAX_STRING_LENGTH];
    bool first = !added;
    int gn, sn;

    gn = group_lookup( group );
    if ( gn == -1 )
    {
        add_buf( output, "<invalid class base group specified - please contact mud administration>" );
        return FALSE;
    }

    for ( sn = 0; sn < MAX_IN_GROUP; ++sn )
    {
        if ( IS_NULLSTR(group_table[gn].spells[sn]) )
            continue;

        if ( group_lookup(group_table[gn].spells[sn]) != -1 )
            continue;

        if ( skill_lookup(group_table[gn].spells[sn]) == -1 )
        {
            sprintf( buf, "<invalid skill, %s, in group %s - please contact mud administration>",
                group_table[gn].spells[sn], group );
            add_buf( output, buf );
            return !first;
        }

        if ( !first )
            sprintf( buf, ", %s", group_table[gn].spells[sn] );
        else
        {
            sprintf( buf, "%s", group_table[gn].spells[sn] );
            first = FALSE;
        }
        add_buf( output, buf );
    }

    return !first;
}


bool disp_groups_in_group( BUFFER *output, const char *group, bool added )
{
    char buf[MAX_STRING_LENGTH];
    bool first = !added;
    int gn, sn;

    gn = group_lookup( group );
    if ( gn == -1 )
    {
        add_buf( output, "<invalid class base group specified - please contact mud administration>" );
        return FALSE;
    }

    for ( first = TRUE, sn = 0; sn < MAX_IN_GROUP; ++sn )
    {
        int group_num;

        if ( IS_NULLSTR(group_table[gn].spells[sn]) )
            continue;

        if ( skill_lookup(group_table[gn].spells[sn]) != -1 )
            continue;

        group_num = group_lookup( group_table[gn].spells[sn] );
        if ( group_num == -1 )
        {
            sprintf( buf, "<invalid sub-group, %s, in group %s - please contact mud administration>",
                group_table[gn].spells[sn], group );
            add_buf( output, buf );
            return FALSE;
        }

        if ( !first )
            sprintf( buf, ", %s", group_table[gn].spells[sn] );
        else
        {
            sprintf( buf, "%s", group_table[gn].spells[sn] );
            first = FALSE;
        }

        add_buf( output, buf );
    }

    return !first;
}


void append_default_skills( CHAR_DATA *ch, BUFFER *output, const char *args )
{
    
    int class = class_lookup( args );
    bool added;

    if ( class == -1 )
    {
        add_buf( output, "<invalid class specified - please contact mud administration>" );
        return;
    }
    
    added  = disp_skills_in_group( output, class_table[class].base_group, FALSE );
    disp_skills_in_group( output, class_table[class].default_group, added );
}


void append_gainable_skills( CHAR_DATA *ch, BUFFER *output, const char *args )
{
    int class = class_lookup( args );
    bool first = TRUE;
    int sn;

    if ( class == -1 )
    {
        add_buf( output, "<invalid class specified - please contact mud administration>" );
        return;
    }

    if ( IS_SET(class_table[class].flags, CLASS_NOCUST) )
	return;

    for ( sn = 0; sn < max_player_skill; ++sn )
    {
        char buf[MAX_STRING_LENGTH];

        if (skill_table[sn].name == NULL)
            break;
 
        if ( skill_table[sn].rating[class] > 0
	&&   skill_table[sn].spell_fun == spell_null
	&&   skill_table[sn].gainable
        &&   skill_not_default(class, sn) )
        {
            if ( first )
            {
                sprintf( buf, "%s", skill_table[sn].name );
                first = FALSE;
            }
            else
                sprintf( buf, ", %s", skill_table[sn].name );
            add_buf( output, buf );
        }
    }
}


void append_default_groups( CHAR_DATA *ch, BUFFER *output, const char *args )
{
    
    int class = class_lookup( args );
    bool added;

    if ( class == -1 )
    {
        add_buf( output, "<invalid class specified - please contact mud administration>" );
        return;
    }
    
    added = disp_groups_in_group( output, class_table[class].base_group, FALSE );
    disp_groups_in_group( output, class_table[class].default_group, added );
}


void append_gainable_groups( CHAR_DATA *ch, BUFFER *output, const char *args )
{
    int class = class_lookup( args );
    bool first = TRUE;
    int gn;

    if ( class == -1 )
    {
        add_buf( output, "<invalid class specified - please contact mud administration>" );
        return;
    }

    if ( IS_SET(class_table[class].flags, CLASS_NOCUST) )
	return;

    for ( gn = 0; gn < MAX_GROUP; ++gn )
    {
        char buf[MAX_STRING_LENGTH];

        if ( IS_NULLSTR(group_table[gn].name) )
            break;

        if ( str_suffix("basics", group_table[gn].name) == 0
        ||   str_suffix("default", group_table[gn].name) == 0 )
            continue;
 
	if ( group_table[gn].rating[ch->class] > 0
	&&   group_table[gn].gainable
	&&   group_not_default(class, gn) )
        {
            if ( first )
            {
                sprintf( buf, "%s", group_table[gn].name );
                first = FALSE;
            }
            else
                sprintf( buf, ", %s", group_table[gn].name );
            add_buf( output, buf );
        }
    }
}


void append_race_cost( CHAR_DATA *ch, BUFFER *output, const char *args )
{
    int race = race_lookup( args );
    char buf[MAX_STRING_LENGTH];
    
    if ( race == -1 ) 
    {
        add_buf( output, "<invalid race specified - please contact mud administration>" );
        return;
    }

    sprintf( buf, "%d", pc_race_table[race].points );
    add_buf( output, buf );
}


void append_race_skills( CHAR_DATA *ch, BUFFER *output, const char *args )
{
    int race = race_lookup( args );
    char buf[MAX_STRING_LENGTH];
    bool first = TRUE;
    int i;
    
    if ( race == -1 ) 
    {
        add_buf( output, "<invalid race specified - please contact mud administration>" );
        return;
    }

    for ( i = 0; i < 5; ++i )
    {
        if ( IS_NULLSTR(pc_race_table[race].skills[i]) )
            continue;

        if ( first )
        {
            sprintf( buf, "%s", pc_race_table[race].skills[i] );
            first = FALSE;
        }
        else
            sprintf( buf, ", %s", pc_race_table[race].skills[i] );
        add_buf( output, buf );
    }
}


void append_ch_cost( CHAR_DATA *ch, BUFFER *output, const char *args )
{
    char buf[ MAX_STRING_LENGTH ];
    sprintf( buf, "Creation points: %d\n\r", ch->pcdata->points );
    add_buf( output, buf );
    if ( ch->gen_data )
    {
        sprintf( buf, "Experience per level: %d\n\r",
	    exp_per_level(ch, ch->gen_data->points_chosen) );
        add_buf( output, buf );
    }
}

