/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "merc.h"
#include "tables.h"
#include "recycle.h"
#include "lookup.h"
#include "olc.h"


/*
 * Macro definitions.
 */
#define SIDE_LEFT	0
#define SIDE_RIGHT	1
#define SIDE_ANY	2


/*
 * Type definitions.
 */
typedef	bool MOBILE_FUN	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
				MOB_INDEX_DATA *pMobIndex,
				char *argument ) );

struct mobile_command
{
    char *	name;
    MOBILE_FUN	*fun;
    bool	update;
    sh_int	olc_level;
};


/*
 * Global function prototypes.
 */
int     flag_lookup     args( ( const char *name,
                                const struct flag_type *flag_table ) );


/*
 * Local function prototypes.
 */
bool	do_mobile_level		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_special	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_immune	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_vuln		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_special	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_resist	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_act		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_offense	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_name		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_short		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_long		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_description	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_damtype	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_damage	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_hitroll	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_sex		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_hp		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_mana		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_race		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_auto		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_limit		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_roomlimit	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_clone		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_purge		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_resets	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_equip		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_alignment	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_wealth	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_size		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_shop		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_aggression	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_affects	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_program	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_form		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_parts		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );
bool	do_mobile_remove	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					MOB_INDEX_DATA *pMobIndex, 
					char *argument ) );


/*
 * The mobile command table.
 */
const struct mobile_command mobile_commands[] =
{
    { "level",		do_mobile_level,	TRUE,	2	},
    { "spec",		do_mobile_special,	TRUE,	2	},
    { "immune",		do_mobile_immune,	TRUE,	2	},
    { "vuln",		do_mobile_vuln,		TRUE,	2	},
    { "resist",		do_mobile_resist,	TRUE,	2	},
    { "act",		do_mobile_act,		TRUE,	2	},
    { "offense",	do_mobile_offense,	TRUE,	2	},
    { "name",		do_mobile_name,		TRUE,	1	},
    { "short",		do_mobile_short,	TRUE,	1	},
    { "long",		do_mobile_long,		TRUE,	1	},
    { "description",	do_mobile_description,	TRUE,	1	},
    { "damtype",	do_mobile_damtype,	TRUE,	2	},
    { "damage",		do_mobile_damage,	TRUE,	2	},
    { "hitroll",	do_mobile_hitroll,	TRUE,	2	},
    { "sex",		do_mobile_sex,		TRUE,	1	},
    { "hp",		do_mobile_hp,		TRUE,	2	},
    { "mana",		do_mobile_mana,		TRUE,	2	},
    { "race",		do_mobile_race,		TRUE,	2	},
    { "auto",		do_mobile_auto,		TRUE,	2	},
    { "limit",		do_mobile_limit,	FALSE,	1	},
    { "roomlimit",	do_mobile_roomlimit,	FALSE,	1	},
    { "clone",		do_mobile_clone,	FALSE,	1	},
    { "purge",		do_mobile_purge,	FALSE,	1	},
    { "resets",		do_mobile_resets,	FALSE,	1	},
    { "equip",		do_mobile_equip,	FALSE,	1	},
    { "alignment",	do_mobile_alignment,	TRUE,	2	},
    { "wealth",		do_mobile_wealth,	TRUE,	4	},
    { "size",		do_mobile_size,		TRUE,	2	},
    { "shop",		do_mobile_shop,		TRUE,	2	},
    { "aggression",	do_mobile_aggression,	FALSE,	2	},
    { "affects",	do_mobile_affects,	TRUE,	2	},
    { "program",	do_mobile_program,	TRUE,	4	},
    { "form",		do_mobile_form,		TRUE,	3	},
    { "parts",		do_mobile_parts,	TRUE,	3	},
    { "remove",		do_mobile_remove,	FALSE,	1	},
    { NULL }
};


/*
 * Mobile auto values.
 */
const struct mobile_auto mobile_autos[MAX_LEVEL] =
{
    { { 2, 6,  10}, { 1, 10, 95}, { 1, 4, 0},  9 },	/*  1 */
    { { 2, 7,  21}, { 1, 12,110}, { 1, 5, 0},  8 },
    { { 2, 6,  35}, { 1, 14,125}, { 1, 6, 0},  7 },
    { { 2, 7,  46}, { 1, 16,140}, { 1, 5, 1},  6 },
    { { 2, 6,  60}, { 1, 18,155}, { 1, 6, 1},  5 },	/*  5 */
    { { 2, 7,  71}, { 1, 20,170}, { 1, 7, 1},  4 },
    { { 2, 6,  85}, { 1, 22,185}, { 1, 8, 1},  4 },
    { { 2, 7,  96}, { 1, 24,200}, { 1, 7, 2},  3 },
    { { 2, 6, 110}, { 1, 26,215}, { 1, 8, 2},  2 },
    { { 2, 7, 121}, { 1, 28,230}, { 2, 4, 2},  1 },	/* 10 */
    { { 2, 8, 134}, { 1, 30,245}, { 1,10, 2},  1 },
    { { 2,10, 150}, { 1, 32,260}, { 1,10, 3},  0 },
    { { 2,10, 170}, { 1, 34,275}, { 2, 5, 3}, -1 },
    { { 2,10, 190}, { 1, 36,290}, { 1,12, 3}, -1 },
    { { 3, 9, 208}, { 1, 38,305}, { 2, 6, 3}, -2 },	/* 15 */
    { { 3, 9, 233}, { 1, 40,320}, { 2, 6, 4}, -2 },
    { { 3, 9, 258}, { 1, 42,335}, { 3, 4, 4}, -3 },
    { { 3, 9, 283}, { 1, 44,350}, { 2, 7, 4}, -3 },
    { { 3, 9, 308}, { 1, 46,365}, { 2, 7, 5}, -4 },
    { { 3, 9, 333}, { 1, 48,380}, { 2, 8, 5}, -4 },	/* 20 */
    { { 4,10, 360}, { 1, 50,395}, { 4, 4, 5}, -5 },
    { { 5,10, 400}, { 1, 52,410}, { 4, 4, 6}, -5 },
    { { 5,10, 450}, { 1, 54,425}, { 3, 6, 6}, -6 },
    { { 5,10, 500}, { 1, 56,440}, { 2,10, 6}, -6 },
    { { 5,10, 550}, { 1, 58,455}, { 2,10, 7}, -7 },	/* 25 */
    { { 5,10, 600}, { 1, 60,470}, { 3, 7, 7}, -7 },
    { { 5,10, 650}, { 1, 62,485}, { 5, 4, 7}, -8 },
    { { 6,12, 703}, { 1, 64,500}, { 2,12, 8}, -8 },
    { { 6,12, 778}, { 1, 66,515}, { 2,12, 8}, -9 },
    { { 6,12, 853}, { 1, 68,530}, { 4, 6, 8}, -9 },	/* 30 */
    { { 6,12, 928}, { 1, 70,545}, { 4, 6, 9},-10 },
    { {10,10,1000}, { 1, 72,560}, { 6, 4, 9},-10 },
    { {10,10,1100}, { 1, 74,575}, { 6, 4,10},-11 },
    { {10,10,1200}, { 1, 76,590}, { 4, 7,10},-11 },
    { {10,10,1300}, { 1, 78,605}, { 4, 7,11},-11 },	/* 35 */
    { {10,10,1400}, { 1, 80,620}, { 3,10,11},-12 },
    { {10,10,1500}, { 1, 82,635}, { 3,10,12},-12 },
    { {10,10,1600}, { 1, 84,650}, { 5, 6,12},-13 },
    { {15,10,1700}, { 1, 86,665}, { 5, 8,13},-13 },
    { {15,10,1850}, { 1, 88,680}, { 4, 8,13},-13 },	/* 40 */
    { {25,10,2000}, { 1, 90,695}, { 4, 8,14},-14 },
    { {25,10,2250}, { 1, 92,710}, { 3,12,14},-14 },
    { {25,10,2500}, { 1, 94,725}, { 3,12,15},-15 },
    { {25,10,2750}, { 1, 96,740}, { 8, 4,15},-15 },
    { {25,10,3000}, { 1, 98,755}, { 8, 4,16},-15 },	/* 45 */
    { {25,10,3250}, { 1,100,770}, { 6, 6,16},-16 },
    { {25,10,3500}, { 1,102,785}, { 6, 6,17},-17 },
    { {25,10,3750}, { 1,104,800}, { 6, 6,18},-18 },
    { {50,10,4000}, { 1,106,815}, { 4,10,18},-19 },
    { {50,10,4500}, { 1,108,830}, { 5, 8,19},-20 },	/* 50 */
    { {50,10,5000}, { 1,110,845}, { 5, 8,20},-21 },
    { {50,10,5500}, { 1,112,860}, { 6, 7,20},-22 },
    { {50,10,6000}, { 1,114,875}, { 6, 7,21},-23 },
    { {50,10,6500}, { 1,116,890}, { 7, 6,22},-24 },
    { {50,10,7000}, { 1,118,905}, {10, 4,23},-25 },	/* 55 */
    { {50,10,7500}, { 1,120,920}, {10, 4,24},-26 },
    { {50,10,8000}, { 1,122,935}, { 6, 8,24},-27 },
    { {50,10,8500}, { 1,124,950}, { 5,10,25},-28 },
    { {50,10,9000}, { 1,126,965}, { 8, 6,26},-29 },
    { {50,10,9500}, { 1,128,980}, { 8, 6,28},-30 }	/* 60 */
};


bool get_dice( char *argument, int *number, int *dice, int *bonus )
{
    char *current, sign = '+';
    
    for ( current = argument; *current != '\0'; current++ )
    {
	if ( *current == 'd' || *current == 'D' )
	{
	    *current = '\0';
	    
	    if ( !is_number(argument) )
		return FALSE;
		
	    *number = atoi( argument );
	    
	    current++;
	    break;
	}
	
	if ( !isdigit(*current) )
	    return FALSE;
    }
    
    argument = current;
    for ( current = argument; *current != '\0'; current++ )
    {
	if ( *current == '+' || *current == '-' )
	{
	    sign = *current;
	    
	    *current = '\0';
	    
	    if ( !is_number(argument) )
		return FALSE;
		
	    *dice = atoi( argument );
	    
	    current++;
	    break;
	}

	if ( !isdigit(*current) )
	    return FALSE;
    }

    if ( !is_number(current) )
	return FALSE;
	
    *bonus = atoi( current );
    if ( sign == '-' )
	*bonus = -1 * (*bonus);

    return TRUE;
}


int wear_location( OBJ_INDEX_DATA *pObjIndex, int side )
{
    if ( pObjIndex->item_type == ITEM_LIGHT )
	return WEAR_HOLD /*WEAR_LIGHT*/;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_FINGER) )
    {
	if ( side == SIDE_LEFT )
	    return WEAR_FINGER_L;
	else
	    return WEAR_FINGER_R;
    }
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_NECK) )
	return WEAR_NECK_1;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_BODY) )
	return WEAR_BODY;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_HEAD) )
	return WEAR_HEAD;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_LEGS) )
	return WEAR_LEGS;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_FEET) )
	return WEAR_FEET;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_HANDS) )
	return WEAR_HANDS;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_ARMS) )
	return WEAR_ARMS;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_SHIELD) )
	return WEAR_SHIELD;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_ABOUT) )
	return WEAR_ABOUT;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_WAIST) )
	return WEAR_WAIST;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_WRIST) )
    {
	if ( side == SIDE_LEFT )
	    return WEAR_WRIST_L;
	else
	    return WEAR_WRIST_R;
    }
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WIELD) )
    {
	if ( side == SIDE_LEFT )
	    return WEAR_SECONDARY;
	else
	    return WEAR_WIELD;
    }
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_HOLD) )
	return WEAR_HOLD;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_FLOAT) )
	return WEAR_FLOAT;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_HAIR) ) /* Llaby */
	return WEAR_HAIR;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_FACE) ) /* Llaby */
	return WEAR_FACE;
    else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_EARS) ) /* Llaby */
	return WEAR_EARS;
    else
	return WEAR_NONE;
}


void do_mobile( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    MOB_INDEX_DATA *pMobIndex;
    AREA_DATA *pArea;
    int command;
    
    if ( ch->in_room == NULL )
    {
	send_to_char( "You need to be in a room before you try doing that.\n\r", ch );
	return;
    }
    
    pArea = ch->in_room->area;

    if ( !is_creator(ch, pArea) )
    {
	send_to_char( "You are not a creator in this area.\n\r", ch );
	return;
    }
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Which mobile to you want to edit?\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg );
    if ( is_number(arg) )
    {
	if ( (pMobIndex = get_mob_index(atoi(arg))) == NULL )
	{
	    send_to_char( "No mobile with that vnum found.\n\r", ch );
	    return;
	}
	
	if ( pMobIndex->vnum < pArea->min_vnum
	||   pMobIndex->vnum > pArea->max_vnum )
	{
	    send_to_char( "That mobile a part of this area.\n\r", ch );
	    return;
	}
    }
    else
    {
	int vnum;
	
	pMobIndex = NULL;
	
	for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
	{
	    if ( (pMobIndex = get_mob_index(vnum)) != NULL
	    &&   is_name(arg, pMobIndex->player_name) )
		break;
	    
	    pMobIndex = NULL;
	}
	
	if ( pMobIndex == NULL )
	{
	    send_to_char( "No mobile with that name found in this area.\n\r", ch );
	    return;
	}
    }

    argument = one_argument( argument, arg );

    for ( command = 0; mobile_commands[command].name != NULL; command++ )
    {
	if ( LOWER(arg[0]) == LOWER(mobile_commands[command].name[0])
	&&   !str_prefix(arg, mobile_commands[command].name))
	{
	    if ( get_olc_level(ch) < mobile_commands[command].olc_level )
	    {
		send_to_char( 
		    "You do not have sufficient OLC privileges to do that.\n\r", 
		    ch );
		return;
	    }
	    
	    if ( (*mobile_commands[command].fun) 
	            ( ch, pArea, pMobIndex, argument ) )
	    {
		SET_BIT( pArea->flags, AREA_DIRTY );
		
		if ( mobile_commands[command].update )
		{
		    CHAR_DATA *current;
		    
		    for ( current = pMobIndex->instances;
		          current != NULL;
		          current = current->next_instance )
		    {
			int i;
			
			free_string( current->name );
			free_string( current->short_descr );
			free_string( current->long_descr );
			free_string( current->description );
			
			current->name		= str_dup( pMobIndex->player_name );
			current->short_descr	= str_dup( pMobIndex->short_descr );
			current->long_descr	= str_dup( pMobIndex->long_descr );
			current->description	= str_dup( pMobIndex->description );
			current->spec_fun	= pMobIndex->spec_fun;
			current->act		= pMobIndex->act;
			current->affected_by	= pMobIndex->affected_by;
			current->affected_by2   = pMobIndex->affected_by2;
			current->alignment	= pMobIndex->alignment;
			current->gold		= pMobIndex->wealth / 100;
			current->silver		= pMobIndex->wealth % 100;
			current->level		= pMobIndex->level;
			current->hitroll	= pMobIndex->hitroll;
			current->damroll	= pMobIndex->damage[DICE_BONUS];
			
			current->max_hit	= dice( pMobIndex->hit[DICE_NUMBER],
						        pMobIndex->hit[DICE_TYPE] ) +
						  pMobIndex->hit[DICE_BONUS];
			current->hit		= current->max_hit;
			
			current->max_mana	= dice( pMobIndex->mana[DICE_NUMBER],
						        pMobIndex->mana[DICE_TYPE]) +
						  pMobIndex->mana[DICE_BONUS];
			current->mana		= current->max_mana;
			
			current->damage[DICE_NUMBER] = pMobIndex->damage[DICE_NUMBER];
			current->damage[DICE_TYPE]   = pMobIndex->damage[DICE_TYPE];
			current->dam_type	     = pMobIndex->dam_type;
			if ( current->dam_type == 0 )
			{
			    switch(number_range(1,3))
			    {
			    case (1): current->dam_type = 3;	break;
			    case (2): current->dam_type = 7;	break;
			    case (3): current->dam_type = 11;	break;
			    }
			}
			
			for ( i=0; i<4; i++ )
			    current->armor[i]	= pMobIndex->ac[i];
			
			current->off_flags	= pMobIndex->off_flags;
			current->imm_flags	= pMobIndex->imm_flags;
			current->res_flags	= pMobIndex->res_flags;
			current->vuln_flags	= pMobIndex->vuln_flags;
			
			current->sex		= pMobIndex->sex;
			if ( current->sex == 3 ) /* random sex */
			    current->sex = number_range( 1, 2 );
			
			current->race		= pMobIndex->race;
			current->form		= pMobIndex->form;
			current->parts		= pMobIndex->parts;
			current->size		= pMobIndex->size;
		    }
		}
	    }

	    return;
	}
    }

    send_to_char( "That's not a sub-command.\n\r", ch );
}


bool do_mobile_level(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int level;
    
    if ( !is_number(argument) )
    {
	send_to_char( "That's not a valid level.\n\r", ch );
	return FALSE;
    }
    
    level = atoi( argument );
    
    if ( level <= 0 )
    {
	send_to_char( "The level must be greater than zero.\n\r", ch );
	return FALSE;
    }
    
    if ( level > MAX_LEVEL )
    {
	send_to_char( "That level is too high.\n\r", ch );
	return FALSE;
    }
    
    kill_table[URANGE(0, pMobIndex->level, MAX_LEVEL-1)].number--;
    pMobIndex->level = level;
    kill_table[URANGE(0, pMobIndex->level, MAX_LEVEL-1)].number++;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_special(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    SPEC_FUN *spec_fun;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "What special function do you wish to give this mobile?\n\r", ch );
	return FALSE;
    }
    
    if ( !str_cmp(argument, "NULL") )
    {
	pMobIndex->spec_fun = NULL;
	
	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    
    if ( !str_prefix(argument, "list") )
    {
        char buf[MAX_STRING_LENGTH];
	int spec, col;
	
	for ( col=spec=0; spec_table[spec].name != NULL; spec++ )
	{
	    sprintf( buf, "%-25s", spec_table[spec].name );
	    send_to_char( buf, ch );
	    
	    if ( ++col % 3 == 0 )
		send_to_char( "\n\r", ch );
	}

	if ( col % 3 == 0 )
	    send_to_char( "\n\rNULL\n\r", ch );
	else
	    send_to_char( "NULL\n\r", ch );
	
	return FALSE;
    }
    
    spec_fun = spec_lookup( argument );
    if ( spec_fun == NULL )
    {
	send_to_char( "Unknown special function.\n\r", ch );
	return FALSE;
    }
    
    pMobIndex->spec_fun = spec_fun;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_immune(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> immune +<flag list>\n\r"
	              "        mob <vnum> immune -<flag list>\n\r"
	              "        mob <vnum> immune =<flag list>\n\r"
	              "        mob <vnum> immune  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, imm_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; imm_flags[pos].name != NULL; pos++ )
    {
        if ( !imm_flags[pos].settable && IS_SET(flags, imm_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        imm_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->imm_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->imm_flags, flags );
	break;
    case '=':
	pMobIndex->imm_flags = flags;
	break;
    default:
	pMobIndex->imm_flags = pMobIndex->imm_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_vuln(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> vuln +<flag list>\n\r"
	              "        mob <vnum> vuln -<flag list>\n\r"
	              "        mob <vnum> vuln =<flag list>\n\r"
	              "        mob <vnum> vuln  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, imm_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; imm_flags[pos].name != NULL; pos++ )
    {
        if ( !imm_flags[pos].settable && IS_SET(flags, imm_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        imm_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->vuln_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->vuln_flags, flags );
	break;
    case '=':
	pMobIndex->vuln_flags = flags;
	break;
    default:
	pMobIndex->vuln_flags = pMobIndex->vuln_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_resist(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> resist +<flag list>\n\r"
	              "        mob <vnum> resist -<flag list>\n\r"
	              "        mob <vnum> resist =<flag list>\n\r"
	              "        mob <vnum> resist  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, imm_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; imm_flags[pos].name != NULL; pos++ )
    {
        if ( !imm_flags[pos].settable && IS_SET(flags, imm_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        imm_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->res_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->res_flags, flags );
	break;
    case '=':
	pMobIndex->res_flags = flags;
	break;
    default:
	pMobIndex->res_flags = pMobIndex->res_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_act(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> act +<flag list>\n\r"
	              "        mob <vnum> act -<flag list>\n\r"
	              "        mob <vnum> act =<flag list>\n\r"
	              "        mob <vnum> act  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, act_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; act_flags[pos].name != NULL; pos++ )
    {
        if ( !act_flags[pos].settable && IS_SET(flags, act_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        act_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->act, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->act, flags );
	break;
    case '=':
	pMobIndex->act = flags;
	break;
    default:
	pMobIndex->act = pMobIndex->act ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_offense(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> offense +<flag list>\n\r"
	              "        mob <vnum> offense -<flag list>\n\r"
	              "        mob <vnum> offense =<flag list>\n\r"
	              "        mob <vnum> offense  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, off_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; off_flags[pos].name != NULL; pos++ )
    {
        if ( !off_flags[pos].settable && IS_SET(flags, off_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        off_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->off_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->off_flags, flags );
	break;
    case '=':
	pMobIndex->off_flags = flags;
	break;
    default:
	pMobIndex->off_flags = pMobIndex->off_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_name(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile name to what?\n\r", ch );
	return FALSE;
    }
    
    smash_tilde( argument );
    
    free_string( pMobIndex->player_name );
    pMobIndex->player_name = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_short(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile short description to what?\n\r", ch );
	return FALSE;
    }
    
    smash_tilde( argument );
    
    free_string( pMobIndex->short_descr );
    pMobIndex->short_descr = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_long(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile long description to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !str_cmp(argument, "NULL") )
    {
	free_string( pMobIndex->long_descr );
	pMobIndex->long_descr = str_dup( "" );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    
    smash_tilde( argument );
    strcpy( buf, argument );
    strcat( buf, "\n\r" );
    
    free_string( pMobIndex->long_descr );
    pMobIndex->long_descr = str_dup( buf );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_description(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile description to what?\n\r", ch );
	return FALSE;
    }
    
    smash_tilde( argument );
    
    return edit_desc( ch, &pMobIndex->description, argument );
}


bool do_mobile_damtype(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int dam_type;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile description to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(argument, "list") )
    {
	char buf[MAX_STRING_LENGTH];
	int col;
	
	for ( col=dam_type=0; attack_table[dam_type].name != NULL; dam_type++ )
	{
	    sprintf( buf, "%-18s", attack_table[dam_type].name );
	    send_to_char( buf, ch );
	    
	    if ( ++col % 4 == 0 )
		send_to_char( "\n\r", ch );
	}
	
	if ( col % 4 == 0 )
	    send_to_char( "\n\r", ch );
	
	return FALSE;
    }
    
    dam_type = attack_lookup( argument );
    if ( dam_type == 0 )
    {
	send_to_char( "Invalid damage type.\n\r", ch );
	return FALSE;
    }
    
    pMobIndex->dam_type = dam_type;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_damage(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int number, dice, bonus;
    
    if ( get_dice(argument, &number, &dice, &bonus) )
    {
	if ( number <= 0 || dice <= 0 )
	{
	send_to_char( "Both the number and type of dice must be positive.\n\r", ch );
	return FALSE;
	}
	
	pMobIndex->damage[DICE_NUMBER]	= number;
	pMobIndex->damage[DICE_TYPE]	= dice;
	pMobIndex->damage[DICE_BONUS]	= bonus;

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    else
    {
	send_to_char( 
	    "Syntax: mob <vnum> damage <number dice>d<dice type>+<bonus>\n\r"
	    "        mob <vnum> damage <number dice>d<dice type>-<bonus>\n\r",
	    ch );
	return FALSE;
    }
}


bool do_mobile_hitroll(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile hitroll to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !is_number(argument) )
    {
	send_to_char( "Invalid hitroll.\n\r", ch );
	return FALSE;
    }
    
    pMobIndex->hitroll = atoi( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_sex(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int sex;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile sex to what?\n\r", ch );
	return FALSE;
    }
    
    sex = sex_lookup( argument );
    if ( sex == -1 )
    {
	send_to_char( "That's not a valid sex.\n\r", ch );
	return FALSE;
    }
    
    pMobIndex->sex = sex;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_hp(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int number, dice, bonus;
    
    if ( get_dice(argument, &number, &dice, &bonus) )
    {
	if ( number <= 0 || dice <= 0 )
	{
	send_to_char( "Both the number and type of dice must be positive.\n\r", ch );
	return FALSE;
	}
	
	pMobIndex->hit[DICE_NUMBER]	= number;
	pMobIndex->hit[DICE_TYPE]	= dice;
	pMobIndex->hit[DICE_BONUS]	= bonus;

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    else
    {
	send_to_char( 
	    "Syntax: mob <vnum> hp <number dice>d<dice type>+<bonus>\n\r"
	    "        mob <vnum> hp <number dice>d<dice type>-<bonus>\n\r",
	    ch );
	return FALSE;
    }
}


bool do_mobile_mana(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int number, dice, bonus;
    
    if ( get_dice(argument, &number, &dice, &bonus) )
    {
	if ( number <= 0 || dice <= 0 )
	{
	send_to_char( "Both the number and type of dice must be positive.\n\r", ch );
	return FALSE;
	}
	
	pMobIndex->mana[DICE_NUMBER]	= number;
	pMobIndex->mana[DICE_TYPE]	= dice;
	pMobIndex->mana[DICE_BONUS]	= bonus;

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    else
    {
	send_to_char( 
	    "Syntax: mob <vnum> mana <number dice>d<dice type>+<bonus>\n\r"
	    "        mob <vnum> mana <number dice>d<dice type>-<bonus>\n\r",
	    ch );
	return FALSE;
    }
}


bool do_mobile_race( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int race, oldRace;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mobile race to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(argument, "list") )
    {
	char buf[MAX_STRING_LENGTH];
	int col;
	
	for ( col=0, race=1; race_table[race].name != NULL; race++ )
	{
	    sprintf( buf, "%-15s", race_table[race].name );
	    send_to_char( buf, ch );
	    
	    if ( ++col % 5 == 0 )
		send_to_char( "\n\r", ch );
	}
	
	if ( col % 5 == 0 )
	    send_to_char( "\n\r", ch );
	
	return FALSE;
    }
    
    race = race_lookup( argument );
    if ( race == 0 && str_prefix(argument, "unique") )
    {
	send_to_char( "That's not a valid race.\n\r", ch );
	return FALSE;
    }
    
    oldRace = pMobIndex->race;
    pMobIndex->race = race;

    pMobIndex->act 		^= race_table[oldRace].act;
    pMobIndex->affected_by	^= race_table[oldRace].aff;
    pMobIndex->affected_by2	^= race_table[oldRace].aff2;
    pMobIndex->off_flags	^= race_table[oldRace].off;
    pMobIndex->imm_flags	^= race_table[oldRace].imm;
    pMobIndex->res_flags	^= race_table[oldRace].res;
    pMobIndex->vuln_flags	^= race_table[oldRace].vuln;

    pMobIndex->act 		|= race_table[race].act;
    pMobIndex->affected_by	|= race_table[race].aff;
    pMobIndex->affected_by2	|= race_table[race].aff2;
    pMobIndex->off_flags	|= race_table[race].off;
    pMobIndex->imm_flags	|= race_table[race].imm;
    pMobIndex->res_flags	|= race_table[race].res;
    pMobIndex->vuln_flags	|= race_table[race].vuln;

    pMobIndex->form		= race_table[race].form;
    pMobIndex->parts		= race_table[race].parts;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_auto( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int level, ac_mod;
    
    level = pMobIndex->level;
    if ( IS_SET(pMobIndex->act, ACT_WARRIOR) )
	level++;
    else if ( IS_SET(pMobIndex->act, ACT_CLERIC) )
	;
    else if ( IS_SET(pMobIndex->act, ACT_THIEF|ACT_MAGE) )
	level--;
    level = URANGE( 0, level - 1, MAX_LEVEL - 1 );
    pMobIndex->hit[0] = mobile_autos[level].hits[0];
    pMobIndex->hit[1] = mobile_autos[level].hits[1];
    pMobIndex->hit[2] = mobile_autos[level].hits[2];
    
    level = pMobIndex->level;
    if ( IS_SET(pMobIndex->act, ACT_MAGE|ACT_CLERIC) )
	level++;
    else if ( IS_SET(pMobIndex->act, ACT_THIEF|ACT_MAGE) )
	level--;
    level = URANGE( 0, level - 1, MAX_LEVEL - 1 );
    pMobIndex->mana[0] = mobile_autos[level].mana[0];
    pMobIndex->mana[1] = mobile_autos[level].mana[1];
    pMobIndex->mana[2] = mobile_autos[level].mana[2];
    
    level = pMobIndex->level;
    if ( IS_SET(pMobIndex->act, ACT_WARRIOR) )
	;
    else if ( IS_SET(pMobIndex->act, ACT_THIEF) )
	level--;
    else if ( IS_SET(pMobIndex->act, ACT_CLERIC) )
	level -= 2;
    else if ( IS_SET(pMobIndex->act, ACT_MAGE) )
	level -= 3;
    level = URANGE( 0, level - 1, MAX_LEVEL - 1 );
    pMobIndex->damage[0] = mobile_autos[level].damage[0];
    pMobIndex->damage[1] = mobile_autos[level].damage[1];
    pMobIndex->damage[2] = mobile_autos[level].damage[2];
    
    level  = pMobIndex->level;
    if ( IS_SET(pMobIndex->act, ACT_WARRIOR) )
	ac_mod  = 4;
    if ( IS_SET(pMobIndex->act, ACT_CLERIC) )
	ac_mod = 3;
    else if ( IS_SET(pMobIndex->act, ACT_THIEF) )
    {
	level--;
	
	ac_mod = 3;
    }
    else if ( IS_SET(pMobIndex->act, ACT_MAGE) )
    {
	level -= 1;

	ac_mod = 2;
    }
    else
	ac_mod = 4;
    level = URANGE( 0, level - 1, MAX_LEVEL - 1 );
    pMobIndex->ac[0] = mobile_autos[level].ac * 10;
    pMobIndex->ac[1] = mobile_autos[level].ac * 10;
    pMobIndex->ac[2] = mobile_autos[level].ac * 10;
    pMobIndex->ac[3] = ((mobile_autos[level].ac - 10) / ac_mod + 10) * 10;
    
    if ( IS_SET(pMobIndex->act, ACT_WARRIOR) )
	SET_BIT( pMobIndex->off_flags, OFF_BASH|OFF_BERSERK|OFF_KICK );
    if ( IS_SET(pMobIndex->act, ACT_THIEF) )
	SET_BIT( pMobIndex->off_flags, OFF_BACKSTAB|OFF_DODGE|OFF_KICK_DIRT );
    if ( IS_SET(pMobIndex->act, ACT_CLERIC) )
	SET_BIT( pMobIndex->off_flags, OFF_KICK );
    
    return TRUE;
}


bool do_mobile_limit( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    RESET_DATA *pReset;
    int limit;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mob limit to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !is_number(argument) )
    {
	send_to_char( "Invalid limit.\n\r", ch );
	return FALSE;
    }
    
    limit = atoi( argument );
    if ( limit <= 0 )
    {
	send_to_char( "The limit must be greater than zero.\n\r", ch );
	return FALSE;
    }
    
    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( pReset->command == 'M'
	&&   pReset->arg1 == pMobIndex->vnum )
	    pReset->arg2 = limit;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_roomlimit( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    RESET_DATA *pReset;
    int limit;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set mob roomlimit to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !is_number(argument) )
    {
	send_to_char( "Invalid roomlimit.\n\r", ch );
	return FALSE;
    }
    
    limit = atoi( argument );
    if ( limit <= 0 )
    {
	send_to_char( "The roomlimit must be greater than zero.\n\r", ch );
	return FALSE;
    }
    
    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( pReset->command == 'M'
	&&   pReset->arg1 == pMobIndex->vnum
	&&   pReset->arg3 == ch->in_room->vnum )
	    pReset->arg4 = limit;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_clone( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int room_limit, limit, mobiles, room_mobiles;
    RESET_DATA *pReset;
    CHAR_DATA *pMobile;

    limit		= 0;
    mobiles		= 0;
    room_limit		= 0;
    room_mobiles	= 0;
    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( pReset->command == 'M'
	&&   pReset->arg1 == pMobIndex->vnum )
	{
	    mobiles++;
	    limit = UMAX( limit, pReset->arg2 );
	    
	    if ( pReset->arg3 == ch->in_room->vnum )
	    {
		room_mobiles++;
		room_limit = UMAX( room_limit, pReset->arg4 );
	    }
	}
    }

    if ( mobiles == 0 )
    {
	limit		= 1;
	room_limit	= 1;
    }
    else if ( mobiles == limit )
        limit++;
    else if ( mobiles > limit )
        limit = mobiles + 1;

    if ( room_mobiles == room_limit )
	room_limit++;
    else if ( room_mobiles > room_limit )
	room_limit = room_mobiles + 1;

    pMobile = create_mobile( pMobIndex );
    char_to_room( pMobile, ch->in_room );

    pReset 		= create_reset( pArea, NULL );
    pReset->command	= 'M';
    pReset->arg1	= pMobIndex->vnum;
    pReset->arg2	= limit;
    pReset->arg3	= ch->in_room->vnum;
    pReset->arg4	= room_limit;
    
    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( pReset->command == 'M'
	&&   pReset->arg1 == pMobIndex->vnum )
	{
	    pReset->arg2 = limit;
		
	    if ( pReset->arg3 == ch->in_room->vnum )
		pReset->arg4 = room_limit;
	}
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_purge( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    RESET_DATA *pReset;
    
    if ( argument[0] != '\0' )
    {
	if ( !str_cmp(argument, "all") )
	    remove_resets( pArea, 'M', pMobIndex->vnum, 0 );
	    
	else if ( !str_cmp(argument, "room") )
	    remove_resets( pArea, 'M', pMobIndex->vnum, ch->in_room->vnum );
	    
	else
	{
	    send_to_char( "Unknown option.\n\r", ch );
	    return FALSE;
	}
    }
    else
    {
	pReset = find_reset( pArea, 'M', pMobIndex->vnum, ch->in_room->vnum );
	if ( pReset == NULL )
	{
	    send_to_char( "There was no reset for that mobie to remove.\n\r", ch );
	    return FALSE;
	}

	remove_reset( pArea, pReset );

	for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
	{
	    if ( pReset->command == 'M'
	    &&   pReset->arg1 == pMobIndex->vnum
	    &&   pReset->arg3 == ch->in_room->vnum )
		pReset->arg4--;
	}
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_resets( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    RESET_DATA *pReset;
    
    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( pReset->command == 'M'
	&&   pReset->arg1 == pMobIndex->vnum )
	{
	    sprintf( buf, "Room: %5d, limit = %3d, room limit = %3d.\n\r",
	        pReset->arg3, pReset->arg2, pReset->arg4 );
	    send_to_char( buf, ch );
	        
	    while ( pReset != NULL )
	    {
		if ( pReset->next == NULL )
		    break;
		    
		pReset = pReset->next;
		if ( pReset->command == 'E' )
		{
		    sprintf( buf, "  Equip: vnum = %5d.\n\r", pReset->arg1 );
		    send_to_char( buf, ch );
		}
		else if ( pReset->command == 'G' )
		{
		    sprintf( buf, "  Give:  vnum = %5d.\n\r", pReset->arg1 );
		    send_to_char( buf, ch );
		}
		else
		    break;
	    }
	}
    }
    
    send_to_char( "OK.\n\r", ch );
    return FALSE;
}


bool do_mobile_equip( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    OBJ_INDEX_DATA *pObjIndex;
    bool all, inventory;
    RESET_DATA *pReset;
    CHAR_DATA *mobile;
    OBJ_DATA *object;
    int location;
    int side;

    argument = one_argument( argument, arg );
    if ( is_number(arg) )
    {
	if ( (pObjIndex = get_obj_index(atoi(arg))) == NULL )
	{
	    send_to_char( "No object with that vnum found.\n\r", ch );
	    return FALSE;
	}

	if ( pMobIndex->vnum < pArea->min_vnum
	||   pMobIndex->vnum > pArea->max_vnum )
	{
	    send_to_char( "That object a part of this area.\n\r", ch );
	    return FALSE;
	}
    }
    else
    {
	int vnum;
	
	pObjIndex = NULL;
	
	for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
	{
	    if ( (pObjIndex = get_obj_index(vnum)) != NULL
	    &&   is_name(arg, pObjIndex->name) )
		break;
	    
	    pObjIndex = NULL;
	}
	
	if ( pObjIndex == NULL )
	{
	    send_to_char( "No object with that name found in this area.\n\r", ch );
	    return FALSE;
	}
    }
    
    side = SIDE_ANY;
    all  = inventory = FALSE;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	    
	if ( !str_prefix(arg, "inventory") )
	    inventory = TRUE;
	else if ( !str_prefix(arg, "all") )
	    all = TRUE;
	else if ( !str_prefix(arg, "left") )
	    side = SIDE_LEFT;
	else if ( !str_prefix(arg, "right") ) 
	    side = SIDE_RIGHT;
	else
	{
	    sprintf( buf, "Unknown option: %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }

    if ( inventory )
	location = WEAR_NONE;
    else
    {
	location = wear_location( pObjIndex, side );
	if ( location == WEAR_NONE )
	{
	    send_to_char( "That item cannot be equiped.\n\r", ch );
	    return FALSE;
	}
    }
    
    if ( all )
    {
	for ( pReset = pArea->reset_first; pReset; pReset = pReset->next )
	{
	    if ( pReset->command == 'M'
	    &&   pReset->arg1 == pMobIndex->vnum )
	    {
		RESET_DATA *equip_reset = NULL;
		RESET_DATA *add_after = pReset;
		
		while ( add_after->next != NULL )
		{
		    if ( add_after->next->command == 'E'
		    &&   location != WEAR_NONE )
		    {
			if ( add_after->next->arg3 == location )
			{
			    equip_reset = add_after->next;
			    break;
			}
		    }
		    else if ( add_after->next->command != 'G' )
		    {
			equip_reset = create_reset( pArea, add_after );
			break;
		    }
		    
		    add_after = add_after->next;
		}
		
		if ( equip_reset == NULL )
		    equip_reset = create_reset( pArea, add_after );
		    
		equip_reset->command	= (location == WEAR_NONE) ? 'G' : 'E';
		equip_reset->arg1	= pObjIndex->vnum;
		equip_reset->arg2	= 6;
		equip_reset->arg3	= (location == WEAR_NONE) 
					? 0 : location;
	    }
	}
	
	for ( mobile = pMobIndex->instances; 
              mobile != NULL; 
	      mobile = mobile->next_instance )
	{
	    object = create_object( pObjIndex, get_trust(ch), FALSE );
	    obj_to_char( object, mobile );

	    if ( location != WEAR_NONE )
	    {
		OBJ_DATA *existing = get_eq_char( mobile, location );
		if ( existing )
		    extract_obj( existing );

		equip_char( mobile, object, location );
	    }
	}
    }
    else
    {
	RESET_DATA *equip_reset = NULL;
	RESET_DATA *add_after;

	pReset = find_reset( pArea, 'M', pMobIndex->vnum, ch->in_room->vnum );
	if ( pReset == NULL )
	{
	    bug( "Do_mobile_equip: missing mobile reset.", 0 );
	    
	    send_to_char( 
	       "There is no mobile reset to precede this equip reset.", ch );
	    
	    return FALSE;
	}
	
	add_after = pReset;
	while ( add_after->next != NULL )
	{
	    if ( add_after->next->command == 'E'
	    &&   location != WEAR_NONE )
	    {
		if ( add_after->next->arg3 == location )
		{
		    equip_reset = add_after->next;
		    break;
		}
	    }
	    else if ( add_after->next->command != 'G' )
	    {
		equip_reset = create_reset( pArea, add_after );
		break;
	    }
	    
	    add_after = add_after->next;
	}

	if ( equip_reset == NULL )
	    equip_reset = create_reset( pArea, add_after );
	    
	equip_reset->command	= (location == WEAR_NONE) ? 'G' : 'E';
	equip_reset->arg1	= pObjIndex->vnum;
	equip_reset->arg2	= 6;
	equip_reset->arg3	= (location == WEAR_NONE) 
				? 0 : location;

	for ( mobile = ch->in_room->people; 
	      mobile != NULL; 
	      mobile = mobile->next_in_room )
	{
	    if ( mobile->pIndexData == pMobIndex )
	    {
		object = create_object( pObjIndex, get_trust(ch), FALSE );
		obj_to_char( object, mobile );

		if ( location != WEAR_NONE )
		{
		    OBJ_DATA *existing = get_eq_char( mobile, location );
		    if ( existing )
			extract_obj( existing );

		    equip_char( mobile, object, location );
		}

		break;
	    }
	}
    }

    pReset = find_reset( pArea, 'O', pObjIndex->vnum, ch->in_room->vnum );
    if ( pReset != NULL )
	remove_reset( pArea, pReset );

    for ( object = ch->in_room->contents; object != NULL; object = object->next_content )
    {
	if ( object->pIndexData == pObjIndex )
	{
	    extract_obj( object );
	    break;
	}
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_alignment( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int alignment;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set alignment to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !is_number(argument) )
    {
	send_to_char( "The alignment must be a number.\n\r", ch );
	return FALSE;
    }
    
    alignment = atoi( argument );
    
    if ( alignment < -1000 || alignment > 1000 )
    {
	send_to_char( "The alignment must lie from -1000 to 1000.\n\r", ch );
	return FALSE;
    }
    
    pMobIndex->alignment = alignment;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_wealth( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int wealth;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set wealth to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !is_number(argument) )
    {
	send_to_char( "The wealth must be a number.\n\r", ch );
	return FALSE;
    }
    
    wealth = atoi( argument );
    
    if ( wealth < 0 )
    {
	send_to_char( "Wealth must be non-negative.\n\r", ch );
	return FALSE;
    }
    
    pMobIndex->wealth = wealth;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_size( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    int size;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set size to what?\n\r", ch );
	return FALSE;
    }
    
    if ( (size = size_lookup(argument)) == -1 )
    {
	send_to_char( "That's not a valid size.\n\r", ch );
	return FALSE;
    }
    
    pMobIndex->size = size;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_shop( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];

    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "buy")
    &&    str_prefix(arg, "hours")
    &&    str_prefix(arg, "margin")
    &&    str_prefix(arg, "purge")) )
    {
	send_to_char( "Syntax: mob <vnum> shop buy [+] <types>\n\r"
	              "        mob <vnum> shop hours <open> <close>\n\r"
	              "        mob <vnum> shop margin <buy> <sell>\n\r"
	              "        mob <vnum> shop purge\n\r", ch );
	return FALSE;
    }

    if ( pMobIndex->pShop == NULL )
    {
	int i;
	
	pMobIndex->pShop		= new_shop();
	pMobIndex->pShop->keeper	= pMobIndex->vnum;
	for ( i=0; i<MAX_TRADE; i++ )
	    pMobIndex->pShop->buy_type[i] = 0;
	pMobIndex->pShop->profit_buy	= 125;
	pMobIndex->pShop->profit_sell	= 75;
	pMobIndex->pShop->open_hour	= 9;
	pMobIndex->pShop->close_hour	= 17;
    }

    if ( !str_prefix(arg, "buy") )
    {
	int buy_number = 0;
	
	if ( argument[0] == '+' )
	{
	    for ( buy_number = 0; buy_number < MAX_TRADE; buy_number++ )
		if ( pMobIndex->pShop->buy_type[buy_number] == 0 )
		    break;

	    argument++;
	}
	
	while ( 1 )
	{
	    int type;
	    
	    argument = one_argument( argument, arg );
	    if ( arg[0] == '\0' )
	        break;
		
	    type = item_lookup( arg );
	    if ( type == -1 )
	    {
		send_to_char( "Invalid item type.\n\r", ch );
		return FALSE;
	    }

	    if ( buy_number == MAX_TRADE )
	    {
		send_to_char( "Cannot add that many buy types.\n\r", ch );
		return FALSE;
	    }
	    
	    pMobIndex->pShop->buy_type[buy_number] = type;
	    
	    buy_number++;
	}
    }
    else if ( !str_prefix(arg, "hours") )
    {
	int open_hour, close_hour;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the shop hours to what?\n\r", ch );
	    return FALSE;
	}
	
	argument = one_argument( argument, arg );
	if ( !is_number(arg) )
	{
	    send_to_char( "The opening hour must be a number.\n\r", ch );
	    return FALSE;
	}
	
	open_hour = atoi( arg );
	if ( open_hour < 0 || open_hour > 23 )
	{
	    send_to_char( "The opening hour must be 0-23.\n\r", ch );
	    return FALSE;
	}
	
	if ( !is_number(argument) )
	{
	    send_to_char( "Missing or bad closing hour.\n\r", ch );
	    return FALSE;
	}
	
	close_hour = atoi( argument );
	if ( close_hour < 0 || close_hour > 23 )
	{
	    send_to_char( "The closing hour must be 0-23.\n\r", ch );
	    return FALSE;
	}
	
	pMobIndex->pShop->open_hour	= open_hour;
	pMobIndex->pShop->close_hour	= close_hour;
    }
    else if ( !str_prefix(arg, "margin") )
    {
	int profit_buy, profit_sell;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the margins to what?\n\r", ch );
	    return FALSE;
	}
	
	argument = one_argument( argument, arg );
	if ( !is_number(arg) )
	{
	    send_to_char( "The buy margin must be a number.\n\r", ch );
	    return FALSE;
	}
	
	profit_buy = atoi( arg );
	if ( profit_buy <= 0 )
	{
	    send_to_char( "The buy margin must be positive.\n\r", ch );
	    return FALSE;
	}
	
	if ( !is_number(argument) )
	{
	    send_to_char( "Missing or bad sell margin.\n\r", ch );
	    return FALSE;
	}
	
	profit_sell = atoi( argument );
	if ( profit_sell <= 0 )
	{
	    send_to_char( "The sell margin must be positive.\n\r", ch );
	    return FALSE;
	}
	
	pMobIndex->pShop->profit_buy	= profit_buy;
	pMobIndex->pShop->profit_sell	= profit_sell;
    }
    else if ( !str_prefix(arg, "purge") )
    {
        free_shop( pMobIndex->pShop );
        pMobIndex->pShop = NULL;
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_aggression(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> aggression +<flag list>\n\r"
	              "        mob <vnum> aggression -<flag list>\n\r"
	              "        mob <vnum> aggression =<flag list>\n\r"
	              "        mob <vnum> aggression  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, aggression_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; aggression_flags[pos].name != NULL; pos++ )
    {
        if ( !aggression_flags[pos].settable
        &&   IS_SET(flags, aggression_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        aggression_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->aggression, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->aggression, flags );
	break;
    case '=':
	pMobIndex->aggression = flags;
	break;
    default:
	pMobIndex->aggression = pMobIndex->aggression ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_affects(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags, flags2;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> affects +<flag list>\n\r"
	              "        mob <vnum> affects -<flag list>\n\r"
	              "        mob <vnum> affects =<flag list>\n\r"
	              "        mob <vnum> affects  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = flags2 = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, affect_flags );
	if ( pos == 0 )
	{
	    pos = flag_lookup( arg, affect2_flags );
	    if ( pos == 0 )
	    {
		sprintf( buf, "Bad flag %s.\n\r", arg );
		send_to_char( buf, ch );
		return FALSE;
	    }
	    SET_BIT( flags2, pos );
	}
	else
	    SET_BIT( flags, pos );
    }
    
    for ( pos = 0; affect_flags[pos].name != NULL; pos++ )
    {
        if ( !affect_flags[pos].settable 
	&&   IS_SET(flags, affect_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        affect_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    for ( pos = 0; affect2_flags[pos].name != NULL; pos++ )
    {
        if ( !affect2_flags[pos].settable 
	&&   IS_SET(flags2, affect2_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        affect2_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->affected_by, flags );
	SET_BIT( pMobIndex->affected_by2, flags2 );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->affected_by, flags );
	REMOVE_BIT( pMobIndex->affected_by2, flags2 );
	break;
    case '=':
	pMobIndex->affected_by = flags;
	pMobIndex->affected_by2 = flags2;
	break;
    default:
	pMobIndex->affected_by = pMobIndex->affected_by ^ flags;
	pMobIndex->affected_by2 = pMobIndex->affected_by2 ^ flags2;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_program(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    MPROG_LIST *pMprog;
    MPROG_CODE *prog;
    int trigger;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0'
    ||   arg2[0] == '\0'
    ||   !is_number(arg2) )
    {
	send_to_char(
	    "Syntax: mob <vnum> program <trigger> <program vnum> [program argument]\n\r"
	    "        mob <vnum> program remove <number>\n\r",
	    ch );
	return FALSE;
    }
    
    if ( !str_cmp(arg1, "remove") )
    {
	MPROG_LIST *current, *remove;
	int number;

	number = atoi( arg2 );

	for ( remove = pMobIndex->mprogs; 
	      remove != NULL; 
	      remove = remove->next )
	{
	    if ( --number == 0 )
		break;
	}

	if ( remove != NULL )
	{
	    trigger = remove->trig_type;

	    if ( pMobIndex->mprogs == remove )
	    {
		pMobIndex->mprogs = pMobIndex->mprogs->next;
		free_mob_program( remove );
	    }
	    else
	    {
		for ( current = pMobIndex->mprogs; 
		      current != NULL; 
		      current = current->next )
		{
		    if ( current->next == NULL )
		    {
			send_to_char( "Mob program not found.\n\r", ch );
			return FALSE;
		    }

		    if ( current->next == remove )
		    {
			current->next = remove->next;
			free_mob_program( remove );

			break;
		    }
		}
	    }

	    /*REMOVE_BIT( pMobIndex->mprog_flags, trigger );*/

	    send_to_char( "OK.\n\r", ch );
	    return TRUE;
	}
	
	send_to_char( "That mob doesn't have that many attached mob programs.\n\r", ch );
	return FALSE;
    }

    if ( (prog = get_mprog_index(atoi(arg2))) == NULL )
    {
	send_to_char( "Invalid mob program vnum.\n\r", ch );
	return FALSE;
    }

    trigger = flag_lookup( arg1, mprog_flags );
    if ( trigger == 0 )
    {
	char buf[MAX_STRING_LENGTH];
	int col;

	send_to_char( "Invalid trigger. Available triggers:\n\r", ch );

	for ( col=0, trigger=0; mprog_flags[trigger].name != NULL; trigger++ )
	{
	    sprintf( buf, "  %-10s", mprog_flags[trigger].name );
	    send_to_char( buf, ch );
	    
	    if ( ++col % 7 == 0 )
		send_to_char( "\n\r", ch );
	}

	if ( col % 7 == 0 )
	    send_to_char( "\n\r", ch );

	return FALSE;
    }

    pMprog		= new_mob_program();
    pMprog->next	= pMobIndex->mprogs;
    pMobIndex->mprogs	= pMprog;
    pMprog->trig_type	= trigger;
    SET_BIT( pMobIndex->mprog_flags, trigger );
    pMprog->vnum	= atoi( arg2 );
    free_string( pMprog->trig_phrase );
    pMprog->trig_phrase	= str_dup( argument );
    pMprog->code	= prog;

    return TRUE;
}


bool do_mobile_form(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> form +<flag list>\n\r"
	              "        mob <vnum> form -<flag list>\n\r"
	              "        mob <vnum> form =<flag list>\n\r"
	              "        mob <vnum> form  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, form_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; form_flags[pos].name != NULL; pos++ )
    {
        if ( !form_flags[pos].settable && IS_SET(flags, form_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        form_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->form, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->form, flags );
	break;
    case '=':
	pMobIndex->form = flags;
	break;
    default:
	pMobIndex->form = pMobIndex->form ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_parts(
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: mob <vnum> parts +<flag list>\n\r"
	              "        mob <vnum> parts -<flag list>\n\r"
	              "        mob <vnum> parts =<flag list>\n\r"
	              "        mob <vnum> parts  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, part_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; part_flags[pos].name != NULL; pos++ )
    {
        if ( !part_flags[pos].settable && IS_SET(flags, part_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        part_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pMobIndex->parts, flags );
	break;
    case '-':
	REMOVE_BIT( pMobIndex->parts, flags );
	break;
    case '=':
	pMobIndex->parts = flags;
	break;
    default:
	pMobIndex->parts = pMobIndex->parts ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_mobile_remove( 
  CHAR_DATA *ch, AREA_DATA *pArea, MOB_INDEX_DATA *pMobIndex, char *argument )
{
    RESET_DATA *pReset, *pNextReset;
    char arg[MAX_INPUT_LENGTH];
    OBJ_INDEX_DATA *pObjIndex;
    bool all, remove, removed;
    int count;

    argument = one_argument( argument, arg );
    if ( is_number(arg) )
    {
	if ( (pObjIndex = get_obj_index(atoi(arg))) == NULL )
	{
	    send_to_char( "No object with that vnum found.\n\r", ch );
	    return FALSE;
	}

	if ( pMobIndex->vnum < pArea->min_vnum
	||   pMobIndex->vnum > pArea->max_vnum )
	{
	    send_to_char( "That object a part of this area.\n\r", ch );
	    return FALSE;
	}
    }
    else
    {
	int vnum;
	
	pObjIndex = NULL;
	
	for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
	{
	    if ( (pObjIndex = get_obj_index(vnum)) != NULL
	    &&   is_name(arg, pObjIndex->name) )
		break;
	    
	    pObjIndex = NULL;
	}
	
	if ( pObjIndex == NULL )
	{
	    send_to_char( "No object with that name found in this area.\n\r", ch );
	    return FALSE;
	}
    }

    if ( is_number(argument) )
    {
	count = atoi(argument);
	if ( count <= 0 )
	{
	    send_to_char( "Invalid remove number.\n\r", ch );
	    return FALSE;
	}

	all = FALSE;
    }
    else
    {
	all	= !str_cmp( "all", argument );
	count	= 1;
    }

    remove = removed = FALSE;
    if ( all )
    {
	for ( pReset = pArea->reset_first; pReset; pReset = pNextReset )
	{
	    pNextReset = pReset->next;

	    if ( pReset->command == 'M' )
		remove = (pReset->arg1 == pMobIndex->vnum);
	    else
	    if ( (pReset->command == 'E' 
	    ||    pReset->command == 'G')
	    &&   pReset->arg1 == pObjIndex->vnum
	    &&   remove )
	    {
		remove_reset( pArea, pReset );
		removed = TRUE;
	    }
	    else if ( remove
	    &&        (pReset->command == 'M'
	    ||         (pReset->arg1 != 'E'
	    &&          pReset->arg1 != 'G')) )
		remove = FALSE;
	}
    }
    else
    {
	for ( pReset = pArea->reset_first; pReset; pReset = pNextReset )
	{
	    pNextReset = pReset->next;

	    if ( pReset->command == 'M' )
	    {
		if ( (remove = (pReset->arg1 == pMobIndex->vnum &&
		                pReset->arg3 == ch->in_room->vnum)) )
		    count--;
	    }
	    else if ( (pReset->command == 'E' 
	    ||         pReset->command == 'G')
	    &&        pReset->arg1 == pObjIndex->vnum
	    &&        count == 0
	    &&        remove )
	    {
		remove_reset( pArea, pReset );
		removed = TRUE;
		break;
	    }
	    else if ( remove
	    &&        (pReset->command == 'M'
	    ||         (pReset->command != 'E'
	    &&          pReset->command != 'G')) )
		remove = FALSE;
	}
    }
    
    if ( !removed )
    {
	if ( !all && count != 0 )
	    send_to_char( "Mob not found.\n\r", ch );
	else
	    send_to_char( "No resets removed!\n\r", ch );
	return FALSE;
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}
