/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif

#include "merc.h"
#include "db.h"
#include "olc.h"
#include "recycle.h"
#include "lookup.h"
#include "tables.h"


extern int flag_lookup  args( ( const char *name,
                                const struct flag_type *flag_table ) );


void load_olc_area( FILE *fp )
{
    AREA_DATA *pArea;

    pArea		= alloc_perm( sizeof(*pArea) );
    pArea->reset_first	= NULL;
    pArea->reset_last	= NULL;
    pArea->version	= fread_number(fp);
    pArea->file_name	= fread_string(fp);
    pArea->name		= fread_string(fp);
    pArea->credits	= fread_string(fp);
    pArea->min_vnum	= fread_number(fp);
    pArea->max_vnum	= fread_number(fp);
    pArea->flags	= fread_flag(fp);
    if ( pArea->version < 2 )
	REMOVE_BIT( pArea->flags, AREA_LAWLESS );
    pArea->creator	= fread_string(fp);
    pArea->age		= 15;
    pArea->nplayer	= 0;
    pArea->empty	= FALSE;

    if ( pArea->version < 3 )
        pArea->sound_track = str_dup( "" );
    else
        pArea->sound_track = fread_string( fp );

    if ( area_first == NULL )
	area_first = pArea;
    if ( area_last  != NULL )
	area_last->next = pArea;
    area_last	= pArea;
    pArea->next	= NULL;

    top_area++;
    return;
}


void load_olc_mobile( FILE *fp )
{
    MOB_INDEX_DATA *pMobIndex;
    int version;
    
    version = fread_number( fp );
 
    for ( ; ; )
    {
	sh_int vnum;
	char letter;
	int iHash;
 
	letter = fread_letter( fp );
	if ( letter != '#' )
	{
	    bug( "load_olc_mobile: # not found.", 0 );
	    remove_bad_area();
	    exit( 1 );
	}
 
	vnum = fread_number( fp );
	if ( vnum == 0 )
	    break;
 
	fBootDb = FALSE;
	if ( get_mob_index( vnum ) != NULL )
	{
	    bug( "Load_mobiles: vnum %d duplicated.", vnum );
	    remove_bad_area();
	    exit( 1 );
	}
	fBootDb = TRUE;
 
	pMobIndex			= alloc_perm( sizeof(*pMobIndex) );
	pMobIndex->vnum			= vnum;
	pMobIndex->area			= area_last;
	pMobIndex->new_format		= TRUE;
	newmobs++;
	pMobIndex->player_name		= fread_string( fp );
	pMobIndex->short_descr		= fread_string( fp );
	pMobIndex->long_descr		= fread_string( fp );
	pMobIndex->description		= fread_string( fp );
	pMobIndex->race			= race_lookup(fread_string( fp ));
 
	pMobIndex->long_descr[0]	= UPPER(pMobIndex->long_descr[0]);
	pMobIndex->description[0]	= UPPER(pMobIndex->description[0]);
 
	pMobIndex->act			= fread_flag( fp ) | ACT_IS_NPC
					| race_table[pMobIndex->race].act;
	if ( version <= 1 )
	    pMobIndex->aggression	= 0;
	else
	    pMobIndex->aggression	= fread_flag( fp );

	pMobIndex->affected_by		= fread_flag( fp )
					| race_table[pMobIndex->race].aff;
	if ( version <= 2 )
	    pMobIndex->affected_by2	= race_table[pMobIndex->race].aff2;
	else
	    pMobIndex->affected_by2	= fread_flag( fp )
					| race_table[pMobIndex->race].aff2;
	pMobIndex->pShop		= NULL;
	pMobIndex->alignment		= fread_number( fp );
	pMobIndex->group		= fread_number( fp );

	pMobIndex->level		= fread_number( fp );
	pMobIndex->hitroll		= fread_number( fp );  

	/* read hit dice */
	pMobIndex->hit[DICE_NUMBER]	= fread_number( fp );  
	/* 'd'		*/		  fread_letter( fp ); 
	pMobIndex->hit[DICE_TYPE]	= fread_number( fp );
	/* '+'		*/		  fread_letter( fp );	
	pMobIndex->hit[DICE_BONUS]	= fread_number( fp ); 

	/* read mana dice */
	pMobIndex->mana[DICE_NUMBER]	= fread_number( fp );
					  fread_letter( fp );
	pMobIndex->mana[DICE_TYPE]	= fread_number( fp );
					  fread_letter( fp );
	pMobIndex->mana[DICE_BONUS]	= fread_number( fp );

	/* read damage dice */
	pMobIndex->damage[DICE_NUMBER]	= fread_number( fp );
					  fread_letter( fp );
	pMobIndex->damage[DICE_TYPE]	= fread_number( fp );
					  fread_letter( fp );
	pMobIndex->damage[DICE_BONUS]	= fread_number( fp );
	pMobIndex->dam_type		= attack_lookup(fread_word(fp));

	/* read armor class */
	pMobIndex->ac[AC_PIERCE]	= fread_number( fp ) * 10;
	pMobIndex->ac[AC_BASH]		= fread_number( fp ) * 10;
	pMobIndex->ac[AC_SLASH]		= fread_number( fp ) * 10;
	pMobIndex->ac[AC_EXOTIC]	= fread_number( fp ) * 10;

	/* read flags and add in data from the race table */
	pMobIndex->off_flags		= fread_flag( fp ) 
					| race_table[pMobIndex->race].off;
	pMobIndex->imm_flags		= fread_flag( fp )
					| race_table[pMobIndex->race].imm;
	pMobIndex->res_flags		= fread_flag( fp )
					| race_table[pMobIndex->race].res;
	pMobIndex->vuln_flags		= fread_flag( fp )
					| race_table[pMobIndex->race].vuln;

	/* vital statistics */
	pMobIndex->start_pos		= position_lookup(fread_word(fp));
	pMobIndex->default_pos		= position_lookup(fread_word(fp));
	pMobIndex->sex			= sex_lookup(fread_word(fp));

	pMobIndex->wealth		= fread_number( fp );

	pMobIndex->form			= fread_flag( fp )
					| race_table[pMobIndex->race].form;
	pMobIndex->parts		= fread_flag( fp )
					| race_table[pMobIndex->race].parts;
	/* size */
	pMobIndex->size			= size_lookup(fread_word(fp));
	pMobIndex->material		= str_dup(fread_word( fp ));
 
	for ( ; ; )
        {
            letter = fread_letter( fp );

            if (letter == 'F')
            {
		char *word;
		long vector;

                word			= fread_word(fp);
		vector			= fread_flag(fp);

		if (!str_prefix(word,"act"))
		    REMOVE_BIT(pMobIndex->act,vector);
                else if (!str_prefix(word,"aff"))
		    REMOVE_BIT(pMobIndex->affected_by,vector);
                else if (!str_prefix(word,"aff2"))
		    REMOVE_BIT(pMobIndex->affected_by2,vector);
		else if (!str_prefix(word,"off"))
		    REMOVE_BIT(pMobIndex->off_flags,vector);
		else if (!str_prefix(word,"imm"))
		    REMOVE_BIT(pMobIndex->imm_flags,vector);
		else if (!str_prefix(word,"res"))
		    REMOVE_BIT(pMobIndex->res_flags,vector);
		else if (!str_prefix(word,"vul"))
		    REMOVE_BIT(pMobIndex->vuln_flags,vector);
		else if (!str_prefix(word,"for"))
		    REMOVE_BIT(pMobIndex->form,vector);
		else if (!str_prefix(word,"par"))
		    REMOVE_BIT(pMobIndex->parts,vector);
		else
		{
		    bug("Flag remove: flag not found.",0);
		    remove_bad_area();
		    exit(1);
		}
	     }
	     else if ( letter == 'M' )
	     {
		MPROG_LIST *pMprog;
		char *word;
		int trigger = 0;
		
		pMprog			= new_mob_program();
		word   			= fread_word( fp );
		if ( !(trigger = flag_lookup( word, mprog_flags)) )
		{
		    bug( "MOBprogs: invalid trigger.", 0 );
		    exit( 1 );
		}
		SET_BIT( pMobIndex->mprog_flags, trigger );
		pMprog->trig_type	= trigger;
		pMprog->vnum		= fread_number( fp );
		pMprog->trig_phrase	= fread_string( fp );

		if ( pMobIndex->mprogs != NULL )
		{
		    MPROG_LIST *tail;
		    for ( tail = pMobIndex->mprogs;
		          tail->next != NULL;
			  tail = tail->next );
		    tail->next		= pMprog;
		}
		else
		{
		    pMobIndex->mprogs	= pMprog;
		}
	     }
	     else
	     {
		ungetc(letter,fp);
		break;
	     }
	}

        iHash			= vnum % MAX_KEY_HASH;
        pMobIndex->next		= mob_index_hash[iHash];
        mob_index_hash[iHash]	= pMobIndex;
        top_mob_index++;
        kill_table[URANGE(0, pMobIndex->level, MAX_LEVEL-1)].number++;
    }
 
    return;
}


void load_olc_object( FILE *fp )
{
    OBJ_INDEX_DATA *pObjIndex;
    int version;
    
    version = fread_number( fp );
 
    for ( ; ; )
    {
        sh_int vnum;
        char letter;
        int iHash;
 
        letter = fread_letter( fp );
        if ( letter != '#' )
        {
            bug( "Load_objects: # not found.", 0 );
            remove_bad_area();
            exit( 1 );
        }
 
        vnum = fread_number( fp );
        if ( vnum == 0 )
            break;
 
        fBootDb = FALSE;
        if ( get_obj_index( vnum ) != NULL )
        {
            bug( "Load_objects: vnum %d duplicated.", vnum );
            remove_bad_area();
            exit( 1 );
        }
        fBootDb = TRUE;
 
        pObjIndex			= alloc_perm( sizeof(*pObjIndex) );
        pObjIndex->vnum			= vnum;
        pObjIndex->area			= area_last;
        pObjIndex->new_format		= TRUE;
	pObjIndex->reset_num		= 0;
	newobjs++;
        pObjIndex->name			= fread_string( fp );
        pObjIndex->short_descr		= fread_string( fp );
        pObjIndex->description		= fread_string( fp );
        pObjIndex->material		= fread_string( fp );
 
        pObjIndex->item_type		= item_lookup(fread_word( fp ));
	pObjIndex->extra_flags		= fread_flag( fp );
        pObjIndex->wear_flags		= fread_flag( fp );
	if ( IS_SET(pObjIndex->wear_flags, ITEM_OLD_NO_SAC) )
	{
	    SET_BIT(pObjIndex->extra_flags, ITEM_NO_SAC );
	    REMOVE_BIT(pObjIndex->wear_flags, ITEM_OLD_NO_SAC );
	}
	if ( version <= 2 )
	    pObjIndex->fit_flags	= 0;
	else
	    pObjIndex->fit_flags	= fread_flag( fp );

	if ( version <= 1 )
	{
	    pObjIndex->restrictions	= 0;
	    if ( IS_SET(pObjIndex->extra_flags, ITEM_ANTI_GOOD) )
		SET_BIT( pObjIndex->restrictions, CHAR_GOOD );
	    if ( IS_SET(pObjIndex->extra_flags, ITEM_ANTI_EVIL) )
		SET_BIT( pObjIndex->restrictions, CHAR_EVIL );
	    if ( IS_SET(pObjIndex->extra_flags, ITEM_ANTI_NEUTRAL) )
		SET_BIT( pObjIndex->restrictions, CHAR_NEUTRAL );
		
	    REMOVE_BIT( pObjIndex->extra_flags, ITEM_ANTI_GOOD );
	    REMOVE_BIT( pObjIndex->extra_flags, ITEM_ANTI_EVIL );
	    REMOVE_BIT( pObjIndex->extra_flags, ITEM_ANTI_NEUTRAL );
	}
	else
	    pObjIndex->restrictions	= fread_flag( fp );

	switch(pObjIndex->item_type)
	{
	case ITEM_WEAPON:
	    pObjIndex->value[0]		= weapon_type(fread_word(fp));
	    pObjIndex->value[1]		= fread_number(fp);
	    pObjIndex->value[2]		= fread_number(fp);
	    pObjIndex->value[3]		= attack_lookup(fread_word(fp));
	    pObjIndex->value[4]		= fread_flag(fp);
	    break;
	case ITEM_CONTAINER:
	    pObjIndex->value[0]		= fread_number(fp);
	    pObjIndex->value[1]		= fread_flag(fp);
	    pObjIndex->value[2]		= fread_number(fp);
	    pObjIndex->value[3]		= fread_number(fp);
	    pObjIndex->value[4]		= fread_number(fp);
	    break;
        case ITEM_DRINK_CON:
	case ITEM_FOUNTAIN:
            pObjIndex->value[0]         = fread_number(fp);
            pObjIndex->value[1]         = fread_number(fp);
            pObjIndex->value[2]         = liq_lookup(fread_word(fp));
            pObjIndex->value[3]         = fread_number(fp);
            pObjIndex->value[4]         = fread_number(fp);
            break;
	case ITEM_WAND:
	case ITEM_STAFF:
	    pObjIndex->value[0]		= fread_number(fp);
	    pObjIndex->value[1]		= fread_number(fp);
	    pObjIndex->value[2]		= fread_number(fp);
	    pObjIndex->value[3]		= skill_lookup(fread_word(fp));
	    pObjIndex->value[4]		= fread_number(fp);
	    break;
	case ITEM_POTION:
	case ITEM_PILL:
	case ITEM_SCROLL:
 	    pObjIndex->value[0]		= fread_number(fp);
	    pObjIndex->value[1]		= skill_lookup(fread_word(fp));
	    pObjIndex->value[2]		= skill_lookup(fread_word(fp));
	    pObjIndex->value[3]		= skill_lookup(fread_word(fp));
	    pObjIndex->value[4]		= skill_lookup(fread_word(fp));
	    break;
	default:
            pObjIndex->value[0]             = fread_flag( fp );
            pObjIndex->value[1]             = fread_flag( fp );
            pObjIndex->value[2]             = fread_flag( fp );
            pObjIndex->value[3]             = fread_flag( fp );
	    pObjIndex->value[4]		    = fread_flag( fp );
	    break;
	}

	pObjIndex->level		= fread_number( fp );
        pObjIndex->weight		= fread_number( fp );
        pObjIndex->cost			= fread_number( fp ); 

        /* condition */
        letter 				= fread_letter( fp );
	switch (letter)
 	{
	    case ('P') :		pObjIndex->condition = 100; break;
	    case ('G') :		pObjIndex->condition =  90; break;
	    case ('A') :		pObjIndex->condition =  75; break;
	    case ('W') :		pObjIndex->condition =  50; break;
	    case ('D') :		pObjIndex->condition =  25; break;
	    case ('B') :		pObjIndex->condition =  10; break;
	    case ('R') :		pObjIndex->condition =   0; break;
	    default:			pObjIndex->condition = 100; break;
	}
 
        for ( ; ; )
        {
            USE_MSG_DATA *use_msg;
            char letter;
 
            letter = fread_letter( fp );
 
            if ( letter == 'A' )
            {
                AFFECT_DATA *paf;
 
                paf			= new_affect();
		paf->where		= TO_OBJECT;
                paf->type		= -1;
                paf->level		= pObjIndex->level;
                paf->duration		= -1;
                paf->location		= fread_number( fp );
                paf->modifier		= fread_number( fp );
                paf->bitvector		= 0;
                paf->procNo		= 0;
                paf->flags		= 0;
		paf->name		= str_dup( "" );
		paf->appearance		= str_dup( "" );
                paf->next		= pObjIndex->affected;
                pObjIndex->affected	= paf;
                top_affect++;
            }

	    else if (letter == 'F')
            {
                AFFECT_DATA *paf;
 
                paf			= new_affect();
		letter 			= fread_letter(fp);
		switch (letter)
	 	{
		case 'A':
                    paf->where		= TO_AFFECTS;
		    break;
		case 'B':
		    paf->where		= TO_AFFECTS2;
		    break;
		case 'I':
		    paf->where		= TO_IMMUNE;
		    break;
		case 'R':
		    paf->where		= TO_RESIST;
		    break;
		case 'V':
		    paf->where		= TO_VULN;
		    break;
		default:
            	    bug( "Load_objects: Bad where on flag set.", 0 );
            	    remove_bad_area();
            	    exit( 1 );
		}
                paf->type		= -1;
                paf->level		= pObjIndex->level;
                paf->duration		= -1;
                paf->location		= fread_number(fp);
                paf->modifier		= fread_number(fp);
                paf->bitvector		= fread_flag(fp);
                paf->procNo		= 0;
                paf->flags		= 0;
		paf->name		= str_dup( "" );
		paf->appearance		= str_dup( "" );
                paf->next		= pObjIndex->affected;
                pObjIndex->affected	= paf;
                top_affect++;
            }
 
            else if ( letter == 'E' )
            {
                EXTRA_DESCR_DATA *ed;
 
                ed			= new_extra_descr();
                ed->keyword		= fread_string( fp );
                ed->description		= fread_string( fp );
                ed->next		= pObjIndex->extra_descr;
                pObjIndex->extra_descr	= ed;
                top_ed++;
            }
 
	    else if ( letter == 'R' )
	    {
		letter = fread_letter( fp );
		use_msg = new_use_msg();
		
		use_msg->next = pObjIndex->use_msgs;
		pObjIndex->use_msgs = use_msg;
	    	
	    	switch ( letter )
	    	{
	    	case 'S':
	    	use_msg->tag = USE_REMOVE_SELF;
	    	use_msg->msg = fread_string_eol( fp );
	    	break;
	    	
	    	case 'R':
	    	use_msg->tag = USE_REMOVE_ROOM;
	    	use_msg->msg = fread_string_eol( fp );
	    	break;
	    	
	    	default:
	    	bug( "Load_olc_objects: bad remove type.", 0 );
	    	remove_bad_area();
	    	exit( 1 );
	    	}
	    }
	    else if ( letter == 'W' )
	    {
	    	letter = fread_letter( fp );
		use_msg = new_use_msg();
		
		use_msg->next = pObjIndex->use_msgs;
		pObjIndex->use_msgs = use_msg;
	    	
	    	switch ( letter )
	    	{
	    	case 'S':
	    	use_msg->tag = USE_WEAR_SELF;
	    	use_msg->msg = fread_string_eol( fp );
	    	break;
	    	
	    	case 'R':
	    	use_msg->tag = USE_WEAR_ROOM;
	    	use_msg->msg = fread_string_eol( fp );
	    	break;
	    	
	    	case 'L':
	    	use_msg->tag = USE_WEAR_LOC;
	    	use_msg->msg = fread_string_eol( fp );
	    	break;
	    	
	    	default:
	    	bug( "Load_olc_objects: bad wear type.", 0 );
	    	remove_bad_area();
	    	exit( 1 );
	    	}
	    }

	    else if ( letter == 'M' )
	    {
		MPROG_LIST *pMprog;
		char *word;
		int trigger = 0;

		pMprog			= new_mob_program();
		word   			= fread_word( fp );
		if ( !(trigger = flag_lookup( word, oprog_flags)) )
		{
		    bug( "OBJprogs: invalid trigger (%s).", (long) word );
		    exit( 1 );
		}
		SET_BIT( pObjIndex->mprog_flags, trigger );
		pMprog->trig_type	= trigger;
		pMprog->vnum		= fread_number( fp );
		pMprog->trig_phrase	= fread_string( fp );

		if ( pObjIndex->mprogs != NULL )
		{
		    MPROG_LIST *tail;
		    for ( tail = pObjIndex->mprogs;
		          tail->next != NULL;
			  tail = tail->next );
		    tail->next		= pMprog;
		}
		else
		{
		    pObjIndex->mprogs	= pMprog;
		}
	    }
	    
	    else if ( letter == 'U' )
	    {
		use_msg 		= new_use_msg();
		
		use_msg->next 		= pObjIndex->use_msgs;
		pObjIndex->use_msgs 	= use_msg;
		
		use_msg->tag		= fread_number( fp );
		use_msg->msg		= fread_string( fp );
	    }

            else
            {
                ungetc( letter, fp );
                break;
            }
        }
 
        iHash			= vnum % MAX_KEY_HASH;
        pObjIndex->next		= obj_index_hash[iHash];
        obj_index_hash[iHash]	= pObjIndex;
        top_obj_index++;
    }
 
    return;
}


void load_olc_rooms( FILE *fp )
{
    ROOM_INDEX_DATA *pRoomIndex;
    PERIODIC_CHECK_DATA *pcd;
    int version;
  
    version = fread_number( fp );

    if ( area_last == NULL )
    {
	bug( "Load_resets: no #AREA seen yet.", 0 );
	remove_bad_area();
	exit( 1 );
    }

    for ( ; ; )
    {
	sh_int vnum;
	char letter;
	int door;
	int iHash;

	letter = fread_letter( fp );
	if ( letter != '#' )
	{
	    bug( "Load_rooms: # not found.", 0 );
	    remove_bad_area();
	    exit( 1 );
	}

	vnum = fread_number( fp );
	if ( vnum == 0 )
	    break;

	fBootDb = FALSE;
	if ( get_room_index( vnum ) != NULL )
	{
	    bug( "Load_rooms: vnum %d duplicated.", vnum );
	    remove_bad_area();
	    exit( 1 );
	}
	fBootDb = TRUE;

	pRoomIndex			= alloc_perm( sizeof(*pRoomIndex) );
	pRoomIndex->owner		= str_dup("");
	pRoomIndex->people		= NULL;
	pRoomIndex->contents		= NULL;
	pRoomIndex->extra_descr		= NULL;
	pRoomIndex->area		= area_last;
	pRoomIndex->vnum		= vnum;
	pRoomIndex->name		= fread_string( fp );
	pRoomIndex->description		= fread_string( fp );
	/* Area number */		  fread_number( fp );
	pRoomIndex->room_flags		= fread_flag( fp );
	/* horrible hack */
	if ( 3000 <= vnum && vnum < 3400 )
	   SET_BIT(pRoomIndex->room_flags,ROOM_LAW);
	pRoomIndex->sector_type		= fread_number( fp );
	pRoomIndex->light		= 0;
	for ( door = 0; door <= 11; door++ )
	    pRoomIndex->exit[door] = NULL;

	/* defaults */
	pRoomIndex->heal_rate = 100;
	pRoomIndex->mana_rate = 100;

	for ( ; ; )
	{
	    letter = fread_letter( fp );

	    if ( letter == 'S' )
		break;

	    if ( letter == 'H') /* healing room */
		pRoomIndex->heal_rate = fread_number(fp);
	
	    else if ( letter == 'M') /* mana room */
		pRoomIndex->mana_rate = fread_number(fp);

	   else if ( letter == 'C') /* clan */
	   {
		if (pRoomIndex->clan)
	  	{
		    bug("Load_rooms: duplicate clan fields.",0);
		    remove_bad_area();
		    exit(1);
		}
		pRoomIndex->clan = clan_lookup(fread_string(fp));
	    }
	

	    else if ( letter == 'D' )
	    {
		EXIT_DATA *pexit;
		int locks;

		door = fread_number( fp );
		if ( door < 0 || door > 11 )
		{
		    bug( "Fread_rooms: vnum %d has bad door number.", vnum );
		    remove_bad_area();
		    exit( 1 );
		}

		pexit			= alloc_perm( sizeof(*pexit) );
		pexit->description	= fread_string( fp );
		pexit->keyword		= fread_string( fp );
		pexit->exit_info	= 0;
		locks			= fread_number( fp );
		pexit->key		= fread_number( fp );
		pexit->u1.vnum		= fread_number( fp );

		switch ( locks )
		{
		case 1: pexit->exit_info = EX_ISDOOR;                break;
		case 2: pexit->exit_info = EX_ISDOOR | EX_PICKPROOF; break;
		case 3: pexit->exit_info = EX_ISDOOR | EX_NOPASS;    break;
		case 4: pexit->exit_info = EX_ISDOOR|EX_NOPASS|EX_PICKPROOF;
			break;
		case 5: pexit->exit_info = EX_ISDOOR|EX_NGBACK;	     break;
		case 6: pexit->exit_info = EX_ISDOOR|EX_NOPASS|EX_NGBACK;
			break;
		case 7: pexit->exit_info = EX_ISDOOR|EX_PICKPROOF|EX_NGBACK;
			break;
		case 8: pexit->exit_info = EX_ISDOOR|EX_NOPASS|EX_PICKPROOF|EX_NGBACK;
			break;
		}

		pRoomIndex->exit[door]	= pexit;
		pRoomIndex->old_exit[door] = pexit;
		top_exit++;
	    }
	    else if ( letter == 'E' )
	    {
		EXTRA_DESCR_DATA *ed;

		ed			= new_extra_descr();
		ed->keyword		= fread_string( fp );
		ed->description		= fread_string( fp );
		ed->next		= pRoomIndex->extra_descr;
		pRoomIndex->extra_descr	= ed;
		top_ed++;
	    }

	    else if (letter == 'O')
	    {
		if (pRoomIndex->owner[0] != '\0')
		{
		    bug("Load_rooms: duplicate owner.",0);
		    remove_bad_area();
		    exit(1);
		}

		pRoomIndex->owner = fread_string(fp);
	    }

	    else if ( letter == 'P' )
	    {
		MPROG_LIST *pMprog;
		char *word;
		int trigger = 0;

		pMprog			= new_mob_program();
		word   			= fread_word( fp );
		if ( !(trigger = flag_lookup( word, rprog_flags)) )
		{
		    bug( "ROOMprogs: invalid trigger (%s).", (long) word );
		    exit( 1 );
		}
		SET_BIT( pRoomIndex->mprog_flags, trigger );
		pMprog->trig_type	= trigger;
		pMprog->vnum		= fread_number( fp );
		pMprog->trig_phrase	= fread_string( fp );

		if ( pRoomIndex->mprogs != NULL )
		{
		    MPROG_LIST *tail;
		    for ( tail = pRoomIndex->mprogs;
		          tail->next != NULL;
			  tail = tail->next );
		    tail->next		= pMprog;
		}
		else
		{
		    pRoomIndex->mprogs	= pMprog;
		}
	    }

	    else
	    {
		bug( "Load_rooms: vnum %d has flag not 'DES'.", vnum );
		remove_bad_area();
		exit( 1 );
	    }
	}

	if ( ROOM_HAS_TRIGGER(pRoomIndex, RTRIG_RANDOM) )
	{
	    pcd			= new_periodic_check();
	    pcd->data		= (void *) pRoomIndex;
	    pcd->next		= periodic_room_checks;
	    periodic_room_checks= pcd;
	}

	iHash			= vnum % MAX_KEY_HASH;
	pRoomIndex->next	= room_index_hash[iHash];
	room_index_hash[iHash]	= pRoomIndex;
	top_room++;
    }

    return;
}


void load_olc_resets( FILE *fp )
{
    RESET_DATA *pReset;
    int version;
    
    version = fread_number( fp );

    if ( area_last == NULL )
    {
	bug( "Load_resets: no #AREA seen yet.", 0 );
	remove_bad_area();
	exit( 1 );
    }

    for ( ; ; )
    {
	ROOM_INDEX_DATA *pRoomIndex;
	OBJ_INDEX_DATA *temp_index;
	EXIT_DATA *pexit;
	char letter;

	if ( ( letter = fread_letter( fp ) ) == 'S' )
	    break;

	if ( letter == '*' )
	{
	    fread_to_eol( fp );
	    continue;
	}

	pReset		= new_reset_data();
	pReset->command	= letter;
	/* if_flag */	  fread_number( fp );
	pReset->arg1	= fread_number( fp );
	pReset->arg2	= fread_number( fp );
	pReset->arg3	= (letter == 'G' || letter == 'R')
			    ? 0 : fread_number( fp );
	pReset->arg4	= (letter == 'P' || letter == 'M' || letter == 'T')
			    ? fread_number(fp) : 0;
	pReset->arg5	= letter == 'T' ? fread_number(fp) : 0;
			  fread_to_eol( fp );

	/*
	 * Validate parameters.
	 * We're calling the index functions for the side effect.
	 */
	switch ( letter )
	{
	default:
	    bug( "Load_resets: bad command '%c'.", letter );
	    remove_bad_area();
	    exit( 1 );
	    break;

	case 'M':
	    get_mob_index  ( pReset->arg1 );
	    get_room_index ( pReset->arg3 );
	    break;

	case 'O':
	    temp_index = get_obj_index  ( pReset->arg1 );
	    temp_index->reset_num++;
	    get_room_index ( pReset->arg3 );
	    break;

	case 'P':
	    temp_index = get_obj_index  ( pReset->arg1 );
	    temp_index->reset_num++;
	    get_obj_index  ( pReset->arg3 );
	    break;

	case 'G':
	    temp_index = get_obj_index  ( pReset->arg1 );
	    temp_index->reset_num++;
	    break;

	case 'E':
	    temp_index = get_obj_index  ( pReset->arg1 );
	    temp_index->reset_num++;

	    if ( version < 3 )	/* convert wear locations */
		pReset->arg3 = convert_wear_location( pReset->arg3, version );

	    break;

	case 'D':
	    pRoomIndex = get_room_index( pReset->arg1 );

	    if ( pReset->arg2 < 0
	    ||   pReset->arg2 > 11
	    || ( pexit = pRoomIndex->exit[pReset->arg2] ) == NULL
	    || !IS_SET( pexit->exit_info, EX_ISDOOR ) )
	    {
		bug( "Load_resets: 'D': exit %d not door.", pReset->arg2 );
		remove_bad_area();
		exit( 1 );
	    }

	    if ( pReset->arg3 < 0 || pReset->arg3 > 2 )
	    {
		bug( "Load_resets: 'D': bad 'locks': %d.", pReset->arg3 );
		remove_bad_area();
		exit( 1 );
	    }

	    break;

	case 'R':
	    pRoomIndex		= get_room_index( pReset->arg1 );

	    if ( pReset->arg2 < 0 || pReset->arg2 > 6 )
	    {
		bug( "Load_resets: 'R': bad exit %d.", pReset->arg2 );
		remove_bad_area();
		exit( 1 );
	    }

	    break;
	
	case 'T':
	    if ( IS_SET(pReset->arg3, TRAP_ROOM) )
		pRoomIndex	= get_room_index( pReset->arg5 );
	    else
		temp_index	= get_obj_index( pReset->arg5 );
	    break;
	}

	if ( area_last->reset_first == NULL )
	    area_last->reset_first	= pReset;
	if ( area_last->reset_last  != NULL )
	    area_last->reset_last->next	= pReset;
	    
	area_last->reset_last	= pReset;
	pReset->next		= NULL;
	top_reset++;
    }

    return;
}


void load_olc_shops( FILE *fp )
{
    SHOP_DATA *pShop;
    int version;
    
    version = fread_number( fp );

    for ( ; ; )
    {
	MOB_INDEX_DATA *pMobIndex;
	int iTrade;

	pShop				= new_shop();
	pShop->keeper			= fread_number( fp );
	if ( pShop->keeper == 0 )
	    break;
	for ( iTrade = 0; iTrade < MAX_TRADE; iTrade++ )
	    pShop->buy_type[iTrade] 	= fread_number( fp );
	pShop->profit_buy		= fread_number( fp );
	pShop->profit_sell		= fread_number( fp );
	pShop->open_hour		= fread_number( fp );
	pShop->close_hour		= fread_number( fp );
					  fread_to_eol( fp );
	pMobIndex			= get_mob_index( pShop->keeper );
	pMobIndex->pShop		= pShop;

	if ( shop_first == NULL )
	    shop_first = pShop;
	if ( shop_last  != NULL )
	    shop_last->next = pShop;

	shop_last	= pShop;
	pShop->next	= NULL;
	top_shop++;
    }

    return;
}


void load_olc_specials( FILE *fp )
{
    int version;
    
    version = fread_number( fp );
    
    for ( ; ; )
    {
	MOB_INDEX_DATA *pMobIndex;
	char letter;

	switch ( letter = fread_letter( fp ) )
	{
	default:
	    bug( "Load_specials: letter '%c' not *MS.", letter );
	    remove_bad_area();
	    exit( 1 );

	case 'S':
	    return;

	case '*':
	    break;

	case 'M':
	    pMobIndex		= get_mob_index	( fread_number ( fp ) );
	    pMobIndex->spec_fun	= spec_lookup	( fread_word   ( fp ) );
	    if ( pMobIndex->spec_fun == 0 )
	    {
		bug( "Load_specials: 'M': vnum %d.", pMobIndex->vnum );
		remove_bad_area();
		exit( 1 );
	    }
	    break;
	}

	fread_to_eol( fp );
    }
}


/* Reset version 1 wear locations. */
#define WEAR_NONE_1		     -1
#define WEAR_LIGHT_1		      0
#define WEAR_FINGER_L_1		      1
#define WEAR_FINGER_R_1		      2
#define WEAR_NECK_1_1		      3
#define WEAR_NECK_2_1		      4
#define WEAR_BODY_1		      5
#define WEAR_HEAD_1		      6
#define WEAR_LEGS_1		      7
#define WEAR_FEET_1		      8
#define WEAR_HANDS_1		      9
#define WEAR_ARMS_1		     10
#define WEAR_SHIELD_1		     11
#define WEAR_ABOUT_1		     12
#define WEAR_WAIST_1		     13
#define WEAR_WRIST_L_1		     14
#define WEAR_WRIST_R_1		     15
#define WEAR_WIELD_1		     16
#define WEAR_HOLD_1		     17
#define WEAR_FLOAT_1		     18
#define WEAR_SECONDARY_1	     19


int convert_wear_location( int old_location, int old_version )
{
    if ( old_version == 1 )
    {
	switch ( old_location )
	{
	case WEAR_NONE_1:		return WEAR_NONE;
	case WEAR_LIGHT_1:		return WEAR_LIGHT;
	case WEAR_FINGER_L_1:		return WEAR_FINGER_L;
	case WEAR_FINGER_R_1:		return WEAR_FINGER_R;
	case WEAR_NECK_1_1:		return WEAR_NECK_1;
	case WEAR_NECK_2_1:		return WEAR_NECK_2;
	case WEAR_BODY_1:		return WEAR_BODY;
	case WEAR_HEAD_1:		return WEAR_HEAD;
	case WEAR_LEGS_1:		return WEAR_LEGS;
	case WEAR_FEET_1:		return WEAR_FEET;
	case WEAR_HANDS_1:		return WEAR_HANDS;
	case WEAR_ARMS_1:		return WEAR_ARMS;
	case WEAR_SHIELD_1:		return WEAR_SHIELD;
	case WEAR_ABOUT_1:		return WEAR_ABOUT;
	case WEAR_WAIST_1:		return WEAR_WAIST;
	case WEAR_WRIST_L_1:		return WEAR_WRIST_L;
	case WEAR_WRIST_R_1:		return WEAR_WRIST_R;
	case WEAR_WIELD_1:		return WEAR_WIELD;
	case WEAR_HOLD_1:		return WEAR_HOLD;
	case WEAR_FLOAT_1:		return WEAR_FLOAT;
	case WEAR_SECONDARY_1:		return WEAR_SECONDARY;
	default:
	    bug( "convert_wear_location: unknown version 1 wear location (%d).",
	         old_location );
	    return old_location;
	}
    }
    else if ( old_version == 2 )
    {
	if ( old_location < 7 )
	    return old_location;
	else if ( old_location == 7 )
	    return 9;
	else if ( old_location == 8 )
	    return 11;
	else if ( old_location == 9 )
	    return 8;
	else if ( old_location == 10 )
	    return 12;
	else if ( (old_location > 10) && (old_location < 21) )
	    return (old_location + 3);
	else if ( (old_location == 21) || (old_location == 22) )
	    return (old_location + 4);
	else {
	    bug( "convert_wear_location: unknown version 2 wear location (%d).",
	         old_location );
	    return old_location;
	    }
	}
    else
    {
	bug( "convert_wear_location: unsupported version (%d).", old_version );
	return old_location;
    }
}
