/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "merc.h"
#include "tables.h"
#include "recycle.h"
#include "lookup.h"


/*
 * Globals. 
 */
extern	const	sh_int	rev_dir[];
extern	char	*const	dir_name[];


/*
 * Function prototypes.
 */
int flag_lookup args( ( const char *name, 
			const struct flag_type *flag_table) );


/*
 * Type definitions.
 */
typedef	bool ROOM_FUN	args( ( CHAR_DATA *ch, char *argument ) );

struct room_command
{
    char *	name;
    ROOM_FUN	*fun;
    sh_int	olc_level;
};


/*
 * Local function prototypes.
 */
bool	do_room_name		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_exit		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_description	args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_extra		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_flags		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_sector		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_door		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_resets		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_heal		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_mana		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_owners		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_clan		args( ( CHAR_DATA *ch, char *argument ) );
bool	do_room_program		args( ( CHAR_DATA *ch, char *argument ) );


/*
 * The room command table.
 */
const struct room_command room_commands[] =
{
    { "name",		do_room_name,		1	},
    { "exit",		do_room_exit,		2	},
    { "description",	do_room_description,	1	},
    { "extra",		do_room_extra,		1	},
    { "flags",		do_room_flags,		2	},
    { "sector",		do_room_sector,		2	},
    { "door",		do_room_door,		2	},
    { "resets",		do_room_resets,		1	},
    { "heal",		do_room_heal,		2	},
    { "mana",		do_room_mana,		2	},
    { "owners",		do_room_owners,		2	},
    { "clan",		do_room_clan,		2	},
    { "program",	do_room_program,	4	},
    { NULL }
};


void do_room( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    AREA_DATA *pArea;
    int command;
    
    if ( ch->in_room == NULL )
    {
	send_to_char( "Which room?\n\r", ch );
	return;
    }
    
    pArea = ch->in_room->area;

    if ( !is_creator(ch, pArea) )
    {
	send_to_char( "You are not a creator in this area.\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg );

    for ( command = 0; room_commands[command].name != NULL; command++ )
    {
	if ( LOWER(arg[0]) == LOWER(room_commands[command].name[0])
	&&   !str_prefix(arg, room_commands[command].name))
	{
	    if ( get_olc_level(ch) < room_commands[command].olc_level )
	    {
		send_to_char( 
		    "You do not have sufficient OLC privileges to do that.\n\r", 
		    ch );
		return;
	    }
	    
	    if ( (*room_commands[command].fun) ( ch, argument ) )
		SET_BIT( pArea->flags, AREA_DIRTY );

	    return;
	}
    }

    send_to_char( "That's not a valid sub-command.\n\r", ch );
}


bool do_room_name( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *pRoom;
    
    pRoom = ch->in_room;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set this room name to what?\n\r", ch );
	return FALSE;
    }
    
    smash_tilde( argument );

    free_string( pRoom->name );
    pRoom->name = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_exit( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    EXIT_DATA *pExit;
    int destination;
    int door;

    pRoom = ch->in_room;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' || !is_number(arg2) )
    {
	send_to_char( "Syntax: room exit <direction> <room> [oneway]\n\r"
	              "        room exit <direction> 0\n\r", ch );
	return FALSE;
    }
    
    if ( is_number(arg1) )
    {
	door = atoi( arg1 );
    }
    else
    {
             if ( !str_prefix(arg1, "north")	)	door = 0;
        else if ( !str_prefix(arg1, "east")	)	door = 1;
        else if ( !str_prefix(arg1, "south")	)	door = 2;
        else if ( !str_prefix(arg1, "west")	)	door = 3;
        else if ( !str_prefix(arg1, "up")	)	door = 4;
        else if ( !str_prefix(arg1, "down")	)	door = 5;
        else if ( !str_prefix(arg1, "northeast"))	door = 6;
        else if ( !str_prefix(arg1, "southeast"))	door = 7;
        else if ( !str_prefix(arg1, "southwest"))	door = 8;
        else if ( !str_prefix(arg1, "northwest"))	door = 9;
        else if ( !str_prefix(arg1, "clockwise"))	door = 10;
        else if ( !str_prefix(arg1, "counterclockwise")) door = 11;

        else 						door = 12;
    }

    if ( door < 0 || door > 11 )
    {
	send_to_char( "Invalid direction.\n\r", ch );
	return FALSE;
    }

    destination = atoi( arg2 );
    if ( destination != 0 )
    {
	ROOM_INDEX_DATA *pDestRoom;
	bool oneway = FALSE;

	if ( (pDestRoom = get_room_index(destination)) == NULL )
	{
	    send_to_char( "Invalid destination vnum.\n\r", ch );
	    return FALSE;
	}

	if ( argument[0] != '\0' && !str_prefix(argument, "oneway") )
	    oneway = TRUE;

	pExit = pRoom->old_exit[door];
	if ( pExit == NULL )
	{
	    pExit			= alloc_perm( sizeof(*pExit) );
	    pExit->keyword		= str_dup( "" );
	    pExit->description		= str_dup( "" );
	    pRoom->exit[door]		= pExit;
	    pRoom->old_exit[door]	= pExit;
	}

	/* 
	 * Set to_room. We leave other fields as they are as this may
	 * have been an existing exit.
	 */
	pExit->u1.to_room	= pDestRoom;
	
	if ( !oneway )
	{
	    int opposite;

	    if ( !is_creator(ch, pDestRoom->area) )
	    {
		send_to_char( "You are not a creator in the destination room."
		              "The backwards exit has not be set.\n\r", ch );
		return TRUE;
	    }

	    opposite	= rev_dir[door];
	    pExit	= pDestRoom->old_exit[opposite];
	    if ( pExit == NULL )
	    {
		pExit				= alloc_perm( sizeof(*pExit) );
		pExit->keyword			= str_dup( "" );
		pExit->description		= str_dup( "" );
		pDestRoom->exit[opposite]	= pExit;
		pDestRoom->old_exit[opposite]	= pExit;
	    }

	    /* 
	     * Set to_room. We leave other fields as they are as this may
	     * have been an existing exit.
	     */
	    pExit->u1.to_room     = pRoom;
	    
	    SET_BIT( pDestRoom->area->flags, AREA_DIRTY );
	}
	
	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    
    if ( pRoom->old_exit[door] == NULL )
    {
	send_to_char( "There is not an exit in that direction to remove!\n\r", ch );
	return FALSE;
    }
    
    pRoom->old_exit[door]->u1.to_room = NULL;
        
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_description( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *pRoom;

    pRoom = ch->in_room;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set the room description to what?\n\r", ch );
	return FALSE;
    }
    
    return edit_desc( ch, &pRoom->description, argument );
}


bool do_room_extra( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    EXTRA_DESCR_DATA *ed;
    char *door_keywords;

    pRoom = ch->in_room;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: room extra <key> <description>\n\r"
	              "        room extra <key> NULL\n\r", ch );
	return FALSE;
    }
    
    smash_tilde( argument );
    
    argument = one_argument( argument, arg1 );
    
    /* look in arg1 for a direction argument */
    door_keywords = one_argument( arg1, arg2 );
    if ( !str_prefix(arg2, "north")
    ||   !str_prefix(arg2, "east")
    ||   !str_prefix(arg2, "south")
    ||   !str_prefix(arg2, "west")
    ||   !str_prefix(arg2, "up")
    ||   !str_prefix(arg2, "down")
    ||	 !str_prefix(arg2, "northeast")
    ||	 !str_prefix(arg2, "southeast")
    ||	 !str_prefix(arg2, "southwest")
    ||	 !str_prefix(arg2, "northwest")
    ||	 !str_prefix(arg2, "clockwise")
    ||	 !str_prefix(arg2, "counterclockwise") )
    {
	EXIT_DATA *pExit;
	int door;
	
	     if ( !str_prefix(arg2, "north")	)	door = 0;
        else if ( !str_prefix(arg2, "east")	)	door = 1;
        else if ( !str_prefix(arg2, "south")	)	door = 2;
        else if ( !str_prefix(arg2, "west")	)	door = 3;
        else if ( !str_prefix(arg2, "up")	)	door = 4;
        else if ( !str_prefix(arg2, "down")	)	door = 5;
        else if ( !str_prefix(arg2, "northeast"))	door = 6;
        else if ( !str_prefix(arg2, "southeast"))	door = 7;
        else if ( !str_prefix(arg2, "southwest"))	door = 8;
        else if ( !str_prefix(arg2, "northwest"))	door = 9;
        else if ( !str_prefix(arg2, "clockwise"))	door = 10;
        else						door = 11;
        
	pExit     = pRoom->old_exit[door];
	if ( pExit != NULL )
	{
	    if ( !strcmp(argument, "NULL") )
	    {
		free_string( pExit->keyword );
		pExit->keyword = str_dup( "" );
		
		free_string( pExit->description );
		pExit->description = str_dup( "" );
		
		send_to_char( "OK.\n\r", ch );
		return TRUE;
	    }

	    free_string( pExit->keyword );
	    pExit->keyword = str_dup( door_keywords );
    
	    return edit_desc( ch, &pExit->description, argument );
	}
	else
	{
	    send_to_char( "There is no exit in that direction.\n\r", ch );
	    return FALSE;
	}
    }
    
    if ( !strcmp(argument, "NULL") )
    {
	ed = find_extra( pRoom->extra_descr, arg1 );
	if ( ed == NULL )
	{
	    send_to_char( "Extra description not found.\n\r", ch );
	    return FALSE;
	}
	else
	{
	    remove_extra( &pRoom->extra_descr, ed );
	    send_to_char( "Extra description removed.\n\r", ch );
	    return TRUE;
	}
    }
    else
    {
	ed = find_extra( pRoom->extra_descr, arg1 );
	if ( ed != NULL )
	    return edit_desc( ch, &ed->description, argument );
	
	ed = new_extra_descr();
	ed->next = pRoom->extra_descr;
	pRoom->extra_descr = ed;
	
	ed->keyword = str_dup( arg1 );
	return edit_desc( ch, &ed->description, argument );
    }
}


bool do_room_flags( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    long flags;
    char type;

    pRoom = ch->in_room;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: room flags +<flag list>\n\r"
	              "        room flags -<flag list>\n\r"
	              "        room flags =<flag list>\n\r"
	              "        room flags  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	int pos;
	
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, room_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pRoom->room_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pRoom->room_flags, flags );
	break;
    case '=':
	pRoom->room_flags = flags;
	break;
    default:
	pRoom->room_flags = pRoom->room_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_sector( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    int sector;

    pRoom = ch->in_room;

    if ( argument[0] == '\0' )
    {
	send_to_char( "What sector type do you want this room to have?\n\r", ch );
	return FALSE;
    }
    
    one_argument( argument, arg );
    
    for ( sector = 0; sector_table[sector].name != NULL; sector++ )
    {
	if ( LOWER(arg[0]) == LOWER(sector_table[sector].name[0])
	&&   !str_prefix(arg, sector_table[sector].name) )
	{
	    pRoom->sector_type = sector;
	    
	    send_to_char( "OK.\n\r", ch );
	    return TRUE;
	}
    }

    send_to_char( "That's not a valid sector type.\n\r", ch );
    return FALSE;
}


bool do_room_door( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *pRoom, *pDest = NULL;
    EXIT_DATA *pExit, *pReverse = NULL;
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    char arg3[MAX_INPUT_LENGTH];
    RESET_DATA *pReset;
    bool oneway;
    int door;

    pRoom = ch->in_room;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    argument = one_argument( argument, arg3 );
    
    if ( arg1[0] == '\0' || arg2[0] == '\0'
    ||   (!str_cmp(arg2, "key") && !is_number(arg3)) )
    {
	send_to_char( "Syntax: room door <direction> none [oneway]\n\r"
	              "        room door <direction> open [oneway]\n\r"
	              "        room door <direction> closed [oneway]\n\r"
	              "        room door <direction> locked [oneway]\n\r"
	              "        room door <direction> passproof [oneway]\n\r"
	              "        room door <direction> pickproof [oneway]\n\r"
		      "        room door <direction> stay [oneway]\n\r"
	              "        room door <direction> key <vnum> [oneway]\n\r", ch );
	return FALSE;
    }

    if ( is_number(arg1) )
	door = atoi( arg1 );
    else
    {
	     if ( !str_prefix(arg1, "north")	)	door = 0;
	else if ( !str_prefix(arg1, "east")	)	door = 1;
	else if ( !str_prefix(arg1, "south")	)	door = 2;
	else if ( !str_prefix(arg1, "west")	)	door = 3;
	else if ( !str_prefix(arg1, "up")	)	door = 4;
	else if ( !str_prefix(arg1, "down")	)	door = 5;
        else if ( !str_prefix(arg1, "northeast"))	door = 6;
        else if ( !str_prefix(arg1, "southeast"))	door = 7;
        else if ( !str_prefix(arg1, "southwest"))	door = 8;
        else if ( !str_prefix(arg1, "northwest"))	door = 9;
        else if ( !str_prefix(arg1, "clockwise"))	door = 10;
        else if ( !str_prefix(arg1, "counterclockwise")) door = 11;

        else 						door = 12;
    }
    
    if ( door < 0 || door > 11 )
    {
	send_to_char( "Invalid direction.\n\r", ch );
	return FALSE;
    }

    if ( str_cmp(arg2, "key") 
    &&   arg3[0] != '\0' 
    &&   !str_prefix(arg3, "oneway") )
	oneway = TRUE;
    else 
    if ( !str_cmp(arg2, "key") 
    &&   argument[0] != '\0' 
    &&   !str_prefix(argument, "oneway") )
	oneway = TRUE;
    else
	oneway = FALSE;
    
    pExit = pRoom->old_exit[door];
    if ( pExit == NULL  )
    {
	send_to_char( "There is not an exit in that direction.\n\r", ch );
	return FALSE;
    }
    
    if ( !oneway )
    {
	if ( (pDest = pExit->u1.to_room) == NULL )
	{
	    send_to_char( "The exit doesn't go anywhere!\n\r", ch );
	    return FALSE;
	}
	
	if ( !is_creator(ch, pDest->area) )
	{
	    send_to_char( "You are not a creator in the destination room.\n\r"
	                  "The reverse door cannot be modified.\n\r", ch );
	}
	else if ( (pReverse = pDest->old_exit[rev_dir[door]]) == NULL )
	{
	    send_to_char( 
		"The exit is one way. The door modifications will only be applied to this room\n\r",
		ch );
	}
    }
	
    if ( pReverse )
	SET_BIT( pDest->area->flags, AREA_DIRTY );
    
    if ( !str_prefix(arg2, "none") )
    {
	remove_resets( pRoom->area, 'D', pRoom->vnum, door );
	pExit->exit_info     = 0;
	
	if ( pReverse )
	{
	    remove_resets( pDest->area, 'D', pDest->vnum, rev_dir[door] );
	    pReverse->exit_info     = 0;
	}
    }
    else if ( !str_prefix(arg2, "open") )
    {
	SET_BIT( pExit->exit_info, EX_ISDOOR );
	REMOVE_BIT( pExit->exit_info, EX_CLOSED|EX_LOCKED );

	pReset = find_reset( pRoom->area, 'D', pRoom->vnum, door );
	if ( pReset == NULL )
	{
	    pReset = create_reset( pRoom->area, NULL );
	    
	    pReset->command = 'D';
	    pReset->arg1    = pRoom->vnum;
	    pReset->arg2    = door;
	}
	    
	pReset->arg3 = 0;
	
	if ( pReverse )
	{
	    SET_BIT( pReverse->exit_info, EX_ISDOOR );
	    REMOVE_BIT( pReverse->exit_info, EX_CLOSED|EX_LOCKED );
	    
	    pReset = find_reset( pDest->area, 'D', pDest->vnum, rev_dir[door] );
	    if ( pReset == NULL )
	    {
		pReset = create_reset( pDest->area, NULL );
	    
		pReset->command = 'D';
		pReset->arg1    = pDest->vnum;
		pReset->arg2    = rev_dir[door];
	    }
	    
	    pReset->arg3 = 0;
	}
    }
    else if ( !str_prefix(arg2, "closed") )
    {
	SET_BIT( pExit->exit_info, EX_ISDOOR|EX_CLOSED );
	REMOVE_BIT( pExit->exit_info, EX_LOCKED );
	
	pReset = find_reset( pRoom->area, 'D', pRoom->vnum, door );
	if ( pReset == NULL )
	{
	    pReset = create_reset( pRoom->area, NULL );
	    
	    pReset->command = 'D';
	    pReset->arg1    = pRoom->vnum;
	    pReset->arg2    = door;
	}
	    
	pReset->arg3 = 1;
	
	if ( pReverse )
	{
	    SET_BIT( pReverse->exit_info, EX_ISDOOR|EX_CLOSED );
	    REMOVE_BIT( pReverse->exit_info, EX_LOCKED );
	
	    pReset = find_reset( pDest->area, 'D', pDest->vnum, rev_dir[door] );
	    if ( pReset == NULL )
	    {
		pReset = create_reset( pDest->area, NULL );
	    
		pReset->command = 'D';
		pReset->arg1    = pDest->vnum;
		pReset->arg2    = rev_dir[door];
	    }
	    
	    pReset->arg3 = 1;
	}
    }
    else if ( !str_prefix(arg2, "locked") )
    {
	SET_BIT( pExit->exit_info, EX_ISDOOR|EX_CLOSED|EX_LOCKED );
	
	pReset = find_reset( pRoom->area, 'D', pRoom->vnum, door );
	if ( pReset == NULL )
	{
	    pReset = create_reset( pRoom->area, NULL );
	    
	    pReset->command = 'D';
	    pReset->arg1    = pRoom->vnum;
	    pReset->arg2    = door;
	}
	    
	pReset->arg3 = 2;
	
	if ( pReverse )
	{
	    SET_BIT( pReverse->exit_info, EX_ISDOOR|EX_CLOSED|EX_LOCKED );
	
	    pReset = find_reset( pDest->area, 'D', pDest->vnum, rev_dir[door] );
	    if ( pReset == NULL )
	    {
		pReset = create_reset( pDest->area, NULL );
	    
		pReset->command = 'D';
		pReset->arg1    = pDest->vnum;
		pReset->arg2    = rev_dir[door];
	    }
	    
	    pReset->arg3 = 2;
	}
    }
    else if ( !str_prefix(arg2, "passproof") )
    {
	if ( IS_SET(pExit->exit_info, EX_NOPASS) )
	{
	    REMOVE_BIT( pExit->exit_info, EX_NOPASS );

	    if ( pReverse )
		REMOVE_BIT( pReverse->exit_info, EX_NOPASS );
	}
	else
	{
	    SET_BIT( pExit->exit_info, EX_ISDOOR|EX_NOPASS );

	    if ( pReverse )
		SET_BIT( pReverse->exit_info, EX_ISDOOR|EX_NOPASS );
	}
    }
    else if ( !str_prefix(arg2, "pickproof") )
    {
	if ( IS_SET(pExit->exit_info, EX_PICKPROOF) )
	{
	    REMOVE_BIT( pExit->exit_info, EX_PICKPROOF );

	    if ( pReverse )
		REMOVE_BIT( pReverse->exit_info, EX_PICKPROOF );
	}
	else
	{
	    SET_BIT( pExit->exit_info, EX_ISDOOR|EX_PICKPROOF );

	    if ( pReverse )
		SET_BIT( pReverse->exit_info, EX_ISDOOR|EX_PICKPROOF );
	}
    }
    else if ( !str_prefix(arg2, "stay") )
    {
	if ( IS_SET(pExit->exit_info, EX_NGBACK) )
	{
	    REMOVE_BIT( pExit->exit_info, EX_NGBACK );

	    if ( pReverse )
		REMOVE_BIT( pReverse->exit_info, EX_NGBACK );
	}
	else
	{
	    SET_BIT( pExit->exit_info, EX_ISDOOR|EX_NGBACK );

	    if ( pReverse )
		SET_BIT( pReverse->exit_info, EX_ISDOOR|EX_NGBACK );
	}
    }
    else if ( !str_prefix(arg2, "key") )
    {
	int vnum = atoi( arg3 );
	
	pExit->key     = vnum;
	
	if ( pReverse )
	    pReverse->key     = vnum;
    }
    else
    {
	send_to_char( "Unknown door sub-command.\n\r", ch );
	return FALSE;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_resets( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    RESET_DATA *pReset;
    bool last = FALSE;

    pRoom = ch->in_room;
    
    for ( pReset = pRoom->area->reset_first; 
          pReset != NULL; 
          pReset = pReset->next )
    {
        
	switch ( pReset->command )
	{
	case 'M':
	    if ( pReset->arg3 == pRoom->vnum )
	    {
		sprintf( buf, 
		    "Mobile: vnum = %d, limit = %d, room limit = %d.\n\r",
		    pReset->arg1, pReset->arg2, pReset->arg4 );
		send_to_char( buf, ch );
		last = TRUE;
	    }
	    else
		last = FALSE;
	    break;
	case 'O':
	    if ( pReset->arg3 == pRoom->vnum )
	    {
		sprintf( buf, "Object: vnum = %d.\n\r", pReset->arg1 );
		send_to_char( buf, ch );
	    }
	    last = FALSE;
	    break;
	case 'G':
	    if ( last )
	    {
		sprintf( buf, "  Give:  vnum = %d.\n\r", pReset->arg1 );
		send_to_char( buf, ch );
	    }
	    break;
	case 'E':
	    if ( last )
	    {
		sprintf( buf, "  Equip: vnum = %d.\n\r", pReset->arg1 );
		send_to_char( buf, ch );
	    }
	    break;
	case 'D':
	    if ( pReset->arg1 == pRoom->vnum )
	    {
		sprintf( buf, "Door: direction = %s, State = ",
		    dir_name[pReset->arg2] );
		    
		if ( pReset->arg3 == 0 )
		    strcat( buf, "Open" );
		else if ( pReset->arg3 == 1 )
		    strcat( buf, "Closed" );
		else if ( pReset->arg3 == 2 )
		    strcat( buf, "Locked" );
		else
		    strcat( buf, "Unknown" );
		
		strcat( buf, ".\n\r" );
		send_to_char( buf, ch );
	    }
	    last = FALSE;
	    break;
	case 'R':
	    if ( pReset->arg1 == pRoom->vnum )
	    {
		sprintf( buf, "Random: max door = %s.\n\r", 
		    dir_name[pReset->arg2] );
		send_to_char( buf, ch );
	    }
	    last = FALSE;
	    break;
	default:
	    last = FALSE;
	    break;
	}
    }
    return FALSE;
}


bool do_room_mana( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    int mana;

    pRoom = ch->in_room;

    one_argument( argument, arg );

    if ( arg[0] == '\0' || !is_number(arg) )
    {
	send_to_char( "What mana rate do you want this room to have?\n\r", ch );
	return FALSE;
    }
    
    mana = atoi( arg );
    if ( mana < 0 )
    {
	send_to_char( "Invalid mana rate.\n\r", ch );
	return FALSE;
    }

    pRoom->mana_rate = mana;
	    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_heal( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    int heal;

    pRoom = ch->in_room;

    one_argument( argument, arg );

    if ( arg[0] == '\0' || !is_number(arg) )
    {
	send_to_char( "What healing rate do you want this room to have?\n\r", ch );
	return FALSE;
    }
    
    heal = atoi( arg );
    if ( heal < 0 )
    {
	send_to_char( "Invalid healing rate.\n\r", ch );
	return FALSE;
    }

    pRoom->heal_rate = heal;
	    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_owners( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *pRoom;

    pRoom = ch->in_room;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: room owners <name list>\n\r"
	              "        room owners NULL\n\r", ch );
	return FALSE;
    }
    
    if ( !str_cmp(argument, "NULL") )
    {
	free_string( pRoom->owner );
	pRoom->owner = str_dup( "" );
	
	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    
    smash_tilde( argument );
    
    free_string( pRoom->owner );
    pRoom->owner = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_clan( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *pRoom;

    pRoom = ch->in_room;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: room clan <clan name>\n\r"
	              "        room clan none\n\r", ch );
	return FALSE;
    }
    
    if ( !str_cmp(argument, "none") )
    {
	pRoom->clan = 0;
    }
    else
    {
	int clan = clan_lookup( argument );
	
	if ( clan == 0 )
	{
	    send_to_char( "Invalid clan name.\n\r", ch );
	    return FALSE;
	}
	
	pRoom->clan = clan;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_room_program( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *pRoom;
    MPROG_LIST *pMprog;
    MPROG_CODE *prog;
    bool random_trig;
    int trigger;

    pRoom = ch->in_room;
    random_trig = ROOM_HAS_TRIGGER( pRoom, RTRIG_RANDOM );

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0'
    ||   arg2[0] == '\0'
    ||   !is_number(arg2) )
    {
	send_to_char(
	    "Syntax: room <vnum> program <trigger> <program vnum> [program argument]\n\r"
	    "        room <vnum> program remove <number>\n\r",
	    ch );
	return FALSE;
    }
    
    if ( !str_cmp(arg1, "remove") )
    {
	MPROG_LIST *current, *remove;
	int number;

	number = atoi( arg2 );

	for ( remove = pRoom->mprogs; 
	      remove != NULL; 
	      remove = remove->next )
	{
	    if ( --number == 0 )
		break;
	}

	if ( remove != NULL )
	{
	    trigger = remove->trig_type;

	    if ( pRoom->mprogs == remove )
	    {
		pRoom->mprogs = pRoom->mprogs->next;
		free_mob_program( remove );
	    }
	    else
	    {
		for ( current = pRoom->mprogs; 
		      current != NULL; 
		      current = current->next )
		{
		    if ( current->next == NULL )
		    {
			send_to_char( "Room program not found.\n\r", ch );
			return FALSE;
		    }

		    if ( current->next == remove )
		    {
			current->next = remove->next;
			free_mob_program( remove );

			break;
		    }
		}
	    }

	    send_to_char( "OK.\n\r", ch );
	    return TRUE;
	}

	send_to_char( "The room doesn't have that many attached programs.\n\r", ch );
	return FALSE;
    }

    if ( (prog = get_mprog_index(atoi(arg2))) == NULL )
    {
	send_to_char( "Invalid program vnum.\n\r", ch );
	return FALSE;
    }

    trigger = flag_lookup( arg1, rprog_flags );
    if ( trigger == 0 )
    {
	char buf[MAX_STRING_LENGTH];
	int col;

	send_to_char( "Invalid trigger. Available triggers:\n\r", ch );

	for ( col=0, trigger=0; rprog_flags[trigger].name != NULL; trigger++ )
	{
	    sprintf( buf, "  %-10s", rprog_flags[trigger].name );
	    send_to_char( buf, ch );
	    
	    if ( ++col % 7 == 0 )
		send_to_char( "\n\r", ch );
	}

	if ( col % 7 == 0 )
	    send_to_char( "\n\r", ch );

	return FALSE;
    }

    pMprog		= new_mob_program();
    pMprog->next	= pRoom->mprogs;
    pRoom->mprogs	= pMprog;
    pMprog->trig_type	= trigger;
    SET_BIT( pRoom->mprog_flags, trigger );
    pMprog->vnum	= atoi( arg2 );
    free_string( pMprog->trig_phrase );
    pMprog->trig_phrase	= str_dup( argument );
    pMprog->code	= prog;

    if ( trigger == RTRIG_RANDOM && !random_trig )
    {
	PERIODIC_CHECK_DATA *pcd;

	pcd                 = new_periodic_check();
	pcd->data           = (void *) pRoom;
	pcd->next           = periodic_room_checks;
	periodic_room_checks= pcd;
    }

    return TRUE;
}

