/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "merc.h"
#include "db.h"
#include "olc.h"


/*
 * Local function prototypes.
 */
void fwrite_room_review		args( ( FILE *fp, AREA_DATA *pArea ) );
void fwrite_mobile_review	args( ( FILE *fp, AREA_DATA *pArea ) );
void fwrite_object_review	args( ( FILE *fp, AREA_DATA *pArea ) );


void do_areview( CHAR_DATA *ch, char *argument )
{
    char strSave[MAX_STRING_LENGTH];
    char buf[MAX_STRING_LENGTH];
    AREA_DATA *pArea;
    char *dot;
    FILE *fp;
    
    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	if ( !str_prefix(argument, pArea->name) )
	    break;
	
    if ( argument[0] == '\0' || pArea == NULL )
    {
	send_to_char( "Review which area?\n\r", ch );
	return;
    }
    
    strcpy( buf, pArea->file_name );
    dot = strstr( buf, "." );
    if ( dot != NULL )
	*dot = '\0';

    sprintf( strSave, "%s%s.rvw", AREA_REVIEWS, buf );
    
    fclose( fpReserve );
    if ( (fp = fopen(strSave, "w")) == NULL )
    {
	bug( "do_areview: fopen.", 0 );
	perror( strSave );

	fpReserve = fopen( NULL_FILE, "r" );
	return;
    }
    
    fwrite_room_review( fp, pArea );
    fwrite_mobile_review( fp, pArea );
    fwrite_object_review( fp, pArea );
        
    fclose( fp );
    fpReserve = fopen( NULL_FILE, "r" );
    
    send_to_char( "OK.\n\r", ch );
}


void fwrite_room_review( FILE *fp, AREA_DATA *pArea )
{
    ROOM_INDEX_DATA *pRoomIndex;
    bool first = TRUE;
    int vnum;

    for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
    {
	pRoomIndex = get_room_index( vnum );
	if ( pRoomIndex == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "ROOM REVIEW\n"
	                 "---- ------\n\n" );
	    
	    first = FALSE;
	}
	
	if ( pRoomIndex->heal_rate > 110 )
	    fprintf( fp, "%5d: healing rate exceeding 110 (%d)\n",
		vnum, pRoomIndex->heal_rate );
	if ( pRoomIndex->mana_rate > 110 )
	    fprintf( fp, "%5d: mana rate exceeding 110 (%d)\n",
		vnum, pRoomIndex->mana_rate );
    }

    if ( !first )
	fprintf( fp, "\n\n\n" );
}


void fwrite_mobile_review( FILE *fp, AREA_DATA *pArea )
{
    MOB_INDEX_DATA *pMobIndex;
    bool first = TRUE;
    int vnum;

    for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
    {
	pMobIndex = get_mob_index( vnum );
	if ( pMobIndex == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "MOBILE REVIEW\n"
	                 "------ ------\n\n" );
	    
	    first = FALSE;
	}
	
	if ( IS_SET(pMobIndex->off_flags, OFF_FADE) )
	    fprintf( fp, "%5d: mobile can fade\n", vnum );
	
	if ( IS_SET(pMobIndex->imm_flags, IMM_WEAPON) )
	    fprintf( fp, "%5d: mobile is immune to weapons\n", vnum );
	if ( IS_SET(pMobIndex->imm_flags, IMM_MAGIC) )
	    fprintf( fp, "%5d: mobile is immune to magic\n", vnum );
	    
	if ( IS_SET(pMobIndex->res_flags, IMM_WEAPON) )
	    fprintf( fp, "%5d: mobile is resistant to weapons\n", vnum );
	if ( IS_SET(pMobIndex->res_flags, IMM_MAGIC) )
	    fprintf( fp, "%5d: mobile is resistant to weapons\n", vnum );
	
	if ( IS_SET(pMobIndex->act, ACT_IS_HEALER) )
	    fprintf( fp, "%5d: mobile is a healer\n", vnum );
	if ( IS_SET(pMobIndex->act, ACT_TRAIN) )
	    fprintf( fp, "%5d: mobile is a trainer\n", vnum );
	if ( IS_SET( pMobIndex->act, ACT_META) )
	    fprintf( fp, "%5d: mobile is a meta-physician\n", vnum );
	
	if ( pMobIndex->hit[DICE_BONUS] < 
	     3 * mobile_autos[pMobIndex->level].hits[DICE_BONUS] / 4 )
	    fprintf( fp, "%5d: mobile low on hit points\n", vnum );
	if ( pMobIndex->mana[DICE_BONUS] < 
	     3 * mobile_autos[pMobIndex->level].mana[DICE_BONUS] / 4 )
	    fprintf( fp, "%5d: mobile low on mana\n", vnum );
	
	if ( IS_SET(pMobIndex->affected_by, AFF_BLIND) )
	    fprintf( fp, "%5d: mobile is flagged blind\n", vnum );
	if ( IS_SET(pMobIndex->affected_by, AFF_CURSE) )
	    fprintf( fp, "%5d: mobile is flagged cursed\n", vnum );
	if ( IS_SET(pMobIndex->affected_by, AFF_POISON) )
	    fprintf( fp, "%5d: mobile is flagged poison\n", vnum );
	if ( IS_SET(pMobIndex->affected_by, AFF_PLAGUE) )
	    fprintf( fp, "%5d: mobile is flagged plagued\n", vnum );
	if ( IS_SET(pMobIndex->affected_by, AFF_SLOW) )
	    fprintf( fp, "%5d: mobile is flagged slowed\n", vnum );
	if ( IS_SET(pMobIndex->affected_by, AFF_PARALYSIS) )
	    fprintf( fp, "%5d: mobile is flagged paralysed\n", vnum );
    }
    
    if ( !first )
	fprintf( fp, "\n\n\n" );
}


void fwrite_object_review( FILE *fp, AREA_DATA *pArea )
{
    OBJ_INDEX_DATA *pObjIndex;
    bool first = TRUE;
    AFFECT_DATA *paf;
    int vnum;

    for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
    {
	int applies = 0;

	pObjIndex = get_obj_index( vnum );
	if ( pObjIndex == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "OBJECT REVIEW\n"
	                 "------ ------\n\n" );
	    
	    first = FALSE;
	}
	
	for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
	{
	    applies++;

            if ( paf->where != TO_SKILL )
            {
	        switch ( paf->location )
	        {
	        case APPLY_DAMROLL:
		    if ( paf->modifier >= 6 )
		        fprintf( fp, "%5d: excessive damroll (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_HITROLL:
		    if ( paf->modifier >= 6 )
		        fprintf( fp, "%5d: excessive hitroll (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_HIT:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive hit point modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_MANA:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive mana modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_MOVE:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive movement modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_STR:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive strength modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_INT:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive intelligence modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_WIS:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive wisdom modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_DEX:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive dexterity modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_CON:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive constitution modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_PER:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive perception modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_CHA:
		    if ( paf->modifier > pObjIndex->level / 2 )
		        fprintf( fp, "%5d: excessive charisma modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_AC:
		    if ( paf->modifier > 3 + 3 * pObjIndex->level / 10 )
		        fprintf( fp, "%5d: excessive ac modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        case APPLY_SAVES:
	        case APPLY_SAVING_ROD:
	        case APPLY_SAVING_PETRI:
	        case APPLY_SAVING_BREATH:
	        case APPLY_SAVING_SPELL:
	        case APPLY_SPELL_AFFECT:
		    if ( paf->modifier < pObjIndex->level / - 10 )
		        fprintf( fp, "%5d: excessive saving throw modification (%d)\n",
			    vnum, paf->modifier );
		    break;
	        }
            }
            else
            {
                fprintf( fp, "%5d: modifies %s by %d.\n",
                    vnum, skill_table[paf->location].name, paf->modifier );
            }

            if ( paf->where == TO_IMMUNE )
                fprintf( fp, "%5d: grants immunity to: %s.\n",
                    vnum, imm_bit_name(paf->bitvector) );
            else if ( paf->where == TO_RESIST )
                fprintf( fp, "%5d: grants resistance to: %s.\n",
                    vnum, imm_bit_name(paf->bitvector) );
            else if ( paf->where == TO_AFFECTS )
                fprintf( fp, "%5d: grants affects: %s.\n",
                    vnum, affect_bit_name(paf->bitvector) );
            else if ( paf->where == TO_RESIST )
                fprintf( fp, "%5d: grants affect2s: %s.\n",
                    vnum, affect2_bit_name(paf->bitvector) );

	    
	    if ( applies > 3 )
		fprintf( fp, "%5d: object may have an excessive number of applies (%d)\n",
		    vnum, applies );
	    
	    if ( pObjIndex->item_type == ITEM_FURNITURE
	    &&   pObjIndex->value[3] > 110 )
		fprintf( fp, "%5d: excessive furniture healing rate (%d)\n",
		     vnum, pObjIndex->value[3] );

	    if ( pObjIndex->item_type == ITEM_FURNITURE
	    &&   pObjIndex->value[4] > 110 )
		fprintf( fp, "%5d: excessive furniture mana rate (%d)\n",
		     vnum, pObjIndex->value[4] );
	
	    if ( pObjIndex->item_type == ITEM_PORTAL
	    &&   IS_SET(pObjIndex->wear_flags, ITEM_TAKE) )
		fprintf( fp, "%5d: object is a portable portal!\n", vnum );

	    if ( pObjIndex->item_type == ITEM_ARMOR
	    &&   (pObjIndex->value[0] > 3 + 3 * pObjIndex->level / 10
	    ||    pObjIndex->value[1] > 3 + 3 * pObjIndex->level / 10
	    ||    pObjIndex->value[2] > 3 + 3 * pObjIndex->level / 10
	    ||    pObjIndex->value[3] > 2 +     pObjIndex->level / 14) )
		fprintf( fp, "%5d: excessive armor strength\n", vnum );

            if ( pObjIndex->item_type == ITEM_POISON )
                fprintf( fp, "%5d: items is type poison\n", vnum );
		
	    if ( pObjIndex->item_type == ITEM_ARMOR
	    &&   IS_SET(pObjIndex->wear_flags, ITEM_WEAR_FINGER) )
		fprintf( fp, "%5d: ring has object type armor\n", vnum );
	
	    if ( pObjIndex->item_type == ITEM_WEAPON
	    &&   pObjIndex->value[1] * (pObjIndex->value[2] + 1) / 2 >
	         weapon_autos[pObjIndex->level - 1].number *
	             (weapon_autos[pObjIndex->level - 1].type + 1) / 2 )
	 	fprintf( fp, "%5d: excessive weapon damage (%dd%d avg %d)\n",
	 	    vnum, 
	 	    pObjIndex->value[1],
	 	    pObjIndex->value[2],
	 	    pObjIndex->value[1] * (pObjIndex->value[2] + 1) / 2 );
	 
	     if ( pObjIndex->item_type == ITEM_WEAPON
	     &&   pObjIndex->value[1] * (pObjIndex->value[2] + 1) / 2 < 0 )
	 	fprintf( fp, "%5d: weapon seems to have negative damage.\n",
	 	    vnum );

	    if ( pObjIndex->item_type == ITEM_POTION
	    ||   pObjIndex->item_type == ITEM_SCROLL
	    ||   pObjIndex->item_type == ITEM_PILL )
	    {
		fprintf( fp, "%d: level %d potion/scroll/pill found (level %d spells",
		    vnum, pObjIndex->level, pObjIndex->value[0] );
		    
		if ( pObjIndex->value[1] >= 0 && pObjIndex->value[1] < MAX_SKILL )
		    fprintf( fp, " '%s'", skill_table[pObjIndex->value[1]].name );
		if ( pObjIndex->value[2] >= 0 && pObjIndex->value[2] < MAX_SKILL )
		    fprintf( fp, " '%s'", skill_table[pObjIndex->value[2]].name );
		if ( pObjIndex->value[3] >= 0 && pObjIndex->value[3] < MAX_SKILL )
		    fprintf( fp, " '%s'", skill_table[pObjIndex->value[3]].name );
		if ( pObjIndex->value[4] >= 0 && pObjIndex->value[4] < MAX_SKILL )
		    fprintf( fp, " '%s'", skill_table[pObjIndex->value[4]].name );
		fprintf( fp, ")\n" );
	    }
	    
	    if ( pObjIndex->item_type == ITEM_WAND
	    ||   pObjIndex->item_type == ITEM_STAFF )
	    {
		fprintf( fp, "%d: level %d wand/staff found "
		             "(%d/%d charges of level %d",
		    vnum, 
		    pObjIndex->level, 
		    pObjIndex->value[1], 
		    pObjIndex->value[2], 
		    pObjIndex->value[0] );
		    
		if ( pObjIndex->value[3] >= 0 && pObjIndex->value[3] < MAX_SKILL )
		    fprintf( fp, " '%s'", skill_table[pObjIndex->value[3]].name );
		fprintf( fp, ")\n" );
	    }
	    
	    if ( pObjIndex->weight == 0 )
		fprintf( fp, "%5d: object has zero weight\n", vnum );
	}
    }
    
    if ( !first )
	fprintf( fp, "\n\n\n" );
}
