/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <time.h>
#include "merc.h"


static const char *renown_words[] =
{
    "monster meat",
    "foolhardy",
    "lives dangerously",
    "none",
    "adventurer",
    "brave adventurer",
    "well-known adventurer",
    "heroic adventurer",
    "famed adventurer",
    "legend"
};


#define MAX_WORDS (sizeof(renown_words) / sizeof(char *))
#define KNOWN_COWARD 20

int default_renown()
{
    static bool first = TRUE;
    static int def = 0;
    
    if ( first )
    {
	int i;
	
	for ( i = 0; i < MAX_WORDS; i++ )
	{
	    if ( !str_cmp(renown_words[i], "none") )
	    {
		first = FALSE;
		break;
	    }
	}
	
	if ( i == MAX_WORDS )
	    i = 0;

	def = 5 + 100 * i / (MAX_WORDS + 1);
	
	first = FALSE;
    }
    
    return def;
}


const char *renown( CHAR_DATA *ch )
{
    if ( ch->pcdata->renown <= 0 )
	return renown_words[0];
    else if ( ch->pcdata->renown >= 100 )
	return renown_words[MAX_WORDS - 1];
	
    return renown_words[ch->pcdata->renown * (MAX_WORDS - 3) / 98 + 1];
}


/*
 * Adjust ch's renown for a kill against victim_level. This should
 * only be called for solo kills.
 */
void adjust_kill_renown( CHAR_DATA *ch, int victim_level )
{
    if ( IS_NPC(ch) )
	return;

    if ( victim_level > ch->level )
    {
	int slowdown;

	if ( ch->pcdata->renown < 0 )
	    slowdown = 1;
	else
	    slowdown = 8 * ch->pcdata->renown / 100 + 1;
	
	ch->pcdata->renown += (victim_level - ch->level + 1) / slowdown;
    }
/*
 *  else if ( victim_level < ch->level
 *  &&        ch->level < LEVEL_HERO
 *  &&        ch->pcdata->renown > KNOWN_COWARD )
 *	ch->pcdata->renown -= (ch->level - victim_level) / 15;
 */	
    ch->pcdata->renown = URANGE( -100, ch->pcdata->renown, 200 );
}


/* 
 * Adjust renown for a flee or recall from combat.
 */
void adjust_flee_renown( CHAR_DATA *ch )
{
    if ( !IS_NPC(ch) && number_range(1, 5) == 1 )
    {
	/* Very difficult to get much lower than 20 in this fashion. */
	if ( number_range(1, KNOWN_COWARD) < ch->pcdata->renown )
	{
	    ch->pcdata->renown -= 1;
	}

	ch->pcdata->renown = URANGE( -100, ch->pcdata->renown, 200 );
    }
}


/*
 * Adjust renown for a dying.
 */
void adjust_death_renown( CHAR_DATA *ch )
{
    if ( !IS_NPC(ch) )
    {
	ch->pcdata->renown -= UMAX( 1, ch->pcdata->renown / 10 );	
	ch->pcdata->renown = URANGE( -100, ch->pcdata->renown, 200 );
    }
}

