/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "music.h"
#include "crash.h"
#include "recycle.h"


/* command procedures needed */
DECLARE_DO_FUN(do_backstab	);
DECLARE_DO_FUN(do_quit		);


/*
 * Local functions.
 */
int	hit_gain	args( ( CHAR_DATA *ch ) );
int	mana_gain	args( ( CHAR_DATA *ch ) );
int	move_gain	args( ( CHAR_DATA *ch ) );
void	mobile_update	args( ( void ) );
void	weather_update	args( ( bool fullUpdate ) );
void	char_update	args( ( void ) );
void	obj_update	args( ( void ) );
void	aggr_update	args( ( void ) );
bool	will_agress	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
void    check_reboot    args( ( void ) );

/* used for saving */

int	save_number = 0;



/*
 * Advancement stuff.
 */
void advance_level( CHAR_DATA *ch, bool hide )
{
    char buf[MAX_STRING_LENGTH];
    int add_hp;
    int add_mana;
    int add_move;
    int add_prac;

    ch->pcdata->last_level = 
	( ch->played + (int) (current_time - ch->logon) ) / 3600;

    add_hp	= con_app[get_real_stat(ch,STAT_CON)/4].hitp + number_range(
		    class_table[ch->class].hp_min,
		    class_table[ch->class].hp_max );
    add_mana 	= number_range(2,(get_real_stat(ch,STAT_INT)/2
				  + get_real_stat(ch,STAT_WIS)/4)/5);
    add_mana	= (add_mana * class_table[ch->class].pMana) / 100;
    add_move	= number_range( 1, (get_real_stat(ch,STAT_CON)
				  + get_real_stat(ch,STAT_DEX))/24 );

/* Changed by Avvrax, July 27, 2001. Pracs gained per level depend on the higher of Wisdom
or constitution. The higher of the two being then applied to the wis_app.practice table.
Why constitution you may ask. A un-wise character with a high constitution is able to train
harder and longer. A wise character with low constitution is able to apply his wisdom on 
the problem better so as to effectively gain more pracs. (Better lateral thinking you could say
and attacking the problem from different angles.) However, how quickly he solves the problem
still depends on his intelligence. The question still remains of what 
about a character with high wis and high cons. Maybe I should be taking an 
average of the two
 */
    add_prac	= wis_app[UMAX( get_real_stat(ch,STAT_WIS), get_real_stat(ch, STAT_CON) )/4].practice;

    add_hp   = add_hp * 9/10;
    add_mana = add_mana * 9/10;
    add_move = add_move * 9/10;

    add_hp	= UMAX(  2, add_hp   );
    add_mana	= UMAX(  2, add_mana );
    add_move	= UMAX(  6, add_move );

    ch->max_hit 	+= add_hp;
    ch->max_mana	+= add_mana;
    ch->max_move	+= add_move;
    ch->practice	+= add_prac;
    ch->train		+= 1;

    ch->pcdata->perm_hit	+= add_hp;
    ch->pcdata->perm_mana	+= add_mana;
    ch->pcdata->perm_move	+= add_move;

    if (!hide)
    {
    	sprintf(buf,
	    "You gain %d hit point%s, %d mana, %d move, and %d practice%s.\n\r",
	    add_hp, add_hp == 1 ? "" : "s", add_mana, add_move,
	    add_prac, add_prac == 1 ? "" : "s");
	send_to_char( buf, ch );
    }

    sound_to_char( "level*", ch );
    
    if ( ch->pet
    &&   IS_NPC(ch->pet)
    &&   IS_SET(ch->pet->act, ACT_FAMILIER)
    &&   number_bits(3) != 0 )
        level_familier( ch->pet, ch->pet->level + 1 );
    return;
}   



void gain_exp( CHAR_DATA *ch, int gain )
{
    char buf[MAX_STRING_LENGTH];

    if ( IS_NPC(ch) || ch->level > LEVEL_HERO )
	return;

    ch->exp = UMAX( exp_per_level(ch, ch->pcdata->points), ch->exp + gain );
    if ( ch->pcdata->true_level == LEVEL_HERO )
	ch->exp = UMAX( 
	    ch->exp, 
	    exp_per_level(ch, ch->pcdata->points) * (LEVEL_HERO - 1) );

    while ( ch->level < LEVEL_HERO 
    &&      ch->exp >= exp_per_level(ch, ch->pcdata->points) * (ch->level+1) )
    {
	if ( ch->level < ch->pcdata->true_level )
	{
	    send_to_char( "You regain a level!!\n\r", ch );
	    ch->level += 1;
	    sprintf( buf, "%s regained level %d", ch->name, ch->level );
	    log_string( buf );
	    sprintf( buf, "$N has regained level %d!", ch->level );
	    wiznet( buf, ch, NULL, WIZ_LEVELS, 0, 0 );
	}
	else
	{
	    send_to_char( "You raise a level!!  ", ch );
	    ch->level += 1;
	    sprintf(buf,"%s gained level %d",ch->name,ch->level);
	    log_string(buf);
	    sprintf(buf,"$N has attained level %d!",ch->level);
	    wiznet(buf,ch,NULL,WIZ_LEVELS,0,0);
	    advance_level(ch,FALSE);
	    ch->pcdata->true_level = ch->level;
	    
	    if ( ch->level == LEVEL_HERO )
		add_message( "$n reaches the level of heroes!", ch, NULL, FALSE );
	}
	
	save_char_obj( ch, FALSE );
    }
    
    while ( ch->exp < exp_per_level(ch, ch->pcdata->points) * ch->level )
    {
	if ( ch->pcdata->true_level == LEVEL_HERO
	&&   ch->level < LEVEL_HERO )
	    break;	/* Heros can only lose 1 level. */

	send_to_char( "You lose a level!!\n\r", ch );
	sprintf( buf, "%s has lost level %d", ch->name, ch->level );
	log_string( buf );
	sprintf( buf, "$N has lost level %d!", ch->level );
	wiznet( buf, ch, NULL, WIZ_LEVELS, 0, 0 );
	ch->level -= 1;
	save_char_obj( ch, FALSE );
    }

    return;
}



/*
 * Regeneration stuff.
 */
int hit_gain( CHAR_DATA *ch )
{
    int gain;
    int number;

    if (ch->in_room == NULL)
	return 0;

    if ( IS_NPC(ch) )
    {
	gain =  5 + ch->level;
	if (IS_AFFECTED(ch,AFF_REGENERATION))
	    gain *= 2;

	switch(ch->position)
	{
	    default : 		gain /= 2;			break;
	    case POS_SLEEPING: 	gain = 3 * gain/2;		break;
	    case POS_RESTING:  					break;
	    case POS_FIGHTING:	gain /= 3;		 	break;
	}

	
    }
    else
    {
	int base_gain;
	
	base_gain  = UMAX(3,get_curr_stat(ch,STAT_CON)/4 - 3 + ch->level/2); 
	base_gain += class_table[ch->class].hp_max - 10;
	gain       = base_gain;
	
	number = number_percent();
	if ( number < get_skill(ch,gsn_fast_healing) )
	{
	    gain += number * gain / 100;
	    if (ch->hit < ch->max_hit)
		check_improve(ch,gsn_fast_healing,TRUE,8,TRUE);
	}
	
	if ( IS_AFFECTED(ch, AFF_REGENERATION) )
	    gain += base_gain;

	switch ( ch->position )
	{
	    default:	   	gain /= 4;			break;
	    case POS_SLEEPING: 					break;
	    case POS_RESTING:  	gain /= 2;			break;
	    case POS_FIGHTING: 	gain /= 6;			break;
	}

	if ( ch->pcdata->condition[COND_HUNGER]   == 0 )
	    gain /= 2;

	if ( ch->pcdata->condition[COND_THIRST] == 0 )
	    gain /= 2;

    }

    if ( !IS_SET(ch->in_room->area->flags, AREA_VIRTUAL) )
        gain = gain * ch->in_room->heal_rate / 100;
    
    if ( ch->on != NULL
    &&   ch->on->item_type == ITEM_FURNITURE
    &&   !IS_SET(ch->on->pIndexData->area->flags, AREA_VIRTUAL) )
	gain = gain * ch->on->value[3] / 100;

    if ( IS_AFFECTED(ch, AFF_POISON) )
	gain /= 4;

    if (IS_AFFECTED(ch, AFF_PLAGUE))
	gain /= 8;

    if (IS_AFFECTED(ch,AFF_HASTE) || IS_AFFECTED(ch,AFF_SLOW))
	gain /=2 ;

    return UMIN(gain, ch->max_hit - ch->hit);
}



int mana_gain( CHAR_DATA *ch )
{
    int gain;
    int number;

    if (ch->in_room == NULL)
	return 0;

    if ( IS_NPC(ch) )
    {
	gain = 5 + ch->level;
	switch (ch->position)
	{
	    default:		gain /= 2;		break;
	    case POS_SLEEPING:	gain = 3 * gain/2;	break;
   	    case POS_RESTING:				break;
	    case POS_FIGHTING:	gain /= 3;		break;
    	}
    }
    else
    {
	gain = (get_curr_stat(ch,STAT_WIS)/4 
	      + get_curr_stat(ch,STAT_INT)/4 + ch->level) / 2;
	number = number_percent();
	if (number < get_skill(ch,gsn_meditation))
	{
	    gain += number * gain / 100;
	    if (ch->mana < ch->max_mana)
	        check_improve(ch,gsn_meditation,TRUE,8,TRUE);
	}
	gain = (gain * class_table[ch->class].pMana) / 100;

	switch ( ch->position )
	{
	    default:		gain /= 4;			break;
	    case POS_SLEEPING: 					break;
	    case POS_RESTING:	gain /= 2;			break;
	    case POS_FIGHTING:	gain /= 6;			break;
	}

	if ( ch->pcdata->condition[COND_HUNGER]   == 0 )
	    gain /= 2;

	if ( ch->pcdata->condition[COND_THIRST] == 0 )
	    gain /= 2;

    }

    if ( !IS_SET(ch->in_room->area->flags, AREA_VIRTUAL) )
        gain = gain * ch->in_room->mana_rate / 100;

    if ( ch->on != NULL
    &&   ch->on->item_type == ITEM_FURNITURE
    &&   !IS_SET(ch->on->pIndexData->area->flags, AREA_VIRTUAL) )
	gain = gain * ch->on->value[4] / 100;

    if ( IS_AFFECTED( ch, AFF_POISON ) )
	gain /= 4;

    if (IS_AFFECTED(ch, AFF_PLAGUE))
        gain /= 8;

    if (IS_AFFECTED(ch,AFF_HASTE) || IS_AFFECTED(ch,AFF_SLOW))
        gain /=2 ;

    return UMIN(gain, ch->max_mana - ch->mana);
}



int move_gain( CHAR_DATA *ch )
{
    int gain;

    if (ch->in_room == NULL)
	return 0;

    if ( IS_NPC(ch) )
    {
	gain = ch->level;
    }
    else
    {
	gain = UMAX( 15, ch->level );

	switch ( ch->position )
	{
	case POS_SLEEPING: gain += get_curr_stat(ch,STAT_DEX)/4;		break;
	case POS_RESTING:  gain += get_curr_stat(ch,STAT_DEX)/8;	break;
	}

	if ( ch->pcdata->condition[COND_HUNGER]   == 0 )
	    gain /= 2;

	if ( ch->pcdata->condition[COND_THIRST] == 0 )
	    gain /= 2;
    }

    gain = gain * ch->in_room->heal_rate/100;

    if (ch->on != NULL && ch->on->item_type == ITEM_FURNITURE)
	gain = gain * ch->on->value[3] / 100;

    if ( IS_AFFECTED(ch, AFF_POISON) )
	gain /= 4;

    if (IS_AFFECTED(ch, AFF_PLAGUE))
        gain /= 8;

    if (IS_AFFECTED(ch,AFF_HASTE) || IS_AFFECTED(ch,AFF_SLOW))
        gain /=2 ;

    return UMIN(gain, ch->max_move - ch->move);
}



void gain_condition( CHAR_DATA *ch, int iCond, int value, bool show )
{
    bool already;
    int condition;

    if ( value == 0 || IS_NPC(ch) || ch->level >= LEVEL_IMMORTAL)
	return;

    already = ch->pcdata->condition[iCond] == 0;

    condition				= ch->pcdata->condition[iCond];
    if (condition == -1)
        return;
    ch->pcdata->condition[iCond]	= URANGE( 0, condition + value, 48 );

    if ( ch->pcdata->condition[iCond] == 0
    &&   (show || !already) )
    {
	switch ( iCond )
	{
	case COND_HUNGER:
	    send_to_char( "You are hungry.\n\r",  ch );
	    break;

	case COND_THIRST:
	    send_to_char( "You are thirsty.\n\r", ch );
	    break;

	case COND_DRUNK:
	    if ( condition != 0 )
		send_to_char( "You are sober.\n\r", ch );
	    break;
	}
    }

    return;
}



/*
 * Mob autonomous action.
 * This function takes 25% to 35% of ALL Merc cpu time.
 * -- Furey
 */
void mobile_update( void )
{
    CHAR_DATA *ch;
    CHAR_DATA *ch_next;
    EXIT_DATA *pexit;
    int door;

    /* Examine all mobs. */
    for ( ch = char_list; ch != NULL; ch = ch_next )
    {
	ch_next = ch->next;

	if ( !IS_NPC(ch)
	||   ch->in_room == NULL
	||   IS_AFFECTED(ch, AFF_CHARM)
	||   IS_AFFECTED(ch, AFF_PARALYSIS) )
	    continue;

	if (ch->in_room->area->empty && !IS_SET(ch->act,ACT_UPDATE_ALWAYS))
	    continue;

	/* Examine call for special procedure */
	if ( ch->spec_fun != 0 )
	{
	    if ( (*ch->spec_fun) ( ch ) )
		continue;
	}

	if (ch->pIndexData->pShop != NULL) /* give him some gold */
	{
	    if ((ch->gold * 100 + ch->silver) < ch->pIndexData->wealth)
	    {
		ch->gold += ch->pIndexData->wealth * number_range(1,20)/5000000;
		ch->silver += ch->pIndexData->wealth * number_range(1,20)/50000;
	    }
	}

	/*
	 * Check triggers only if mobile still in default position
	 */
	if ( ch->position == ch->pIndexData->default_pos )
	{
	    /* Delay */
	    if ( HAS_TRIGGER( ch, TRIG_DELAY) 
	    &&   ch->mprog_delay > 0 )
	    {
		if ( --ch->mprog_delay <= 0 )
		{
		    mp_percent_trigger( ch, NULL, NULL, NULL, TRIG_DELAY );
		    continue;
		}
	    } 
	    if ( HAS_TRIGGER( ch, TRIG_RANDOM) )
	    {
		if( mp_percent_trigger( ch, NULL, NULL, NULL, TRIG_RANDOM ) )
		continue;
	    }
	}

	/* That's all for sleeping / busy monster, and empty zones */
	if ( ch->position != POS_STANDING )
	    continue;

	/* Scavenge */
	if ( IS_SET(ch->act, ACT_SCAVENGER)
	&&   ch->in_room->contents != NULL
	&&   number_bits( 6 ) == 0 )
	{
	    OBJ_DATA *obj;
	    OBJ_DATA *obj_best;
	    int max;

	    max         = 1;
	    obj_best    = 0;
	    for ( obj = ch->in_room->contents; obj; obj = obj->next_content )
	    {
		if ( CAN_WEAR(obj, ITEM_TAKE) && can_loot(ch, obj)
		&&   obj->cost > max
		&&   obj->cost > 0 )
		{
		    if ( obj->item_type == ITEM_FURNITURE )
		    {
			bool found = FALSE;
			CHAR_DATA *gch;
			
			for ( gch = obj->in_room->people;
			      gch != NULL;
			      gch = gch->next_in_room )
			{
			    if ( gch->on == obj )
			    {
				found = TRUE;
				break;
			    }
			}
			
			if ( found )
			    continue;
		    }

		    obj_best    = obj;
		    max         = obj->cost;
		}
	    }

	    if ( obj_best )
	    {
		obj_from_room( obj_best );
		obj_to_char( obj_best, ch );
		act( "$n gets $p.", ch, obj_best, NULL, TO_ROOM );
	    }
	}

	/* Wander */
	if ( !IS_SET(ch->act, ACT_SENTINEL) 
	&& number_bits(3) == 0
	&& ( door = number_bits( 5 ) ) <= 5
	&& ( pexit = ch->in_room->exit[door] ) != NULL
	&&   pexit->u1.to_room != NULL
	&&   !IS_SET(pexit->exit_info, EX_CLOSED)
	&&   !IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB)
	&& ( !IS_SET(ch->act, ACT_STAY_AREA)
	||   pexit->u1.to_room->area == ch->in_room->area ) 
	&& ( !IS_SET(ch->act, ACT_OUTDOORS)
	||   !IS_SET(pexit->u1.to_room->room_flags,ROOM_INDOORS)) 
	&& ( !IS_SET(ch->act, ACT_INDOORS)
	||   IS_SET(pexit->u1.to_room->room_flags,ROOM_INDOORS)))
	{
	    move_char( ch, door, FALSE );
	}
	
	/* Hunt */
	if ( IS_SET(ch->off_flags, OFF_HUNTER)
	&&   ch->memory != NULL
	&&   number_bits(2) == 0 )
	    hunt( ch );
	if ( IS_SET(ch->off_flags, OFF_NASTY_HUNTER)
	&&   ch->memory != NULL
	&&   number_bits(1) == 0 )
	    hunt( ch );

	/* Animal call. */
	if ( IS_AFFECTED(ch, AFF_ANIMAL_CALL)
	&&   ch->memory != NULL
	&&   number_bits(1) == 0 )
	    answer_call( ch );

        /* Fire track. */
        if ( IS_AFFECTED2(ch, AFF2_FIRE_TRACK)
        &&   number_bits(2) )
            fire_track( ch );
    }

    return;
}


/*
 * Update the weather.
 */
void weather_update( bool fullUpdate )
{
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;
    bool thunder;
    int volume = 0;
    int diff;

    thunder = FALSE;

    buf[0] = '\0';

    if ( fullUpdate )
    {
        switch ( ++time_info.hour )
        {
        case  5:
	    weather_info.sunlight = SUN_LIGHT;
	    strcat( buf, "A false dawn lightens the sky with the approaching day.\n\r" );
	    break;

        case  6:
	    weather_info.sunlight = SUN_RISE;
	    strcat( buf, "The sun crests and rises from the eastern horizon.\n\r" );
	    break;

        case 19:
	    weather_info.sunlight = SUN_SET;
	    strcat( buf, "The sun slowly disappears below the western horizon.\n\r" );
	    break;

        case 20:
	    weather_info.sunlight = SUN_DARK;
	    strcat( buf, "...And darkness descends upon the lands.\n\r" );
	    break;

        case 24:
	    time_info.hour = 0;
	    time_info.day++;
	    time_info.lunar_day++;
	    break;
        }
    
        if ( time_info.hour == MOON_RISES
        &&	 (weather_info.sunlight == SUN_DARK
        ||	  weather_info.sunlight == SUN_SET) )
        {
	    switch ( LUNAR_PHASE )
	    {
	    case MOON_NEW:
	        break;
	    case MOON_WAXING_CRESENT:
	    case MOON_WANING_CRESENT:
	        if ( weather_info.sky == SKY_CLOUDLESS )
		    strcat( buf, "A thin cresent moon rises in the east.\n\r" );
	        break;
	    case MOON_WAXING_NEAR_FULL:
	        if ( weather_info.sky == SKY_CLOUDLESS )
		    strcat( buf, "A near full moon rises in the east.\n\r" );
	        break;
	    case MOON_WANING_NEAR_FULL:
	        if ( weather_info.sky == SKY_CLOUDLESS )
		    strcat( buf, "A waning moon rises in the east.\n\r" );
	        break;
	    case MOON_FULL:
	        if ( weather_info.sky == SKY_CLOUDLESS
	        ||	 weather_info.sky == SKY_CLOUDY )
		    strcat( buf, "A full moon rises in the east.\n\r" );
	        break;
	    }
        }
        if ( time_info.hour == MOON_SINKS
        &&	 (weather_info.sunlight == SUN_DARK
        ||	  weather_info.sunlight == SUN_SET) )
        {
	    switch ( LUNAR_PHASE )
	    {
	    case MOON_NEW:
	        break;
	    case MOON_WAXING_CRESENT:
	    case MOON_WANING_CRESENT:
	        if ( weather_info.sky == SKY_CLOUDLESS )
		    strcat( buf, "A thin cresent moon sinks below the western horizon.\n\r" );
	        break;
	    case MOON_WAXING_NEAR_FULL:
	        if ( weather_info.sky == SKY_CLOUDLESS )
		    strcat( buf, "A near full moon sinks below the western horizon.\n\r" );
	        break;
	    case MOON_WANING_NEAR_FULL:
	        if ( weather_info.sky == SKY_CLOUDLESS )
		    strcat( buf, "A waning moon sinks below the western horizon.\n\r" );
	        break;
	    case MOON_FULL:
	        if ( weather_info.sky == SKY_CLOUDLESS
	        ||	 weather_info.sky == SKY_CLOUDY )
		    strcat( buf, "A full moon sinks below the western horizon.\n\r" );
	        break;
	    }
        }

        if ( time_info.day   >= DAYS_IN_MONTH )
        {
	    time_info.day = 0;
	    time_info.month++;
        }
    
        if ( time_info.lunar_day >= DAYS_IN_LUNAR_MONTH )
	    time_info.lunar_day = 0;

        if ( time_info.month >= MONTHS_IN_YEAR )
        {
	    time_info.month = 0;
	    time_info.year++;
        }
    
        /*
         * Weather change.
         */
        if ( time_info.month >= 9 && time_info.month <= 16 )
	    diff = weather_info.mmhg >  985 ? -2 : 2;
        else
	    diff = weather_info.mmhg > 1015 ? -2 : 2;

        weather_info.change   += diff * dice(1, 4) + dice(2, 6) - dice(2, 6);
        weather_info.change    = UMAX(weather_info.change, -12);
        weather_info.change    = UMIN(weather_info.change,  12);

        weather_info.mmhg += weather_info.change;
        weather_info.mmhg  = UMAX(weather_info.mmhg,  960);
        weather_info.mmhg  = UMIN(weather_info.mmhg, 1040);
    }

    switch ( weather_info.sky )
    {
    default: 
	bug( "Weather_update: bad sky %d.", weather_info.sky );
	weather_info.sky = SKY_CLOUDLESS;
	break;

    case SKY_CLOUDLESS:
	if ( weather_info.mmhg <  990
	|| ( weather_info.mmhg < 1010 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "The sky is getting cloudy.\n\r" );
	    weather_info.sky = SKY_CLOUDY;
	}
	break;

    case SKY_CLOUDY:
	if ( weather_info.mmhg <  970
	|| ( weather_info.mmhg <  990 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "It starts to rain.\n\r" );
	    weather_info.sky = SKY_RAINING;
	}

	if ( weather_info.mmhg > 1030 && number_bits( 2 ) == 0 )
	{
	    strcat( buf, "The clouds disappear.\n\r" );
	    weather_info.sky = SKY_CLOUDLESS;
	}
	break;

    case SKY_RAINING:
               
	if ( weather_info.mmhg <  970 && number_bits( 2 ) == 0 )
	{
	    strcat( buf, "Lightning flashes in the sky.\n\r" );
	    weather_info.sky = SKY_LIGHTNING;
            volume  = number_range( 40, 80 );
            thunder = TRUE;
	}

	if ( weather_info.mmhg > 1030
	|| ( weather_info.mmhg > 1010 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "The rain stopped.\n\r" );
	    weather_info.sky = SKY_CLOUDY;
	}
	break;

    case SKY_LIGHTNING:
	if ( weather_info.mmhg > 1010
	|| ( weather_info.mmhg >  990 && number_bits( 2 ) == 0 ) )
	{
	    strcat( buf, "The lightning has stopped.\n\r" );
	    weather_info.sky = SKY_RAINING;
	    break;
	}
        volume  = number_range( 70, 100 );
        thunder = TRUE;
	break;
    }

    if ( buf[0] != '\0' || thunder )
    {
        char sound_buf[MAX_STRING_LENGTH];

        sprintf( sound_buf, "thunder* V=%d P=100 T=weather", volume );

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
	    if ( d->connected == CON_PLAYING
	    &&   IS_OUTSIDE(d->character)
	    &&   IS_AWAKE(d->character) )
            {
                CHAR_DATA *ch = CH( d );

                if ( buf[0] != '\0' && fullUpdate )
		    send_to_char( buf, ch );

                if ( thunder )
                    sound_to_char( sound_buf, ch );
            }
	}
    }

    return;
}



/*
 * Update all chars, including mobs.
*/
void char_update( void )
{   
    CHAR_DATA *ch;
    CHAR_DATA *ch_next;
    CHAR_DATA *ch_quit;

    ch_quit	= NULL;

    /* update save counter */
    save_number++;

    if (save_number > 29)
	save_number = 0;

    for ( ch = char_list; ch != NULL; ch = ch_next )
    {
	AFFECT_DATA *paf;
	AFFECT_DATA *paf_next;
	MEM_DATA *pmem;
	MEM_DATA *pmem_next;

	ch_next = ch->next;

	if ( ch->timer > 30 )
	    ch_quit = ch;
            
	/* Remove any old memories. */
        crash_set_last_char( ch->name );
        crash_set_last_cmd( "UpdateChar (remove memories)" );
	for ( pmem = ch->memory; pmem != NULL; pmem = pmem->next )
	{
	    pmem_next = pmem->next;
	    
	    if ( pmem->duration >= 0 )
	    {
		pmem->duration--;
		
		if ( pmem->duration < 0 )
		    remove_memory( ch, pmem );
	    }
	}

	if ( ch->position >= POS_STUNNED )
	{
	    /* check to see if we need to go home */
            crash_set_last_cmd( "UpdateChar (wander home)" );
	    if ( IS_NPC(ch)
	    &&   ch->zone != NULL && ch->zone != ch->in_room->area
	    &&   ch->desc == NULL
	    &&   ch->fighting == NULL
	    &&   !IS_AFFECTED(ch,AFF_PARALYSIS)
	    &&   !IS_AFFECTED(ch,AFF_CHARM)
	    &&   number_percent() < 5 )
	    {
		act("$n wanders on home.",ch,NULL,NULL,TO_ROOM);
		extract_char(ch,TRUE);
		continue;
	    }

            crash_set_last_cmd( "UpdateChar (update hmv)" );
	    if ( !IS_AFFECTED(ch, AFF_PARALYSIS) )
	    {
		if ( ch->hit  < ch->max_hit )
		    ch->hit  += hit_gain(ch);
		else
		    ch->hit = ch->max_hit;
                if ( has_customization(ch, DISADVANTAGE_PERMANENT_WOUND) )
                    ch->hit = 9 * ch->hit / 10;

		if ( ch->mana < ch->max_mana )
		    ch->mana += mana_gain(ch);
		else
		    ch->mana = ch->max_mana;

		if ( ch->move < ch->max_move )
		    ch->move += move_gain(ch);
		else
		    ch->move = ch->max_move;
	    }
	}

        if ( ch->position == POS_STUNNED
	&&   !IS_AFFECTED(ch, AFF_PARALYSIS) )
        {
            crash_set_last_cmd( "UpdateChar (update position)" );
	    update_pos( ch );
        }

	if ( !IS_AFFECTED(ch, AFF_PARALYSIS)
	&&   IS_NPC(ch)
	&&   IS_SET(ch->act, ACT_CRIER) )
        {
            crash_set_last_cmd( "UpdateChar (cry)" );
	    cry( ch );
        }

	if ( !IS_NPC(ch) && ch->level < LEVEL_IMMORTAL )
	{
            CUST_DATA *cust;
	    OBJ_DATA *obj;

            crash_set_last_cmd( "UpdateChar (handle light)" );

	    if ( ( obj = get_eq_char( ch, WEAR_HOLD /*WEAR_LIGHT*/ ) ) != NULL
	    &&   obj->item_type == ITEM_LIGHT
	    &&   obj->value[2] > 0 )
	    {
		if ( --obj->value[2] == 0 && ch->in_room != NULL )
		{
		    --ch->in_room->light;
		    act( "$p goes out.", ch, obj, NULL, TO_ROOM );
		    act( "$p flickers and goes out.", ch, obj, NULL, TO_CHAR );
		    extract_obj( obj );
		}
		else if ( obj->value[2] <= 5 && ch->in_room != NULL)
		    act("$p flickers.",ch,obj,NULL,TO_CHAR);
	    }

	    if ( IS_IMMORTAL(ch) )
		ch->timer = 0;

	    if ( ++ch->timer >= 12 )
	    {
                crash_set_last_cmd( "UpdateChar (shift to void)" );

		if ( ch->was_in_room == NULL && ch->in_room != NULL )
		{
		    ch->was_in_room = ch->in_room;
		    if ( ch->fighting != NULL )
			stop_fighting( ch, TRUE );
		    act( "$n disappears into the void.",
			ch, NULL, NULL, TO_ROOM );
		    send_to_char( "You disappear into the void.\n\r", ch );
		    if (ch->level > 1)
		        save_char_obj( ch, FALSE );
		    char_from_room( ch, TRUE );
		    char_to_room( ch, get_room_index( ROOM_VNUM_LIMBO ) );
		}
	    }

	    if ( !IS_AFFECTED(ch, AFF_PARALYSIS)
	    &&   (ch->in_room == NULL
	    ||    !IS_SET(ch->in_room->area->flags, AREA_VIRTUAL)) )
	    {
                crash_set_last_cmd( "UpdateChar (adjust conditions)" );
		gain_condition( ch, COND_DRUNK, -1, TRUE );
		gain_condition( ch, COND_FULL, 
		    CHAR_SIZE(ch) > SIZE_MEDIUM ? -4 : -2, TRUE );
		gain_condition( ch, COND_THIRST, -1, TRUE );
		gain_condition( ch, COND_HUNGER,
		    CHAR_SIZE(ch) > SIZE_MEDIUM ? -2 : -1, TRUE );
	    }

            crash_set_last_cmd( "UpdateChar (process customizations)" );
            for ( cust = ch->pcdata->customizations;
                  cust != NULL;
                  cust = cust->next )
            {
                if ( cust->affect.where == -1 )
                    continue;
                if ( IS_SET(cust->affect.flags, AFF_CONT_UPDATE) )
                {
                    affect_procs[cust->affect.procNo].fn( ch, NULL, &(cust->affect), AFF_PROC_PERIODIC );
                    if ( CHAR_KILLED(ch) )
                        break;
                }
            }
            if ( CHAR_KILLED(ch) )
            {
                crash_set_last_char( "" );
                crash_set_last_cmd( "" );
                continue; /* Skip onto next character. */
            }
	}

        crash_set_last_cmd( "UpdateChar (process affects)" );
	for ( paf = ch->affected; paf != NULL; paf = paf_next )
	{
	    paf_next	= paf->next;
	    
	    if ( paf->duration > 0 )
	    {
		paf->duration--;
		if (number_range(0,4) == 0 && paf->level > 0)
		    paf->level--;  /* spell strength fades with time */

		if ( IS_SET(paf->flags, AFF_CONT_UPDATE) )
		{
		    affect_procs[paf->procNo].fn( ch, NULL, paf, AFF_PROC_PERIODIC );
		    if ( CHAR_KILLED(ch) )
			break;
		}
	    }
	    else if ( paf->duration < 0 )
	    {
		if ( IS_SET(paf->flags, AFF_CONT_UPDATE) )
		{
		    affect_procs[paf->procNo].fn( ch, NULL, paf, AFF_PROC_PERIODIC );
		    if ( CHAR_KILLED(ch) )
			break;
		}
	    }
	    else
	    {
		if ( paf_next == NULL
		||   paf_next->type != paf->type
		||   paf_next->duration > 0 )
		{
		    if ( paf->type > 0
		    &&   skill_table[paf->type].msg_off[0] != '\0' )
		    {
			send_to_char( skill_table[paf->type].msg_off, ch );
			send_to_char( "\n\r", ch );
		    }
		}

		affect_procs[paf->procNo].fn( ch, NULL, paf, AFF_PROC_FINISHED );
		if ( CHAR_KILLED(ch) )
		    break;

		affect_remove( ch, paf );
	    }
	}

	/* If the affect processing killed ch we skip this char. */
	if ( CHAR_KILLED(ch) ) {
            crash_set_last_char( "" );
            crash_set_last_cmd( "" );
	    continue;
	}

	if ( ch->position == POS_INCAP 
	&&   number_range(0,1) == 0
	&&   !IS_AFFECTED(ch, AFF_PARALYSIS) )
	{
            crash_set_last_cmd( "UpdateChar (incap damage)" );
	    damage( ch, ch, 1, TYPE_UNDEFINED, DAM_NONE, FALSE, NULL );
	}
	else if ( ch->position == POS_MORTAL
	&&   !IS_AFFECTED(ch, AFF_PARALYSIS) )
	{
            crash_set_last_cmd( "UpdateChar (mortal damage)" );
	    damage( ch, ch, 1, TYPE_UNDEFINED, DAM_NONE, FALSE, NULL );
	}
    }

    /*
     * Autosave and autoquit.
     * Check that these chars still exist.
     */
    crash_set_last_cmd( "UpdateChar (autosaving)" );
    for ( ch = char_list; ch != NULL; ch = ch_next )
    {
        ch_next = ch->next;

	if (ch->desc != NULL && ch->desc->descriptor % 30 == save_number)
        {
            crash_set_last_cmd( "UpdateChar (saving character)" );
	    save_char_obj(ch, FALSE);
        }

        if ( ch == ch_quit )
            do_quit( ch, "" );
    }

    tail_chain( );
    return;
}




/*
 * Update all objs.
 * This function is performance sensitive.
 */
void obj_update( void )
{   
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;
    AFFECT_DATA *paf, *paf_next;

    for ( obj = object_list; obj != NULL; obj = obj_next )
    {
	CHAR_DATA *rch;
	char *message;

	obj_next = obj->next;
	
	/* Apply any harmful affects to the wearer. */
	if ( obj->wear_loc != WEAR_NONE )
	{
	    if ( IS_SET(obj->pIndexData->restrictions, RESTRICT_WASTING) )
		gain_condition( obj->carried_by, COND_HUNGER, -10, FALSE );
	    if ( IS_SET(obj->pIndexData->restrictions, RESTRICT_CORRUPTING) )
		obj->carried_by->alignment =
		    UMAX( -1000, obj->carried_by->alignment - obj->level / 4 );
	}

	/* go through affects and decrement */
	for ( paf = obj->affected; paf != NULL; paf = paf_next )
	{
	    paf_next    = paf->next;
            
	    if ( paf->duration > 0 )
	    {
		paf->duration--;
		if (number_range(0,4) == 0 && paf->level > 0)
		    paf->level--;  /* spell strength fades with time */
					
		if ( IS_SET(paf->flags, AFF_CONT_UPDATE) )
		{
		    affect_procs[paf->procNo].fn( NULL, obj, paf, AFF_PROC_PERIODIC );
		    if ( OBJ_EXTRACTED(obj) )
			break;
		}
	    }
	    else if ( paf->duration < 0 )
		;
	    else
	    {	
		if ( paf_next == NULL
		||   paf_next->type != paf->type
		||   paf_next->duration > 0 )
		{
		    if ( paf->type > 0
		    &&   skill_table[paf->type].msg_obj[0] != '\0' )
		    {
			if (obj->carried_by != NULL)
			{
			    rch = obj->carried_by;
			    act( skill_table[paf->type].msg_obj,
				rch, obj, NULL, TO_CHAR );
			}
				
			if ( obj->in_room != NULL 
			&&   obj->in_room->people != NULL )
			{
			    rch = obj->in_room->people;
			    act( skill_table[paf->type].msg_obj,
				rch, obj, NULL, TO_ALL );
			}
		    }
		}

		affect_procs[paf->procNo].fn( NULL, obj, paf, AFF_PROC_FINISHED );
		if ( OBJ_EXTRACTED(obj) )
		    break;

		affect_remove_obj( obj, paf );
	    }
	}
	
	/* If the affect processing toasted the object skip it. */
	if ( OBJ_EXTRACTED(obj) ) {
	    continue;
	}

	if ( obj->timer <= 0 || --obj->timer > 0 )
	    continue;

	switch ( obj->item_type )
	{
	    default:              message = "$p crumbles into dust.";	break;
	    case ITEM_FOUNTAIN:   message = "$p dries up.";		break;
	    case ITEM_CORPSE_NPC: message = "$p decays into dust.";	break;
	    case ITEM_CORPSE_PC:  message = "$p decays into dust.";	break;
	    case ITEM_FOOD:       message = "$p decomposes.";		break;
	    case ITEM_POTION:     message = "$p has evaporated from disuse.";	
									break;
	    case ITEM_PORTAL:     message = "$p fades out of existence.";
									break;
	    case ITEM_CONTAINER: 
		if (CAN_WEAR(obj,ITEM_WEAR_FLOAT))
		    if (obj->contains)
			message = 
			    "$p flickers and vanishes, spilling its contents on the floor.";
		    else
			message = "$p flickers and vanishes.";
		else
		    message = "$p crumbles into dust.";
		break;
	}

	if ( obj->carried_by != NULL )
	{
	    if (IS_NPC(obj->carried_by) 
	    &&  obj->carried_by->pIndexData->pShop != NULL)
		obj->carried_by->silver += obj->cost/5;
	    else
	    {
		act( message, obj->carried_by, obj, NULL, TO_CHAR );
		if ( obj->wear_loc == WEAR_FLOAT)
		    act(message,obj->carried_by,obj,NULL,TO_ROOM);
	    }
	}
	else if ( obj->in_room != NULL
	&&      ( rch = obj->in_room->people ) != NULL )
	{
	    if ( !(obj->in_obj && obj->in_obj->pIndexData->vnum == OBJ_VNUM_PIT
	    &&   !CAN_WEAR(obj->in_obj,ITEM_TAKE)) )
	    {
		act( message, rch, obj, NULL, TO_ROOM );
		act( message, rch, obj, NULL, TO_CHAR );
	    }
	}

        if ( (obj->item_type == ITEM_CORPSE_PC || obj->wear_loc == WEAR_FLOAT)
	&&   obj->contains )
	{   /* save the contents */
     	    OBJ_DATA *t_obj, *next_obj;

	    for (t_obj = obj->contains; t_obj != NULL; t_obj = next_obj)
	    {
		next_obj = t_obj->next_content;
		obj_from_obj(t_obj);

		if (obj->in_obj) /* in another object */
		    obj_to_obj(t_obj,obj->in_obj);

		else if (obj->carried_by)  /* carried */
		    if (obj->wear_loc == WEAR_FLOAT)
			if (obj->carried_by->in_room == NULL)
			    extract_obj(t_obj);
			else
			    obj_to_room(t_obj,obj->carried_by->in_room);
		    else
		    	obj_to_char(t_obj,obj->carried_by);

		else if (obj->in_room == NULL)  /* destroy it */
		    extract_obj(t_obj);

		else /* to a room */
		    obj_to_room(t_obj,obj->in_room);
	    }
	}

	extract_obj( obj );
    }

    return;
}



/*
 * Aggress.
 *
 * for each mortal PC
 *     for each mob in room
 *         aggress on some random PC
 *
 * This function takes 25% to 35% of ALL Merc cpu time.
 * Unfortunately, checking on each PC move is too tricky,
 *   because we don't the mob to just attack the first PC
 *   who leads the party into the room.
 *
 * -- Furey
 */
void aggr_update( void )
{
    CHAR_DATA *wch;
    CHAR_DATA *wch_next;
    CHAR_DATA *ch;
    CHAR_DATA *ch_next;
    CHAR_DATA *vch;
    CHAR_DATA *vch_next;
    CHAR_DATA *victim;

    for ( wch = char_list; wch != NULL; wch = wch_next )
    {
	wch_next = wch->next;
	if ( IS_NPC(wch)
	||   wch->level >= LEVEL_IMMORTAL
	||   wch->in_room == NULL 
	||   wch->in_room->area->empty
	||   IS_AFFECTED(wch, AFF_PARALYSIS) )
	    continue;

	for ( ch = wch->in_room->people; ch != NULL; ch = ch_next )
	{
	    int count;

	    ch_next	= ch->next_in_room;

	    if ( !IS_NPC(ch)
	    ||   IS_AFFECTED(ch, AFF_PARALYSIS)
	    ||   !IS_SET(ch->act, ACT_AGGRESSIVE)
	    ||   IS_SET(ch->in_room->room_flags,ROOM_SAFE)
	    ||   IS_AFFECTED(ch,AFF_CALM)
	    ||   ch->fighting != NULL
	    ||   IS_AFFECTED(ch, AFF_CHARM)
	    ||   !IS_AWAKE(ch)
	    ||   ( IS_SET(ch->act, ACT_WIMPY) && IS_AWAKE(wch) )
	    ||   !can_see( ch, wch ) 
	    ||   number_bits(1) == 0)
		continue;

	    /*
	     * Ok we have a 'wch' player character and a 'ch' npc aggressor.
	     * Now make the aggressor fight a RANDOM pc victim in the room,
	     *   giving each 'vch' an equal chance of selection.
	     */
	    count	= 0;
	    victim	= NULL;
	    for ( vch = wch->in_room->people; vch != NULL; vch = vch_next )
	    {
		vch_next = vch->next_in_room;

		if ( !IS_NPC(vch)
		&&   will_agress(ch, vch)
		&&   can_see(ch, vch)
		&&   vch->level < LEVEL_IMMORTAL
		&&   (!IS_SET(ch->act, ACT_WIMPY)
		||    !IS_AWAKE(vch)) )
		{
		    if ( number_range( 0, count ) == 0 )
			victim = vch;
		    count++;
		}
	    }

	    if ( victim == NULL )
		continue;

/*
 *	    if ( IS_SET(ch->off_flags, OFF_BACKSTAB) )
 *	    {
 *		do_backstab( ch, victim->name );
 *		if ( !CHAR_KILLED(victim)
 *		&&   ch->fighting != victim )
 *		    multi_hit( ch, victim, TYPE_UNDEFINED );
 *	    }
 *	    else
 */
		multi_hit( ch, victim, TYPE_UNDEFINED );
	}
    }

    return;
}


/*
 * Process all object periodic polls (handles rand object programs).
 */
void obj_periodic()
{
    PERIODIC_CHECK_DATA *pcd, *pcd_next;
    
    for ( pcd = periodic_object_checks; pcd != NULL; pcd = pcd_next )
    {
	pcd_next = pcd->next;
	
	op_percent_trigger( (OBJ_DATA *) pcd->data,
	                    NULL, NULL, NULL, OTRIG_RANDOM );
    }
}


/*
 * Process all room periodic polls (handles rand room programs).
 */
void room_periodic()
{
    PERIODIC_CHECK_DATA *pcd;

    for ( pcd = periodic_room_checks; pcd != NULL; pcd = pcd->next )
    {
	rp_percent_trigger( (ROOM_INDEX_DATA *) pcd->data,
	                    NULL, NULL, NULL, RTRIG_RANDOM );
    }
}





/*
 * Handle all kinds of updates.
 * Called once per pulse from game loop.
 * Random times to defeat tick-timing clients and players.
 */

void update_handler( void )
{
    static  int     pulse_area;
    static  int     pulse_mobile;
    static  int     pulse_violence;
    static  int     pulse_point;
    static  int	    pulse_music;
    static  int     pulse_rcheck;

    while ( extracted_chars != NULL )
    {
	CHAR_DATA *tail = extracted_chars->next;
	free_char( extracted_chars );
	extracted_chars = tail;
    }

    while ( extracted_objs != NULL )
    {
	OBJ_DATA *tail = extracted_objs->next;
	free_obj( extracted_objs );
	extracted_objs = tail;
    }

    if ( --pulse_area     <= 0 )
    {
	pulse_area	= PULSE_AREA;
	/* number_range( PULSE_AREA / 2, 3 * PULSE_AREA / 2 ); */
	area_update	( );
    }

    if ( --pulse_music	  <= 0 )
    {
	pulse_music	= PULSE_MUSIC;
	song_update();
    }

    if ( --pulse_mobile   <= 0 )
    {
	pulse_mobile	= PULSE_MOBILE;
	mobile_update	( );
	obj_periodic	( );
	room_periodic	( );
    }

    if ( --pulse_violence <= 0 )
    {
	pulse_violence	= PULSE_VIOLENCE;
	violence_update	( );
    }

    if ( --pulse_point    <= 0 )
    {
	static int ticks = 0;
        bool fullUpdate = ticks % TICKS_IN_HOUR;
	
	wiznet( "TICK!", NULL, NULL, WIZ_TICKS, 0, 0 );
	pulse_point     = PULSE_TICK;
        weather_update( fullUpdate );
	if ( fullUpdate )
	{
	    update_crier_messages();
	    save_accounts();
	}

	char_update	( );
	obj_update	( );

        ticks++;
    }

    if ( --pulse_rcheck <= 0 )
    {
	pulse_rcheck	= PULSE_RCHECK;
	check_reboot	( );
    }

    aggr_update( );
    tail_chain( );
    return;
}


bool will_agress( CHAR_DATA *ch, CHAR_DATA *victim )
{
    int level_mod = 0;

    if ( is_creator(victim, ch->pIndexData->area)
    ||   (IS_SET(CHAR_FORM(ch), FORM_UNDEAD)
    &&    IS_AFFECTED2(victim, AFF2_CORPSE_FORM)) )
	return FALSE;

    if ( !IS_NPC(victim) )
    {
        if ( victim->pcdata->renown >= 100 )
            level_mod = 6;
        else if ( victim->pcdata->renown >= 90 )
            level_mod = 3;
        else if ( victim->pcdata->renown >= 60 )
            level_mod = 1;
        else if ( victim->pcdata->renown <= 20 )
            level_mod = -5;
    }

    if ( IS_AFFECTED2(victim, AFF2_ALKAR)
    &&   !IS_AFFECTED2(ch, AFF2_DETECT_ILLUSION) )
    {
	AFFECT_DATA *paf = affect_find( victim->affected, gsn_alkar );

	if ( paf == NULL )
	    bug( "will_agress: NULL paf.", 0 );
	else
	    level_mod += paf->modifier;
    }

    if ( IS_AFFECTED2(victim, AFF2_CORPSE_FORM) )
    {
	AFFECT_DATA *paf = affect_find( victim->affected, gsn_corpse_form );

	if ( paf == NULL )
	    bug( "will_agress: NULL paf.", 0 );
	else
	    level_mod += paf->modifier;
    }

    if ( has_customization(ch, ADVANTAGE_BLAND) ||
         has_customization(ch, ADVANTAGE_MEEK) )
        level_mod += 5;

    if ( ch->pIndexData->aggression == 0 )
	return ch->level >= victim->level + level_mod - 5;

    if ( IS_SET(ch->pIndexData->aggression, AGG_EVERYONE)
    ||   char_matches(victim, ch->pIndexData->aggression) )
    {	
	if ( IS_SET(ch->pIndexData->aggression, AGG_CHICKEN)
	&&   ch->level >= victim->level + level_mod )
	    return TRUE;
	
	if ( IS_SET(ch->pIndexData->aggression, AGG_PSYCHO)
	&&   ch->level <= victim->level + level_mod )
	    return TRUE;
	
	if ( !IS_SET(ch->pIndexData->aggression, AGG_PSYCHO|AGG_CHICKEN)
	&&   ch->level >= victim->level + level_mod )
	    return TRUE;
    }
    
    return FALSE;
}

void check_reboot( void )
{
    extern time_t reboot_time;
    extern bool reboot_set;
    int ttr;
    extern bool merc_down;
    DESCRIPTOR_DATA *d,*d_next;
    CHAR_DATA *vch;

    if ( !reboot_set )
	return;

    ttr = reboot_time - current_time;

    if (ttr == 600)
	for ( d = descriptor_list; d; d = d->next )
	    if ( d->connected == CON_PLAYING )
		send_to_char( "Scheduled reboot in 10 minutes.\n\r", d->character );

    if (ttr == 300)
	for ( d = descriptor_list; d; d = d->next )
	    if ( d->connected == CON_PLAYING )
		send_to_char( "Scheduled reboot in 5 minutes.\n\r", d->character );

    if (ttr == 60)
	for ( d = descriptor_list; d; d = d->next )
	    if ( d->connected == CON_PLAYING )
		send_to_char( "Scheduled reboot in 1 minute.\n\r", d->character );

    if (ttr < 1) {
	for ( d = descriptor_list; d; d = d->next )
	    if ( d->connected == CON_PLAYING )
		send_to_char( "Scheduled reboot.\n\r", d->character );

	log_string( "Scheduled reboot." );

        merc_down = TRUE;
        for ( d = descriptor_list; d != NULL; d = d_next )
        {
            d_next = d->next;
            vch = d->original ? d->original : d->character;
            if (vch != NULL)
                save_char_obj(vch, FALSE);
            close_socket(d, TRUE);
        }

        save_accounts();

        return;
    }
    return;
}